@@ -275,12 +275,13 @@ extern int errno;
 #define true  1
 #define false 0
 #define CF_BUFSIZE 4096
-#define CF_EXPANDSIZE 2*CF_BUFSIZE
-#define CF_ALLCLASSSIZE 4*CF_BUFSIZE
+#define CF_EXPANDSIZE (2*CF_BUFSIZE)
+#define CF_ALLCLASSSIZE (4*CF_BUFSIZE)
 #define CF_BUFFERMARGIN 32
 #define CF_BLOWFISHSIZE 16
+#define CF_SMALLBUF 128
 #define CF_MAXVARSIZE 1024
-#define CF_NONCELEN CF_BUFSIZE/16
+#define CF_NONCELEN (CF_BUFSIZE/16)
 #define CF_MAXLINKSIZE 256
 #define CF_MAXLINKLEVEL 4
 #define CF_MAXARGS 31
@@ -291,19 +292,27 @@ extern int errno;
 #define CF_MACROALPHABET 61    /* a-z, A-Z plus a bit */
 #define CF_MAXSHELLARGS 30
 #define CF_SAMEMODE 0
-#define CF_SAME_OWNER (uid_t)-1
-#define CF_UNKNOWN_OWNER (uid_t)-2
-#define CF_SAME_GROUP (gid_t)-1
-#define CF_UNKNOWN_GROUP (gid_t)-2
+#define CF_SAME_OWNER ((uid_t)-1)
+#define CF_UNKNOWN_OWNER ((uid_t)-2)
+#define CF_SAME_GROUP ((gid_t)-1)
+#define CF_UNKNOWN_GROUP ((gid_t)-2)
 #define CF_NOSIZE    -1
 #define CF_EXTRASPC 8      /* pads items during AppendItem for eol handling in editfiles */
-#define CF_INFINITY (int)999999999
+#define CF_INFINITY ((int)999999999)
 #define CF_TICKS_PER_DAY 86400 /* 60 * 60 *24 */
+#define CF_TICKS_PER_HOUR 3600 /* 60 * 60 */
+#define CF_HALF_HOUR 1800      /* 60 * 30 */ 
 #define CF_NOT_CONNECTED -1
 #define CF_RECURSION_LIMIT 100
 #define CF_MONDAY_MORNING 342000
 #define CF_NOVAL -0.7259285297502359
 
+#define CF_MAXDIGESTNAMELEN 7
+#define CF_CHKSUMKEYOFFSET  CF_MAXDIGESTNAMELEN+1
+
+#define CF_IFREQ 2048    /* Reportedly the largest size that does not segfault 32/64 bit*/
+#define CF_ADDRSIZE 128
+
 #define CF_METHODEXEC 0
 #define CF_METHODREPLY  1
 #define CF_EXEC_IFELAPSED 5
@@ -336,15 +345,19 @@ extern int errno;
 
 /*******************************************************************/
 
-#define CF_AVDB_FILE     "cf_learning.db"
+#define CF_AVDB_FILE     "cf_observations.db"
+#define CF_OLDAVDB_FILE  "cf_learning.db"
 #define CF_STATEDB_FILE  "cf_state.db"
-#define CF_LASTDB_FILE   "cf_lastseen.db"
+#define CF_OLDLASTDB_FILE "cf_lastseen.db"
+#define CF_LASTDB_FILE   "cf_LastSeen.db"
 #define CF_STATELOG_FILE "state_log"
-#define CF_ENVNEW_FILE  "env_data.new"
+#define CF_ENVNEW_FILE   "env_data.new"
 #define CF_ENV_FILE      "env_data"
 #define CF_TCPDUMP_COMM "/usr/sbin/tcpdump -t -n -v"
 
 
+#define CF_CHKDB "checksum_digests.db"
+
 #define CF_INPUTSVAR "CFINPUTS"          /* default name for file path var */
 #define CF_ALLCLASSESVAR "CFALLCLASSES"  /* default name for CFALLCLASSES env */
 #define CF_INF_RECURSE -99             /* code used to signify inf in recursion */
@@ -365,8 +378,15 @@ extern int errno;
 #define CF_PROTO_OFFSET 16
 #define CF_INBAND_OFFSET 8
 #define CF_SMALL_OFFSET 2
+
+/* digest sizes */
 #define CF_MD5_LEN 16
 #define CF_SHA_LEN 20
+#define CF_SHA1_LEN 20
+#define CF_SHA224_LEN 28
+#define CF_SHA256_LEN 32
+#define CF_SHA384_LEN 48
+#define CF_SHA512_LEN 64
 
 #define CF_DONE 't'
 #define CF_MORE 'm'
@@ -410,7 +430,21 @@ extern int errno;
 #define CF_WEEK   (7.0*24.0*3600.0)
 #define CF_MEASURE_INTERVAL (5.0*60.0)
 
+#define CF_OBSERVABLES 91
+
+struct QPoint
+   {
+   double q;
+   double expect;
+   double var;
+   };
+
 struct Averages
+   {
+   struct QPoint Q[CF_OBSERVABLES];
+   };
+
+struct OldAverages /* For conversion to new db */
    {
    double expect_number_of_users;
    double expect_rootprocs;
@@ -419,8 +453,7 @@ struct Averages
    double expect_loadavg;
    double expect_incoming[ATTR];
    double expect_outgoing[ATTR];
-   double expect_pH[PH_LIMIT];
-      
+   double expect_pH[PH_LIMIT];      
    double var_number_of_users;
    double var_rootprocs;
    double var_otherprocs;
@@ -429,23 +462,13 @@ struct Averages
    double var_incoming[ATTR];
    double var_outgoing[ATTR];
    double var_pH[PH_LIMIT];
-
-      /* tcpdump data - tag them on here ... */
-      
    double expect_netin[CF_NETATTR];
    double expect_netout[CF_NETATTR];
    double var_netin[CF_NETATTR];
    double var_netout[CF_NETATTR];
    };
 
-
-struct LastSeen
-   {
-   double expect_lastseen;
-   time_t lastseen;      
-   };
-
-
+/******************************************************************/
 
 struct LockData
    {
@@ -473,6 +496,7 @@ struct LockData
    (including indirect blocks). */
 
 
+#define SMALL_BLOCK_BUF_SIZE 512
 
 #ifndef HAVE_ST_BLOCKS
 # define ST_BLKSIZE(statbuf) DEV_BSIZE
@@ -523,6 +547,8 @@ enum PROTOS
    cfd_sauth,
    cfd_ssynch,
    cfd_sget,
+   cfd_version,
+   cfd_sopendir,
    cfd_bad
    };
 
@@ -624,7 +650,9 @@ enum builtin
    fn_islink,
    fn_isplain,
    fn_execresult,
+   fn_execshellresult,
    fn_returnszero,
+   fn_returnszeroshell,
    fn_iprange,
    fn_hostrange,
    fn_isdefined,
@@ -921,6 +949,7 @@ enum vnames
    cfdefcopy,
    cfredef,
    cfdefpkgmgr,
+   cfabortclasses,
    nonexistentvar
    };
 
@@ -1026,6 +1055,7 @@ enum editnames
    AppendIfNoSuchLinesFromFile,
    IncrementPointer,
    ReplaceLineWith,
+   ExpandVariables,
    DeleteToLineMatching,
    HashCommentToLineMatching,
    PercentCommentToLineMatching,
@@ -1040,8 +1070,10 @@ enum editnames
    GotoLastLine,
    BreakIfLineMatches,
    BeginGroupIfNoMatch,
+   BeginGroupIfMatch,
    BeginGroupIfNoLineMatching,
    BeginGroupIfNoSuchLine,
+   BeginGroupIfLineMatching,
    EndGroup,
    Append,
    Prepend,
@@ -1067,6 +1099,7 @@ enum editnames
    BeginGroupIfFileIsNewer,
    BeginGroupIfFileExists,
    BeginGroupIfNoLineContaining,
+   BeginGroupIfLineContaining,
    BeginGroupIfDefined,
    BeginGroupIfNotDefined,
    AutoCreate,
@@ -1126,7 +1159,7 @@ enum SignalNames
    cfsys,
    cfpipe,
    cfalrm,
-   cfterm,
+   cfterm
    };
 
 #define highest_signal 64
@@ -1140,7 +1173,8 @@ enum cfoutputlevel
    cfverbose,
    cfeditverbose,
    cferror,
-   cflogonly
+   cflogonly,
+   cfloginform
    };
 
 /*******************************************************************/
@@ -1168,7 +1202,8 @@ enum cmpsense   /* For package version comparison */
    cmpsense_lt,
    cmpsense_ge,
    cmpsense_le,
-   cmpsense_ne
+   cmpsense_ne,
+   cmpsense_none
    };
 
 /*******************************************************************/
@@ -1178,6 +1213,8 @@ enum pkgmgrs    /* Available package managers to query in packages: */
    pkgmgr_rpm,
    pkgmgr_dpkg,
    pkgmgr_sun,
+   pkgmgr_aix,
+   pkgmgr_portage,
    pkgmgr_none
    };
 
@@ -1220,12 +1257,69 @@ enum iptypes
    tcpmisc
    };
 
+enum observables
+    {
+    ob_users,
+    ob_rootprocs,
+    ob_otherprocs,
+    ob_diskfree,
+    ob_loadavg,
+    ob_netbiosns_in,
+    ob_netbiosns_out,
+    ob_netbiosdgm_in,
+    ob_netbiosdgm_out,
+    ob_netbiosssn_in,
+    ob_netbiosssn_out,
+    ob_irc_in,
+    ob_irc_out,
+    ob_cfengine_in,
+    ob_cfengine_out,
+    ob_nfsd_in,
+    ob_nfsd_out,
+    ob_smtp_in,
+    ob_smtp_out,
+    ob_www_in,
+    ob_www_out,
+    ob_ftp_in,
+    ob_ftp_out,
+    ob_ssh_in,
+    ob_ssh_out,
+    ob_wwws_in,
+    ob_wwws_out,
+    ob_icmp_in,
+    ob_icmp_out,
+    ob_udp_in,
+    ob_udp_out,
+    ob_dns_in,
+    ob_dns_out,
+    ob_tcpsyn_in,
+    ob_tcpsyn_out,
+    ob_tcpack_in,
+    ob_tcpack_out,
+    ob_tcpfin_in,
+    ob_tcpfin_out,
+    ob_tcpmisc_in,
+    ob_tcpmisc_out,
+    ob_spares
+    };
+
+
+struct sock
+   {
+   char *portnr;
+   char *name;
+   enum observables in;
+   enum observables out;
+   };
+
+
 /*******************************************************************/
 
 struct cfagent_connection
    {
    int sd;
    int trust;
+   int protoversion;
    int family;     /* AF_INET or AF_INET6 */
    char localip[CF_MAX_IP_LEN];
    char remoteip[CF_MAX_IP_LEN];
@@ -1385,7 +1479,7 @@ struct Mountables
 struct Tidy
    {
    int          maxrecurse;              /* sets maxval */
-   char         done;       /* Too intensive in Tidy Pattern */
+   char         done;                    /* Too intensive in Tidy Pattern */
    char         *scope;
    char        *path;
    char         xdev;
@@ -1440,6 +1534,7 @@ struct MiscMount
    char *scope;
    char *from;
    char *onto;
+   char *mode;
    char *options;
    char *classes;
    int ifelapsed;
@@ -1868,6 +1963,17 @@ struct Package      /* For packages: */
    int expireafter;
    };
 
+
+/*******************************************************************/
+/* Checksum database structures                                    */
+/*******************************************************************/
+
+struct Checksum_Value
+   {
+   unsigned char mess_digest[EVP_MAX_MD_SIZE+1];  /* Content digest */
+   unsigned char attr_digest[EVP_MAX_MD_SIZE+1];  /* Attribute digest */
+   };
+
 /*******************************************************************/
 /* Ultrix/BSD don't have all these from sys/stat.h                 */
 /*******************************************************************/
