@@ -36,17 +36,18 @@ extern pthread_mutex_t MUTEX_SYSCALL;
 extern pthread_mutex_t MUTEX_LOCK;
 #endif
 
+extern short ALL_SINGLECOPY;
 extern int PASS;
 extern RSA *PRIVKEY, *PUBKEY;
 extern char BINDINTERFACE[CF_BUFSIZE];
-extern char *ECGSOCKS[ATTR][2];
+extern struct sock ECGSOCKS[ATTR];
 extern char *TCPNAMES[CF_NETATTR];
 extern char **METHODARGV;
 extern int GOTMETHODARGS;
-
 extern struct Item *QUERYVARS;
-extern struct Item *METHODRETURNVARS;;
+extern struct Item *METHODRETURNVARS;
 extern struct Item *METHODRETURNCLASSES;
+
 extern char METHODFILENAME[CF_BUFSIZE];
 extern char *VMETHODPROTO[];
 extern int METHODARGC;
@@ -61,6 +62,7 @@ extern struct cfagent_connection *CONN;
 extern int AUTHENTICATED;
 extern struct Item *IPADDRESSES;
 
+extern char PIDFILE[CF_BUFSIZE];
 extern char  STR_CFENGINEPORT[16];
 extern unsigned short SHORT_CFENGINEPORT;
 
@@ -136,7 +138,6 @@ extern time_t CFINITSTARTTIME;
 extern int CF_TIMEOUT;
 
 extern struct utsname VSYSNAME;
-
 extern int LINENUMBER;
 extern mode_t DEFAULTMODE;
 extern mode_t DEFAULTSYSTEMMODE;
@@ -163,6 +164,7 @@ extern char PRESERVETIMES;
 extern char TRUSTKEY;
 extern char FORK;
 
+extern short FULLENCRYPT;
 extern short SKIPIDENTIFY;
 extern short COMPATIBILITY_MODE;
 extern short LINKSILENT;
@@ -183,7 +185,7 @@ extern char *GROUPBUFF;
 extern char *ACTIONBUFF;
 extern char *CLASSBUFF;
 extern char ALLCLASSBUFFER[4*CF_BUFSIZE];
-extern char CHROOT[CF_ALLCLASSSIZE];
+extern char CHROOT[CF_BUFSIZE];
 extern char ELSECLASSBUFFER[CF_BUFSIZE];
 extern char FAILOVERBUFFER[CF_BUFSIZE];
 extern char *LINKFROM;
@@ -253,9 +255,11 @@ extern struct Item *VACLBUILD;
 extern struct Item *VFILTERBUILD;
 extern struct Item *VSTRATEGYBUILD;
 
+
 extern struct Item *VMOUNTLIST;
 extern struct Item *VHEAP;      /* Points to the base of the attribute heap */
 extern struct Item *VNEGHEAP;
+extern struct Item *ABORTHEAP;
 
 /* For packages: */
 extern struct Package *VPKG;
@@ -507,7 +511,13 @@ extern int FOREACHLEVEL;
 
 extern char COMMENTSTART[], COMMENTEND[];
 
+extern char *OBS[CF_OBSERVABLES];
+
 /* GNU REGEXP */
 
 extern struct re_pattern_buffer *SEARCHPATTBUFF;
 extern struct re_pattern_buffer *PATTBUFFER;
+
+
+extern char *CF_DIGEST_TYPES[4][2];
+extern int CF_DIGEST_SIZES[4];
