@@ -39,26 +39,25 @@
 /* Functions internal to cfengine.c                                */
 /*******************************************************************/
 
-int main ARGLIST((int argc,char *argv[]));
-void Initialize ARGLIST((int argc, char **argv));
-void PreNetConfig ARGLIST((void));
-void ReadRCFile ARGLIST((void));
-void EchoValues ARGLIST((void));
-void CheckSystemVariables ARGLIST((void));
-void SetReferenceTime ARGLIST((int setclasses));
-void SetStartTime ARGLIST((int setclasses));
-void DoTree ARGLIST((int passes, char *info));
-enum aseq EvaluateAction ARGLIST((char *action, struct Item **classlist, int pass));
-void CheckOpts ARGLIST((int argc, char **argv));
-int GetResource ARGLIST((char *var));
-void BuildClassEnvironment ARGLIST((void));
-void Syntax ARGLIST((void));
-void EmptyActionSequence ARGLIST((void));
-void GetEnvironment ARGLIST((void));
-int NothingLeftToDo ARGLIST((void));
-void SummarizeObjects ARGLIST((void));
-void SetContext ARGLIST((char *id));
-void DeleteCaches ARGLIST((void));
+int main (int argc,char *argv[]);
+void Initialize (int argc, char **argv);
+void PreNetConfig (void);
+void ReadRCFile (void);
+void EchoValues (void);
+void CheckSystemVariables (void);
+void SetReferenceTime (int setclasses);
+void SetStartTime (int setclasses);
+void DoTree (int passes, char *info);
+enum aseq EvaluateAction (char *action, struct Item **classlist, int pass);
+void CheckOpts (int argc, char **argv);
+int GetResource (char *var);
+void Syntax (void);
+void EmptyActionSequence (void);
+void GetEnvironment (void);
+int NothingLeftToDo (void);
+void SummarizeObjects (void);
+void SetContext (char *id);
+void DeleteCaches (void);
 void QueryCheck(void);
 
 /*******************************************************************/
@@ -68,7 +67,7 @@ void QueryCheck(void);
 int main(int argc,char *argv[])
 
 { struct Item *ip;
-    
+ 
 SetContext("global");
 SetSignals(); 
  
@@ -173,12 +172,12 @@ ReadRCFile(); /* Should come before parsing so that it can be overridden */
  
  CfOpenLog();
  CheckSystemVariables();
- 
+
  SetReferenceTime(false); /* Reset */
 
  QueryCheck();
-     
- DoTree(4,"Main Tree"); 
+
+ DoTree(3,"Main Tree"); 
  DoAlerts();
  
  CheckMethodReply();
@@ -288,7 +287,21 @@ for (i = 1; i < argc; i++)
     {
     CreateEmptyFile(ebuff);
     }
+
+ sprintf(ebuff,"%s/state/cf_rootprocs",CFWORKDIR);
  
+ if (stat(ebuff,&statbuf) == -1)
+    {
+    CreateEmptyFile(ebuff);
+    }
+ 
+ sprintf(ebuff,"%s/state/cf_otherprocs",CFWORKDIR);
+ 
+ if (stat(ebuff,&statbuf) == -1)
+    {
+    CreateEmptyFile(ebuff);
+    }
+
  strcpy(VLOGDIR,CFWORKDIR); 
  strcpy(VLOCKDIR,VLOGDIR);  /* Same since 2.0.a8 */
  
@@ -570,7 +583,8 @@ while (!feof(fp))
    class[0] = '\0';
    name[0] = '\0';
    value[0] = '\0';
-   fscanf(fp,"%256s",class);
+
+   fgets(class,CF_BUFSIZE-1,fp);
 
    if (feof(fp))
       {
@@ -579,7 +593,8 @@ while (!feof(fp))
 
    if (strstr(class,"="))
       {
-      sscanf(class,"%255[^=]=%255s",name,value);
+      sscanf(class,"%255[^=]=%255[^\n]",name,value);
+
       if (!GetMacroValue(CONTEXTID,name))
          {
          AddMacroValue(CONTEXTID,name,value);
@@ -603,8 +618,6 @@ void EchoValues()
   char ebuff[CF_EXPANDSIZE];
 
 ebuff[0] = '\0';
-  
-Verbose("Accepted domain name: %s\n\n",VDOMAIN); 
 
 if (GetMacroValue(CONTEXTID,"OutputPrefix"))
    {
@@ -767,6 +780,28 @@ if (VACTIONSEQ == NULL)
 
 sprintf(id,"%d",geteuid());   /* get effective user id */
 
+ebuff[0] = '\0';
+
+if (!StrStr(VSYSNAME.nodename,VDOMAIN))
+   {
+   snprintf(VFQNAME,CF_BUFSIZE,"%s.%s",VSYSNAME.nodename,ToLowerStr(VDOMAIN));
+   strcpy(VUQNAME,VSYSNAME.nodename);
+   }
+else
+   {
+   int n = 0;
+   strcpy(VFQNAME,VSYSNAME.nodename);
+   
+   while(VSYSNAME.nodename[n++] != '.')
+      {
+      }
+   
+   strncpy(VUQNAME,VSYSNAME.nodename,n-1);        
+   }
+  
+Verbose("Accepted domain name: %s\n\n",VDOMAIN); 
+
+
 if (VACCESSLIST != NULL && !IsItemIn(VACCESSLIST,id))
    {
    FatalError("Access denied - user not listed in access list");
@@ -893,6 +928,11 @@ if (OptionIs(CONTEXTID,"LastSeen",false))
    LASTSEEN = false;
    }
 
+if (OptionIs(CONTEXTID,"FullEncrypt",true))
+   {
+   FULLENCRYPT = true;
+   }
+
 if (OptionIs(CONTEXTID,"Inform",true))
    {
    INFORM = true;
@@ -994,18 +1034,11 @@ if (GetMacroValue(CONTEXTID,"DefaultCopyType"))
  
 if (GetMacroValue(CONTEXTID,"ChecksumDatabase"))
    {
-   ExpandVarstring("$(ChecksumDatabase)",ebuff,NULL);
-
-   CHECKSUMDB = strdup(ebuff);
-
-   if (*CHECKSUMDB != '/')
-      {
-      FatalError("$(ChecksumDatabase) does not expand to an absolute filename\n");
-      }
+   FatalError("ChecksumDatabase variable is deprecated - comment it out");
    }
 else
    {
-   snprintf(ebuff,CF_BUFSIZE,"%s/checksum.db",CFWORKDIR);
+   snprintf(ebuff,CF_BUFSIZE,"%s/%s",CFWORKDIR,CF_CHKDB);
    CHECKSUMDB = strdup(ebuff);
    }
 
@@ -1048,6 +1081,23 @@ if (GetMacroValue(CONTEXTID,"TimeOut"))
       }
    }
 
+/* Make sure we have a healthy binserver list so binserver expansion works, even if binserver not defined */
+
+if (VBINSERVERS == NULL)
+   {
+   PrependItem(&VBINSERVERS,VUQNAME,NULL);
+   }
+
+if (VBINSERVERS->name != NULL)
+   {
+   VDEFAULTBINSERVER = *VBINSERVERS;
+   Verbose("Default binary server seems to be %s\n",VDEFAULTBINSERVER.name);
+   }
+
+AppendItem(&VBINSERVERS,VUQNAME,NULL);
+
+/* Done binserver massage */
+
 if (NOSPLAY)
    {
    return;
@@ -1119,6 +1169,8 @@ exit(0);
 void DoTree(int passes,char *info)
 
 { struct Item *action;
+
+Debug("DoTree(%d,%s)\n",passes,info);
  
 for (PASS = 1; PASS <= passes; PASS++)
    {
@@ -1150,7 +1202,7 @@ for (PASS = 1; PASS <= passes; PASS++)
       Verbose(" %s Sched: %s pass %d @ %s",info,action->name,PASS,ctime(&CFINITSTARTTIME));
       Verbose("*********************************************************************\n\n");
       
-      switch(ACTION = EvaluateAction(action->name,&VADDCLASSES,PASS))
+      switch (EvaluateAction(action->name,&VADDCLASSES,PASS))
          {
          case mountinfo:
              if (PASS == 1)
@@ -1333,6 +1385,7 @@ int NothingLeftToDo()
   struct Process *vproclist;
   struct Tidy *vtidy;
   struct Package *vpkg;
+  struct Image *vcopy;
 
 if (IsWildItemIn(VACTIONSEQ,"process*"))
    {
@@ -1340,31 +1393,44 @@ if (IsWildItemIn(VACTIONSEQ,"process*"))
       {
       if ((vproclist->done == 'n') && IsDefinedClass(vproclist->classes))
          {
-         printf("Still waiting for rule:: Proc <%s> / %s\n",vproclist->expr,vproclist->classes);
+         Verbose("Checking for potential rule:: Proc <%s> / %s\n",vproclist->expr,vproclist->classes);
          return false;
          }
       }
    }
 
-if (IsWildItemIn(VACTIONSEQ,"shellcommand*"))
+if (IsWildItemIn(VACTIONSEQ,"shellcomman*"))
    {
    for (vscript = VSCRIPT; vscript != NULL; vscript=vscript->next)
       {
-      if (vscript->done == 'n'  && IsDefinedClass(vscript->classes))
+      if ((vscript->done == 'n')  && IsDefinedClass(vscript->classes))
          {
-         printf("Still waiting for rule:: Shell <%s> / %s\n",vscript->name,vscript->classes);
+         Verbose("Checking for potential rule:: Shell <%s> / %s\n",vscript->name,vscript->classes);
          return false;
          }
       }
    }
 
+if (IsWildItemIn(VACTIONSEQ,"cop*"))
+   {
+   for (vcopy = VIMAGE; vcopy != NULL; vcopy=vcopy->next)
+      {
+      if ((vcopy->done == 'n')  && IsDefinedClass(vcopy->classes))
+         {
+         Verbose("Checking for potential rule:: Copy <%s> / %s\n",vcopy->path,vcopy->classes);
+         return false;
+         }
+      }
+   }
+
+
 if (IsWildItemIn(VACTIONSEQ,"file*"))
    {
    for (vfile = VFILE; vfile != NULL; vfile=vfile->next)
       {
-      if (vfile->done == 'n' && IsDefinedClass(vfile->classes))
+      if ((vfile->done == 'n') && IsDefinedClass(vfile->classes))
          {
-         printf("Still waiting for rule:: File <%s>/ %s\n",vfile->path,vfile->classes);
+         Verbose("Checking for potential rule:: File <%s>/ %s\n",vfile->path,vfile->classes);
          return false;
          }
       }
@@ -1376,8 +1442,23 @@ if (IsWildItemIn(VACTIONSEQ,"tid*"))
       {
       if (vtidy->done == 'n')
          {
-         printf("Still waiting for rule:: Tidy <%s>\n",vtidy->path);
-         return false;
+         struct TidyPattern *tp;
+         int active = 0;
+         
+         for (tp = vtidy->tidylist; tp != NULL; tp=tp->next)
+            {
+            if (IsDefinedClass(tp->classes))
+               {
+               active=1;
+               break;
+               }
+            }
+
+         if (active)
+            {
+            Verbose("Checking for potential rule:: Tidy <%s>\n",vtidy->path);
+            return false;
+            }
          }
       }
    }
@@ -1394,14 +1475,21 @@ if (IsWildItemIn(VACTIONSEQ,"editfile*"))
          if (IsDefinedClass(ep->classes))
             {
             something_to_do = true;
-            printf("Defined Edit %s / %s\n",ep->data,ep->classes);
+            if (ep->data)
+               {
+               Verbose("Defined Edit %s / %s\n",ep->data,ep->classes);
+               }
+            else
+               {
+               Verbose("Defined Edit nodata / %s\n",ep->classes);
+               }
             break;
             }
          }
    
       if (veditlist->done == 'n' && something_to_do)
          {
-         printf("Still waiting for rule:: Edit <%s>\n",veditlist->fname);
+         Verbose("Checking for potential rule:: Edit <%s>\n",veditlist->fname);
          return false;
          }
       }
@@ -1413,7 +1501,7 @@ if (IsWildItemIn(VACTIONSEQ,"process*"))
       {
       if (vdisablelist->done == 'n' && IsDefinedClass(vdisablelist->classes))
          {
-         printf("Still waiting for rule:: Disable <%s> / %s\n",vdisablelist->name,vdisablelist->classes);
+         Verbose("Checking for potential rule:: Disable <%s> / %s\n",vdisablelist->name,vdisablelist->classes);
          return false;
          }
       }
@@ -1425,7 +1513,7 @@ if (IsWildItemIn(VACTIONSEQ,"director*"))
       {
       if (vmakepath->done == 'n' && IsDefinedClass(vmakepath->classes))
          {
-         printf("Still waiting for rule:: makePath <%s>\n",vmakepath->path);
+         Verbose("Checking for potential rule:: makePath <%s>\n",vmakepath->path);
          return false;
          }
       }
@@ -1437,7 +1525,7 @@ if (IsWildItemIn(VACTIONSEQ,"link*"))
       {
       if (vlink->done == 'n' && IsDefinedClass(vlink->classes))
          {
-         printf("Still waiting for rule:: Link <%s>\n",vlink->from);
+         Verbose("Checking for potential rule:: Link <%s>\n",vlink->from);
          return false;
          }
       }
@@ -1446,7 +1534,7 @@ if (IsWildItemIn(VACTIONSEQ,"link*"))
       {
       if (vchlink->done == 'n' && IsDefinedClass(vchlink->classes))
          {
-         printf("Still waiting for rule:: CLink <%s>\n",vlink->from);
+         Verbose("Checking for potential rule:: CLink <%s>\n",vchlink->from);
          return false;
          }
       }
@@ -1459,7 +1547,7 @@ if (IsWildItemIn(VACTIONSEQ,"unmoun*"))
       {
       if (vunmount->done == 'n' && IsDefinedClass(vunmount->classes))
          {
-         printf("Still waiting for rule:: Umount <%s>\n",vunmount->name);
+         Verbose("Checking for potential rule:: Umount <%s>\n",vunmount->name);
          return false;
          }
       }
@@ -1471,12 +1559,13 @@ if (IsWildItemIn(VACTIONSEQ,"packag*"))
       {
       if (vpkg->done == 'n' && IsDefinedClass(vpkg->classes))
          {
-         printf("Still waiting for rule:: Packages <%s>\n",vpkg->name);
+         Verbose("Checking for potential rule:: Packages <%s>\n",vpkg->name);
          return false;
          }
       }
    }
 
+
 return true; 
 }
 
@@ -1813,94 +1902,6 @@ FatalError(OUTPUT);
 return 0;
 }
 
-/*******************************************************************/
-
-void BuildClassEnvironment()
-
-{ struct Item *ip;
- int size = 0;
- char file[CF_BUFSIZE], *sp;
- FILE *fp;
- 
-Debug("(BuildClassEnvironment)\n");
-
-snprintf(ALLCLASSBUFFER,CF_BUFSIZE,"%s=",CF_ALLCLASSESVAR);
-
-if (!IsPrivileged())
-   {
-   Verbose("\n(Non privileged user...)\n\n");
-   
-   if ((sp = getenv("HOME")) == NULL)
-      {
-      FatalError("You do not have a HOME variable pointing to your home directory");
-      }  
-
-   snprintf(file,CF_BUFSIZE,"%s/.cfagent/allclasses",sp);
-   }
-else
-   {
-   snprintf(file,CF_BUFSIZE,"%s/state/allclasses",WORKDIR);
-   }
-
-
-if ((fp = fopen(file,"w")) == NULL)
-   {
-   CfLog(cfinform,"Could not open allclasses cache file","");
-   return;
-   }
-
-for (ip = VHEAP; ip != NULL; ip=ip->next)
-   {
-   if (IsDefinedClass(ip->name))
-      {
-      if ((size += strlen(ip->name)) > CF_ALLCLASSSIZE - CF_BUFFERMARGIN)
-         {
-         Verbose("Class buffer overflowed, dumping class environment for modules\n");
-         Verbose("This would probably crash the exec interface on most machines\n");
-         }
-      else
-         {
-         size++; /* Allow for : separator */
-         strcat(ALLCLASSBUFFER,ip->name);
-         strcat(ALLCLASSBUFFER,":");
-         }
-
-      fprintf(fp,"%s\n",ip->name);
-      }
-   }
- 
- for (ip = VALLADDCLASSES; ip != NULL; ip=ip->next)
-    {
-    if (IsDefinedClass(ip->name))
-      {
-      if ((size += strlen(ip->name)) > 4*CF_BUFSIZE - CF_BUFFERMARGIN)
-         {
-         Verbose("Class buffer overflowed, dumping class environment for modules\n");
-         Verbose("This would probably crash the exec interface on most machines\n");
-         }
-      else
-         {
-         size++; /* Allow for : separator */
-         strcat(ALLCLASSBUFFER,ip->name);
-         strcat(ALLCLASSBUFFER,":");         
-         }
-      
-      fprintf(fp,"%s\n",ip->name);
-      }
-    }
- 
- Debug2("---\nENVIRONMENT: %s\n---\n",ALLCLASSBUFFER);
- 
- if (USEENVIRON)
-    {
-    if (putenv(ALLCLASSBUFFER) == -1)
-       {
-       perror("putenv");
-       }
-    }
-
- fclose(fp);
-}
 
 /*******************************************************************/
 
@@ -1921,6 +1922,6 @@ printf("\nDebug levels: 1=parsing, 2=running, 3=summary, 4=expression eval\n");
 
 printf("\nBug reports to bug-cfengine@gnu.org (News: gnu.cfengine.bug)\n");
 printf("General help to help-cfengine@gnu.org (News: gnu.cfengine.help)\n");
-printf("Info & fixes at http://www.iu.hio.no/cfengine\n");
+printf("Info & fixes at http://www.cfengine.org\n");
 }
 /* EOF */
