@@ -66,14 +66,14 @@ char CFLOCK[CF_BUFSIZE] = "dummy";
 /* Functions internal to cfrun.c                                   */
 /*******************************************************************/
 
-void CheckOptsAndInit  ARGLIST((int argc,char **argv));
-int PollServer ARGLIST((char *host, char *options, int storeinfile));
-void SendClassData ARGLIST((int sd, char *sendbuffer));
-void CheckAccess ARGLIST((char *users));
-void cfrunSyntax ARGLIST((void));
-void ReadCfrunConf ARGLIST((char* cfg_fic));
-int ParseHostname ARGLIST((char *hostname, char *new_hostname));
-void FileOutput ARGLIST((FILE *fp, enum fileoutputlevels level, char *message));
+void CheckOptsAndInit  (int argc,char **argv);
+int PollServer (char *host, char *options, int storeinfile);
+void SendClassData (int sd, char *sendbuffer);
+void CheckAccess (char *users);
+void cfrunSyntax (void);
+void ReadCfrunConf (char* cfg_fic);
+int ParseHostname (char *hostname, char *new_hostname);
+void FileOutput (FILE *fp, enum fileoutputlevels level, char *message);
 
 /*******************************************************************/
 /* Level 0 : Main                                                  */
@@ -289,8 +289,8 @@ int PollServer(char *host,char *options,int storeinfile)
   struct Image addresses;
 #ifdef HAVE_GETADDRINFO
   struct addrinfo query, *response=NULL, *ap;
-  char strport[16];
 #endif
+  char strport[16];
   
 CONN = NewAgentConn();
 
@@ -425,7 +425,7 @@ if (!gotkey)
       }
    }
  
-if (!RemoteConnect(parsed_host,forceipv4,SHORT_CFENGINEPORT,STR_CFENGINEPORT))
+if (!RemoteConnect(parsed_host,forceipv4,port,strport))
    {
    CfLog(cferror,"Couldn't open a socket","socket");
    FileOutput(fp, fopl_error, "Couldn't open a socket\n");
@@ -489,7 +489,7 @@ while (true)
 
       close(CONN->sd);
       free(addresses.server);
-      return true;;
+      return true;
       }
 
    if ((sp = strstr(recvbuffer,CFD_TERMINATOR)) != NULL)
@@ -603,9 +603,9 @@ while (!feof(fp))
 
    if (strncmp(line,"hostnamekeys",6) == 0)
       {
-      char buf[16];
+      char buf[256];
       buf[0] = '\0';
-      sscanf(line,"hostnamekeys = %295[^# \n]",buf);
+      sscanf(line,"hostnamekeys = %255[^# \n]",buf);
       Verbose("Hostname keys\n");
 
       if (strcmp(buf,"yes") == 0||strcmp(buf,"true") == 0)
