@@ -67,6 +67,7 @@ int MULTIPLECONNS = false;
 int TRIES = 0;
 int MAXTRIES = 5;
 int LOGCONNS = false;
+int LOGENCRYPT = false;
 
 struct option CFDOPTIONS[] =
    {
@@ -88,50 +89,51 @@ struct Item *CONNECTIONLIST = NULL;
 /* Functions internal to cfservd.c                                 */
 /*******************************************************************/
 
-void CheckOptsAndInit ARGLIST((int argc,char **argv));
-void CheckVariables ARGLIST((void));
-void SummarizeParsing ARGLIST((void));
-void StartServer ARGLIST((int argc, char **argv));
-int OpenReceiverChannel ARGLIST((void));
-void Syntax ARGLIST((void));
-void PurgeOldConnections ARGLIST((struct Item **list,time_t now));
-void SpawnConnection ARGLIST((int sd_reply, char *ipaddr));
-void CheckFileChanges ARGLIST((int argc, char **argv, int sd));
-void *HandleConnection ARGLIST((struct cfd_connection *conn));
-int BusyWithConnection ARGLIST((struct cfd_connection *conn));
-void *ExitCleanly ARGLIST((int signum));
-int MatchClasses ARGLIST((struct cfd_connection *conn));
-void DoExec ARGLIST((struct cfd_connection *conn, char *sendbuffer, char *args));
-int GetCommand ARGLIST((char *str));
-int VerifyConnection ARGLIST((struct cfd_connection *conn, char *buf));
-void RefuseAccess ARGLIST((struct cfd_connection *conn, char *sendbuffer, int size, char *errormsg));
-int AccessControl ARGLIST((char *filename, struct cfd_connection *conn, int encrypt));
-int CheckStoreKey  ARGLIST((struct cfd_connection *conn, RSA *key));
-int StatFile ARGLIST((struct cfd_connection *conn, char *sendbuffer, char *filename));
-void CfGetFile ARGLIST((struct cfd_get_arg *args));
-void CompareLocalChecksum ARGLIST((struct cfd_connection *conn, char *sendbuffer, char *recvbuffer));
-int CfOpenDirectory ARGLIST((struct cfd_connection *conn, char *sendbuffer, char *dirname));
-void Terminate ARGLIST((int sd));
-void DeleteAuthList ARGLIST((struct Auth *ap));
-int AllowedUser ARGLIST((char *user));
-void ReplyNothing ARGLIST((struct cfd_connection *conn));
-struct cfd_connection *NewConn ARGLIST((int sd));
-void DeleteConn ARGLIST((struct cfd_connection *conn));
-time_t SecondsTillAuto ARGLIST((void));
-void SetAuto ARGLIST((int seconds));
-int cfscanf ARGLIST((char *in, int len1, int len2, char *out1, char *out2, char *out3));
-int AuthenticationDialogue ARGLIST((struct cfd_connection *conn,char *buffer, int buffersize));
-char *MapAddress ARGLIST((char *addr));
-int IsWildKnownHost ARGLIST((RSA *oldkey,RSA *newkey,char *addr,char *user));
-void AddToKeyDB ARGLIST((RSA *key,char *addr));
-int SafeOpen ARGLIST((char *filename));
-void SafeClose ARGLIST((int fd));
+void CheckOptsAndInit (int argc,char **argv);
+void CheckVariables (void);
+void SummarizeParsing (void);
+void StartServer (int argc, char **argv);
+int OpenReceiverChannel (void);
+void Syntax (void);
+void PurgeOldConnections (struct Item **list,time_t now);
+void SpawnConnection (int sd_reply, char *ipaddr);
+void CheckFileChanges (int argc, char **argv, int sd);
+void *HandleConnection (struct cfd_connection *conn);
+int BusyWithConnection (struct cfd_connection *conn);
+void *ExitCleanly (int signum);
+int MatchClasses (struct cfd_connection *conn);
+void DoExec (struct cfd_connection *conn, char *sendbuffer, char *args);
+int GetCommand (char *str);
+int VerifyConnection (struct cfd_connection *conn, char *buf);
+void RefuseAccess (struct cfd_connection *conn, char *sendbuffer, int size, char *errormsg);
+int AccessControl (char *filename, struct cfd_connection *conn, int encrypt);
+int CheckStoreKey  (struct cfd_connection *conn, RSA *key);
+int StatFile (struct cfd_connection *conn, char *sendbuffer, char *filename);
+void CfGetFile (struct cfd_get_arg *args);
+void CompareLocalChecksum (struct cfd_connection *conn, char *sendbuffer, char *recvbuffer);
+int CfOpenDirectory (struct cfd_connection *conn, char *sendbuffer, char *dirname);
+int CfSecOpenDirectory (struct cfd_connection *conn, char *sendbuffer, char *dirname);
+void Terminate (int sd);
+void DeleteAuthList (struct Auth *ap);
+int AllowedUser (char *user);
+void ReplyNothing (struct cfd_connection *conn);
+struct cfd_connection *NewConn (int sd);
+void DeleteConn (struct cfd_connection *conn);
+time_t SecondsTillAuto (void);
+void SetAuto (int seconds);
+int cfscanf (char *in, int len1, int len2, char *out1, char *out2, char *out3);
+int AuthenticationDialogue (struct cfd_connection *conn,char *buffer, int buffersize);
+char *MapAddress (char *addr);
+int IsWildKnownHost (RSA *oldkey,RSA *newkey,char *addr,char *user);
+void AddToKeyDB (RSA *key,char *addr);
+int SafeOpen (char *filename);
+void SafeClose (int fd);
 
 /*
  * HvB
 */
 
-unsigned find_inet_addr ARGLIST((char *host));
+unsigned find_inet_addr (char *host);
 
 /*******************************************************************/
 /* Level 0 : Main                                                  */
@@ -142,9 +144,12 @@ int main (int argc,char **argv)
 {
 CheckOptsAndInit(argc,argv);
 ParseInputFile(VINPUTFILE);
+
 /* We call CfOpenLog again so, if we configure SyslogFacility, 
    we open syslog again */
+closelog();
 CfOpenLog();
+
 CheckVariables();
 SummarizeParsing();
 
@@ -333,11 +338,6 @@ if (OptionIs(CONTEXTID,"CheckIdent", true))
    CHECK_RFC931 = true;
    }
 
-if (OptionIs(CONTEXTID,"LastSeen",false))
-   {
-   LASTSEEN = false;
-   }
-
 if (OptionIs(CONTEXTID,"DenyBadClocks", false)) 
    {
    DENYBADCLOCKS = false;
@@ -348,19 +348,20 @@ if (OptionIs(CONTEXTID,"LogAllConnections", true))
    LOGCONNS = true;
    }
 
-if (GetMacroValue(CONTEXTID,"ChecksumDatabase"))
+if (OptionIs(CONTEXTID,"LogEncryptedTransfers", true)) 
    {
-   ExpandVarstring("$(ChecksumDatabase)",ebuff,NULL);
-
-   CHECKSUMDB = strdup(ebuff);
-
-   if (*CHECKSUMDB != '/')
-      {
-      FatalError("$(ChecksumDatabase) does not expand to an absolute filename\n");
-      }
+   LOGENCRYPT = true;
    }
 
- memset(CFRUNCOMMAND,0,CF_BUFSIZE);
+if (GetMacroValue(CONTEXTID,"ChecksumDatabase"))
+   {
+   snprintf(ebuff,CF_BUFSIZE,"%s/%s",CFWORKDIR,CF_CHKDB);
+   /* CHECKSUMDB = strdup(ebuff);*/
+   CHECKSUMDB = NULL;
+   Verbose("Checksum database is no longer used in cfservd\n");
+   }
+
+memset(CFRUNCOMMAND,0,CF_BUFSIZE);
  
 if (GetMacroValue(CONTEXTID,"cfrunCommand"))
    {
@@ -389,7 +390,7 @@ if (GetMacroValue(CONTEXTID,"MaxConnections"))
    
    MAXTRIES = CFD_MAXPROCESSES / 3;  /* How many attempted connections over max
             before we commit suicide? */
-   if ((CFD_MAXPROCESSES < 1) || (CFD_MAXPROCESSES > 1000))
+   if ((CFD_MAXPROCESSES < 1) || (CFD_MAXPROCESSES > 10000))
       {
       FatalError("cfservd MaxConnections with silly value");
       }
@@ -413,7 +414,9 @@ if (OptionIs(CONTEXTID,"ChecksumUpdates", false))
  
 i = 0;
 
-if (StrStr(VSYSNAME.nodename,ToLowerStr(VDOMAIN)))
+strncpy(ebuff,ToLowerStr(VDOMAIN),127);
+
+if (StrStr(VSYSNAME.nodename,ebuff))
    {
    strncpy(VFQNAME,VSYSNAME.nodename,CF_MAXVARSIZE-1);
    
@@ -541,7 +544,7 @@ if (ERRORCOUNT > 0)
 void StartServer(int argc,char **argv)
 
 { char ipaddr[CF_MAXVARSIZE],intime[64];
-  int sd,sd_reply,ageing;
+  int sd,sd_reply;
   fd_set rset;
   time_t now;
   struct timeval timeout;
@@ -581,6 +584,7 @@ if ((!NO_FORK) && (fork() != 0))
    {
    snprintf(OUTPUT,CF_BUFSIZE*2,"cfservd starting %.24s\n",ctime(&CFDSTARTTIME));
    CfLog(cfinform,OUTPUT,"");
+   closelog();
    exit(0);
    }
 
@@ -588,8 +592,8 @@ if (!NO_FORK)
    {
    ActAsDaemon(sd);
    }
- 
-ageing = 0;
+
+WritePID("cfservd.pid");
 
 /* Andrew Stribblehill <ads@debian.org> -- close sd on exec */ 
 fcntl(sd, F_SETFD, FD_CLOEXEC);
@@ -604,11 +608,12 @@ while (true)
    FD_ZERO(&rset);
    FD_SET(sd,&rset);
    
-   timeout.tv_sec = 5;  // Set a 5 second timeout for select
+   timeout.tv_sec = 10;  /* Set a 10 second timeout for select */
    timeout.tv_usec = 0;
    
    ret_val = select((sd+1),&rset,NULL,NULL,&timeout);
-   if (ret_val == -1)   // Error received from call to select
+
+   if (ret_val == -1)   /* Error received from call to select */
       {
       if (errno == EINTR)
          {
@@ -620,19 +625,13 @@ while (true)
          exit(1);
          }
       }
-   else if (!ret_val)   // No data waiting, we must have timed out!
+   else if (!ret_val)   /* No data waiting, we must have timed out! */
       {
       continue;
       }
    
    if ((sd_reply = accept(sd,(struct sockaddr *)&cin,&addrlen)) != -1)
       {
-      if (ageing++ > CFD_MAXPROCESSES*50) /* Insurance against stale db */
-         {                                /* estimate number of clients */
-         unlink(CHECKSUMDB);              /* arbitrary policy ..        */
-         ageing = 0;
-         }
-      
       memset(ipaddr,0,CF_MAXVARSIZE);
       snprintf(ipaddr,CF_MAXVARSIZE-1,"%s",sockaddr_ntop((struct sockaddr *)&cin));
       
@@ -697,7 +696,6 @@ while (true)
       return;
       }
 #endif
-
       
       SpawnConnection(sd_reply,ipaddr);
       }
@@ -1055,8 +1053,14 @@ if (CFDSTARTTIME < newstat.st_mtime)
    DeleteItemList(VNEGHEAP);
    DeleteItemList(TRUSTKEYLIST);
    DeleteItemList(VIMPORT);
+   DeleteItemList(SKIPVERIFY);
+   DeleteItemList(DHCPLIST);
+   DeleteItemList(ATTACKERLIST);
+   DeleteItemList(NONATTACKERLIST);
+   DeleteItemList(MULTICONNLIST);
    DeleteAuthList(VADMIT);
    DeleteAuthList(VDENY);
+
    strcpy(VDOMAIN,"undefined.domain");
 
    VADMIT = VADMITTOP = NULL;
@@ -1064,6 +1068,11 @@ if (CFDSTARTTIME < newstat.st_mtime)
    VHEAP  = VNEGHEAP  = NULL;
    VIMPORT = NULL;
    TRUSTKEYLIST = NULL;
+   SKIPVERIFY = NULL;
+   DHCPLIST = NULL;
+   ATTACKERLIST = NULL;
+   NONATTACKERLIST = NULL;
+   MULTICONNLIST = NULL;
 
    BlankHashTable("server");
 
@@ -1183,13 +1192,13 @@ int BusyWithConnection(struct cfd_connection *conn)
   /* and extract the information from the message */
 
 { time_t tloc, trem = 0;
-  char recvbuffer[CF_BUFSIZE+128], sendbuffer[CF_BUFSIZE],check[CF_BUFSIZE];  
+  char recvbuffer[CF_BUFSIZE+CF_BUFEXT], sendbuffer[CF_BUFSIZE],check[CF_BUFSIZE];  
   char filename[CF_BUFSIZE],buffer[CF_BUFSIZE],args[CF_BUFSIZE],out[CF_BUFSIZE];
   long time_no_see = 0;
   int len=0, drift, plainlen, received;
   struct cfd_get_arg get_args;
 
-memset(recvbuffer,0,CF_BUFSIZE+128);
+memset(recvbuffer,0,CF_BUFSIZE+CF_BUFEXT);
 memset(&get_args,0,sizeof(get_args));
 
 if ((received = ReceiveTransaction(conn->sd_reply,recvbuffer,NULL)) == -1)
@@ -1244,6 +1253,17 @@ switch (GetCommand(recvbuffer))
        DoExec(conn,sendbuffer,args);
        Terminate(conn->sd_reply);
        return false;
+
+   case cfd_version:
+
+       if (! conn->id_verified)
+          {
+          RefuseAccess(conn,sendbuffer,0,recvbuffer);
+          }
+
+       snprintf(conn->output,CF_BUFSIZE,"OK: %s",VERSION);
+       SendTransaction(conn->sd_reply,conn->output,0,CF_DONE);
+       return conn->id_verified;
        
    case cfd_cauth:
 
@@ -1367,6 +1387,52 @@ switch (GetCommand(recvbuffer))
        
        CfGetFile(&get_args);
        return true;
+
+   case cfd_sopendir:
+
+       memset(buffer,0,CF_BUFSIZE);
+       sscanf(recvbuffer,"SOPENDIR %d",&len);
+
+       if (received != len+CF_PROTO_OFFSET)
+          {
+          Debug("Protocol error: %d\n",len);
+          RefuseAccess(conn,sendbuffer,0,recvbuffer);
+          return false;
+          }
+
+       if (conn->session_key == NULL)
+          {
+          RefuseAccess(conn,sendbuffer,0,recvbuffer);
+          return false;
+          }
+
+       memcpy(out,recvbuffer+CF_PROTO_OFFSET,len);
+       
+       plainlen = DecryptString(out,recvbuffer,conn->session_key,len);
+
+       if (strncmp(recvbuffer,"OPENDIR",7) !=0)
+          {
+          RefuseAccess(conn,sendbuffer,0,recvbuffer);
+          return true;
+          }
+
+       memset(filename,0,CF_BUFSIZE);
+       sscanf(recvbuffer,"OPENDIR %[^\n]",filename);
+       
+       if (! conn->id_verified)
+          {
+          RefuseAccess(conn,sendbuffer,0,recvbuffer);
+          return false;
+          }
+       
+       if (!AccessControl(filename,conn,true)) /* opendir don't care about privacy */
+          {
+          RefuseAccess(conn,sendbuffer,0,recvbuffer);
+          return false;   
+          }       
+       
+       CfSecOpenDirectory(conn,sendbuffer,filename);
+       return true;
        
    case cfd_opendir:
 
@@ -1526,6 +1592,7 @@ for (i = 0; i < MAXTHREAD ; i++)
 #endif 
  
 HandleSignal(signum);
+unlink(PIDFILE);
 return NULL;
 }
 
@@ -1599,7 +1666,7 @@ while (true && (count < 10))  /* arbitrary check to avoid infinite loop, DoS att
 void DoExec(struct cfd_connection *conn,char *sendbuffer,char *args)
 
 { char ebuff[CF_EXPANDSIZE], line[CF_BUFSIZE], *sp;
-  int print = false,i;
+  int print = false,i, opt=false;
   FILE *pp;
 
 if ((CFSTARTTIME = time((time_t *)NULL)) == -1)
@@ -1617,7 +1684,7 @@ if (GetMacroValue(CONTEXTID,"cfrunCommand") == NULL)
 
 for (sp = args; *sp != '\0'; sp++) /* Blank out -K -f */
    {
-   if ((strncmp(sp,"-K",2) == 0) || (strncmp(sp,"-f",2) == 0) || (strncmp(sp,"f ",2) == 0))
+   if ((strncmp(sp,"-K",2) == 0) || (strncmp(sp,"-f",2) == 0))
       {
       *sp = ' ';
       *(sp+1) = ' ';
@@ -1637,6 +1704,36 @@ for (sp = args; *sp != '\0'; sp++) /* Blank out -K -f */
          }
       }
    }
+
+for (sp = args; *sp != '\0'; sp++) /* Blank out -f */
+   { 
+
+   if (strncmp(sp,"-D",2) == 0)
+      {
+      opt = false;
+      sp+=2;
+      }
+
+   if (strncmp(sp,"--define",8) == 0)
+      {
+      opt = false;
+      sp+=8;
+      }
+
+   switch (*sp)
+      {
+      case '-': opt = true;
+          break;
+      case ' ': opt = false;
+          break;
+      case 'f': if (opt)
+                   {
+                   *sp = 'q'; /* Replace with non-empty char which is defined anyway */
+                   }
+          break;
+      }
+   
+   }
     
 ExpandVarstring("$(cfrunCommand) --no-splay --inform",ebuff,"");
  
@@ -1788,11 +1885,7 @@ if ((conn->trust == false) || IsFuzzyItemIn(SKIPVERIFY,MapAddress(conn->ipaddr))
       conn->uid = pw->pw_uid;
       }
 
-   if (LASTSEEN)
-      {
-      LastSeen(dns_assert,cf_accept);
-      }
-   
+   LastSeen(dns_assert,cf_accept);
    return true;
    }
  
@@ -1999,40 +2092,46 @@ int AccessControl(char *filename,struct cfd_connection *conn,int encrypt)
 
 { struct Auth *ap;
   int access = false;
-  char realname[CF_BUFSIZE];
+  char realname[CF_BUFSIZE],path[CF_BUFSIZE],lastnode[CF_BUFSIZE];
   struct stat statbuf;
 
 Debug("AccessControl(%s)\n",filename);
 memset(realname,0,CF_BUFSIZE);
 
-if (lstat(filename,&statbuf) == -1)
+/* Separate path first, else this breaks for lastnode = symlink */
+
+strncpy(path,filename,CF_BUFSIZE-1);
+strncpy(lastnode,ReadLastNode(filename),CF_BUFSIZE-1);
+ChopLastNode(path);
+
+/* Eliminate links from path */
+
+#ifdef HAVE_REALPATH
+if (realpath(path,realname) == NULL)
    {
-   snprintf(conn->output,CF_BUFSIZE*2,"Couldn't stat filename %s from host %s\n",filename,conn->hostname);
+   snprintf(conn->output,CF_BUFSIZE*2,"Couldn't resolve filename %s from host %s\n",filename,conn->hostname);
+   CfLog(cfverbose,conn->output,"lstat");
+   CfLog(cflogonly,conn->output,"lstat");   
+   return false;
+   }
+#else
+CompressPath(realname,path); /* in links.c */
+#endif
+
+/* Rejoin the last node and stat the real thing */
+
+AddSlash(realname);
+strcat(realname,lastnode);
+
+if (lstat(realname,&statbuf) == -1)
+   {
+   snprintf(conn->output,CF_BUFSIZE*2,"Couldn't stat filename %s (i.e. %s) from host %s\n",filename,realname,conn->hostname);
    CfLog(cfverbose,conn->output,"lstat");
    CfLog(cflogonly,conn->output,"lstat");   
    return false;
    }
 
-if (S_ISLNK(statbuf.st_mode)) 
-   {
-   strncpy(realname,filename,CF_BUFSIZE);
-   }
- else
-    {
-#ifdef HAVE_REALPATH
-    if (realpath(filename,realname) == NULL)
-       {
-       snprintf(conn->output,CF_BUFSIZE*2,"Couldn't resolve filename %s from host %s\n",filename,conn->hostname);
-       CfLog(cfverbose,conn->output,"lstat");
-       CfLog(cflogonly,conn->output,"lstat");   
-       return false;
-       }
-#else
-    CompressPath(realname,filename); /* in links.c */
-#endif
-    }
- 
-Debug("AccessControl(%s,%s) encrypt request=%d\n",realname,conn->hostname,encrypt);
+Debug("AccessControl, match(%s,%s) encrypt request=%d\n",realname,conn->hostname,encrypt);
  
 if (VADMIT == NULL)
    {
@@ -2059,7 +2158,7 @@ for (ap = VADMIT; ap != NULL; ap=ap->next)
    
    if (res)
       {
-      Debug("Found a matching rule in access list (%s,%s)\n",realname,ap->path);
+      Debug("Found a matching rule in access list (%s in %s)\n",realname,ap->path);
       if (stat(ap->path,&statbuf) == -1)
          {
          snprintf(OUTPUT,CF_BUFSIZE,"Warning cannot stat file object %s in admit/grant, or access list refers to dangling link\n",ap->path);
@@ -2121,15 +2220,20 @@ for (ap = VADMIT; ap != NULL; ap=ap->next)
     {
     snprintf(conn->output,CF_BUFSIZE*2,"Host %s granted access to %s\n",conn->hostname,realname);
     CfLog(cfverbose,conn->output,"");
+    
+    if (encrypt && LOGENCRYPT)
+       {
+       /* Log files that were marked as requiring encryption */
+       CfLog(cfloginform,conn->output,"");
+       }
     }
  else
     {
     snprintf(conn->output,CF_BUFSIZE*2,"Host %s denied access to %s\n",conn->hostname,realname);
     CfLog(cfverbose,conn->output,"");
-    CfLog(cflogonly,conn->output,"");
+    CfLog(cfloginform,conn->output,"");
     }
  
- 
  if (!conn->rsa_auth)
     {
     CfLog(cfverbose,"Cannot map root access without RSA authentication","");
@@ -2147,8 +2251,9 @@ int AuthenticationDialogue(struct cfd_connection *conn,char *recvbuffer, int rec
 { char in[CF_BUFSIZE],*out, *decrypted_nonce;
   BIGNUM *counter_challenge = NULL;
   unsigned char digest[EVP_MAX_MD_SIZE+1];
-  unsigned int crypt_len, nonce_len = 0,len = 0, encrypted_len = 0, keylen;
+  unsigned int crypt_len, nonce_len = 0,encrypted_len = 0;
   char sauth[10], iscrypt ='n';
+  int len = 0,keylen;
   unsigned long err;
   RSA *newkey;
 
@@ -2413,6 +2518,9 @@ memset(in,0,CF_BUFSIZE);
 
 if (ReceiveTransaction(conn->sd_reply,in,NULL) == -1)
    {
+   BN_free(counter_challenge);
+   free(out);
+   RSA_free(newkey);   
    return false;
    }
  
@@ -2446,11 +2554,17 @@ memset(in,0,CF_BUFSIZE);
 
 if ((keylen = ReceiveTransaction(conn->sd_reply,in,NULL)) == -1)
    {
+   BN_free(counter_challenge);
+   free(out);
+   RSA_free(newkey); 
    return false;
    }
 
 if (keylen > CF_BUFSIZE/2)
    {
+   BN_free(counter_challenge);
+   free(out);
+   RSA_free(newkey); 
    snprintf(conn->output,CF_BUFSIZE,"Session key length received from %s is too long",conn->ipaddr);
    CfLog(cfinform,conn->output,"");
    return false;
@@ -2472,10 +2586,12 @@ conn->session_key = malloc(CF_BLOWFISHSIZE);
 
 if (conn->session_key == NULL)
    {
+   BN_free(counter_challenge);
+   free(out);
+   RSA_free(newkey); 
    return false;
    }
 
-
 if (keylen == CF_BLOWFISHSIZE) /* Old, non-encrypted */
    {
    memcpy(conn->session_key,in,CF_BLOWFISHSIZE);
@@ -2495,8 +2611,6 @@ else /* New encrypted */
       
       snprintf(conn->output,CF_BUFSIZE,"Private decrypt failed = %s\n",ERR_reason_error_string(err));
       CfLog(cferror,conn->output,"");
-      free(conn->session_key);
-      conn->session_key = NULL;
       return false;
       }
    
@@ -2533,6 +2647,7 @@ int StatFile(struct cfd_connection *conn,char *sendbuffer,char *filename)
 { struct cfstat cfst;
   struct stat statbuf;
   char linkbuf[CF_BUFSIZE];
+  int islink = false;
 
   Debug("StatFile(%s)\n",filename);
 
@@ -2562,6 +2677,7 @@ memset(linkbuf,0,CF_BUFSIZE);
 
 if (S_ISLNK(statbuf.st_mode))
    {
+   islink = true;
    cfst.cf_type = cf_link;                   /* pointless - overwritten */
    cfst.cf_lmode = statbuf.st_mode & 07777;
    cfst.cf_nlink = statbuf.st_nlink;
@@ -2579,7 +2695,7 @@ if (S_ISLNK(statbuf.st_mode))
    cfst.cf_readlink = linkbuf;
    }
 
-if (stat(filename,&statbuf) == -1)
+if (!islink && (stat(filename,&statbuf) == -1))
    {
    snprintf(sendbuffer,CF_BUFSIZE,"BAD: unable to stat file %s\n",filename);
    CfLog(cfverbose,conn->output,"stat");
@@ -2667,7 +2783,7 @@ snprintf(sendbuffer,CF_BUFSIZE,"OK: %d %d %d %d %d %ld %d %d %d %d %d %d %d",
 
 SendTransaction(conn->sd_reply,sendbuffer,0,CF_DONE);
 
- memset(sendbuffer,0,CF_BUFSIZE);
+memset(sendbuffer,0,CF_BUFSIZE);
 
 if (cfst.cf_readlink != NULL)
    {
@@ -2726,7 +2842,7 @@ if (uid != 0 && !args->connect->maproot) /* should remote root be local root */
       }
    }
  
- if (args->buf_size < 512)
+ if (args->buf_size < SMALL_BLOCK_BUF_SIZE)
     {
     snprintf(args->connect->output,CF_BUFSIZE,"blocksize for %s was only %d\n",filename,args->buf_size);
     CfLog(cferror,args->connect->output,"");
@@ -2864,6 +2980,8 @@ void CompareLocalChecksum(struct cfd_connection *conn,char *sendbuffer,char *rec
   char *sp;
   int i;
 
+/* TODO - when safe change this to sha2 */
+  
 sscanf(recvbuffer,"MD5 %[^\n]",filename);
 
 sp = recvbuffer + strlen(recvbuffer) + CF_SMALL_OFFSET;
@@ -2941,6 +3059,61 @@ closedir(dirh);
 return 0;
 }
 
+/**************************************************************/
+
+int CfSecOpenDirectory(struct cfd_connection *conn,char *sendbuffer,char *dirname)
+
+{ DIR *dirh;
+  struct dirent *dirp;
+  int offset,cipherlen;
+  char out[CF_BUFSIZE];
+
+Debug("CfSecOpenDirectory(%s)\n",dirname);
+  
+if (*dirname != '/')
+   {
+   sprintf(sendbuffer,"BAD: request to access a non-absolute filename\n");
+   SendTransaction(conn->sd_reply,sendbuffer,0,CF_DONE);
+   return -1;
+   }
+
+if ((dirh = opendir(dirname)) == NULL)
+   {
+   Debug("cfengine, couldn't open dir %s\n",dirname);
+   snprintf(sendbuffer,CF_BUFSIZE,"BAD: cfengine, couldn't open dir %s\n",dirname);
+   SendTransaction(conn->sd_reply,sendbuffer,0,CF_DONE);
+   return -1;
+   }
+
+/* Pack names for transmission */
+
+memset(sendbuffer,0,CF_BUFSIZE);
+
+offset = 0;
+
+for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
+   {
+   if (strlen(dirp->d_name)+1+offset >= CF_BUFSIZE - CF_MAXLINKSIZE)
+      {
+      cipherlen = EncryptString(sendbuffer,out,CONN->session_key,offset+1);
+      SendTransaction(conn->sd_reply,out,cipherlen,CF_MORE);
+      offset = 0;
+      memset(sendbuffer,0,CF_BUFSIZE);
+      memset(out,0,CF_BUFSIZE);
+      }
+
+   strncpy(sendbuffer+offset,dirp->d_name,CF_MAXLINKSIZE);
+   offset += strlen(dirp->d_name) + 1;     /* + zero byte separator */
+   }
+ 
+strcpy(sendbuffer+offset,CFD_TERMINATOR);
+cipherlen = EncryptString(sendbuffer,out,conn->session_key,offset+2+strlen(CFD_TERMINATOR));
+SendTransaction(conn->sd_reply,out,cipherlen,CF_DONE);
+Debug("END CfSecOpenDirectory(%s)\n",dirname);
+closedir(dirh);
+return 0;
+}
+
 /***************************************************************/
 
 void Terminate(int sd)
@@ -3011,7 +3184,7 @@ else
    }
  
 snprintf(sendbuffer,CF_BUFSIZE,"%s",CF_FAILEDSTR);
-CfLog(cfinform,"Host authorization/authentication failed or access denied\n","");
+CfLog(cfinform,"Unspecified refusal by server\n","");
 SendTransaction(conn->sd_reply,sendbuffer,size,CF_DONE);
 snprintf(sendbuffer,CF_BUFSIZE,"From (host=%s,user=%s,ip=%s)",hostname,username,ipaddr);
 CfLog(cfinform,sendbuffer,"");
@@ -3019,7 +3192,7 @@ CfLog(cfinform,sendbuffer,"");
 if (strlen(errmesg) > 0)
    {
    snprintf(OUTPUT,CF_BUFSIZE,"ID from connecting host: (%s)",errmesg);
-   CfLog(cflogonly,OUTPUT,"");
+   CfLog(cfloginform,OUTPUT,"");
    }
 }
 
@@ -3113,22 +3286,45 @@ if (savedkey = HavePublicKey(keyname))
    
    return true;
    }
- else if ((TRUSTKEYLIST != NULL) && IsFuzzyItemIn(TRUSTKEYLIST,MapAddress(conn->ipaddr)))
-    {
-    Verbose("Host %s/%s was found in the list of hosts to trust\n",conn->hostname,conn->ipaddr);
-    conn->trust = true;
-    /* conn->maproot = false; ?? */
-    SendTransaction(conn->sd_reply,"OK: key was accepted on trust",0,CF_DONE);
-    SavePublicKey(keyname,key);
-    AddToKeyDB(key,MapAddress(conn->ipaddr));
-    return true;
-    }
- else
-    {
-    Verbose("No previous key found, and unable to accept this one on trust\n");
-    SendTransaction(conn->sd_reply,"BAD: key could not be accepted on trust",0,CF_DONE);
-    return false; 
-    }
+else if ((DHCPLIST != NULL) && IsFuzzyItemIn(DHCPLIST,MapAddress(conn->ipaddr)))
+   {
+   /* If the host is expected to have a dynamic address, check for the key */
+   
+   if ((DHCPLIST != NULL) && IsFuzzyItemIn(DHCPLIST,MapAddress(conn->ipaddr)))
+      {
+      int result;
+      result = IsWildKnownHost(savedkey,key,MapAddress(conn->ipaddr),conn->username);
+      RSA_free(savedkey);
+      if (result)
+         {
+         SendTransaction(conn->sd_reply,"OK: key accepted",0,CF_DONE);
+         }
+      else
+         {
+         SendTransaction(conn->sd_reply,"BAD: keys did not match",0,CF_DONE);
+         }
+      return result;
+      }
+   }
+
+/* Finally, if we're still here, we should consider trusting a new key ... */
+
+if ((TRUSTKEYLIST != NULL) && IsFuzzyItemIn(TRUSTKEYLIST,MapAddress(conn->ipaddr)))
+   {
+   Verbose("Host %s/%s was found in the list of hosts to trust\n",conn->hostname,conn->ipaddr);
+   conn->trust = true;
+   /* conn->maproot = false; ?? */
+   SendTransaction(conn->sd_reply,"OK: unknown key was accepted on trust",0,CF_DONE);
+   SavePublicKey(keyname,key);
+   AddToKeyDB(key,MapAddress(conn->ipaddr));
+   return true;
+   }
+else
+   {
+   Verbose("No previous key found, and unable to accept this one on trust\n");
+   SendTransaction(conn->sd_reply,"BAD: key could not be accepted on trust",0,CF_DONE);
+   return false; 
+   }
 }
 
 /***************************************************************/
@@ -3148,7 +3344,7 @@ int IsWildKnownHost(RSA *oldkey,RSA *newkey,char *mipaddr,char *username)
 snprintf(keyname,CF_MAXVARSIZE,"%s-%s",username,mipaddr);
 snprintf(keydb,CF_MAXVARSIZE,"%s/ppkeys/dynamic",CFWORKDIR); 
 
-Debug("The key does not match the known, key but the host has dynamic IP...\n"); 
+Debug("The key does not match a known key but the host could have a dynamic IP...\n"); 
  
 if ((TRUSTKEYLIST != NULL) && IsFuzzyItemIn(TRUSTKEYLIST,mipaddr))
    {
@@ -3157,14 +3353,14 @@ if ((TRUSTKEYLIST != NULL) && IsFuzzyItemIn(TRUSTKEYLIST,mipaddr))
    }
 else
    {
-   Debug("Will not accept the new key, unless we have seen it before\n");
+   Debug("Will not accept this key, unless we have seen it before\n");
    }
 
 /* If the host is allowed to have a variable IP range, we can accept
    the new key on trust for the given IP address provided we have seen
    the key before.  Check for it in a database .. */
 
-Debug("Checking to see if we have seen the new key before..\n"); 
+Debug("Checking to see if we have seen the key before..\n"); 
   
 if ((errno = db_create(&dbp,NULL,0)) != 0)
    {
@@ -3210,10 +3406,12 @@ if ((errno = dbp->get(dbp,NULL,&key,&value,0)) != 0)
          {
          dbp->err(dbp,errno,NULL);
          }
+
+      DeletePublicKey(keyname);
       }
    else
       {
-      Debug("Found no grounds for trusting this new key from %s\n",mipaddr);
+      Debug("Found no grounds for trusting this new from %s\n",mipaddr);
       }
    }
 else
@@ -3340,6 +3538,11 @@ void DeleteConn(struct cfd_connection *conn) /* destruct */
 Debug("***Closing socket %d from %s\n",conn->sd_reply,conn->ipaddr);
 
 close(conn->sd_reply);
+
+if (conn->session_key != NULL)
+   {
+   free(conn->session_key);
+   }
  
 if (conn->ipaddr != NULL)
    {
