@@ -36,11 +36,7 @@
 
 /*******************************************************************/
 
-int CompareCheckSums(file1,file2,ip,sstat,dstat)
-
-char *file1, *file2;
-struct Image *ip;
-struct stat *sstat, *dstat;
+int CompareCheckSums(char *file1,char *file2,struct Image *ip,struct stat *sstat,struct stat *dstat)
 
 { static unsigned char digest1[EVP_MAX_MD_SIZE+1], digest2[EVP_MAX_MD_SIZE+1];
   int i;
@@ -80,17 +76,13 @@ else
 
 /*******************************************************************/
 
-int CompareBinarySums(file1,file2,ip,sstat,dstat)
+int CompareBinarySums(char *file1,char *file2,struct Image *ip,struct stat *sstat,struct stat *dstat)
 
      /* See the md5 algorithms in pub-lib/md5.c */
      /* file 1 is source                        */
 
-char *file1, *file2;
-struct Image *ip;
-struct stat *sstat, *dstat;
-
  { int fd1, fd2,bytes1,bytes2;
-   char	buff1[BUFSIZ],buff2[BUFSIZ];
+   char buff1[BUFSIZ],buff2[BUFSIZ];
 
 Debug("CompareBinarySums(%s,%s)\n",file1,file2);
 
@@ -113,33 +105,30 @@ if (strcmp(ip->server,"localhost") == 0)
       bytes2 = read(fd2, buff2, BUFSIZ);
 
       if ((bytes1 != bytes2) || (memcmp(buff1, buff2, bytes1) != 0))
-	 {
-  	 Verbose("Binary Comparison mismatch...\n");
-  	 close(fd2);
-  	 close(fd1);
-  	 return true;
-  	 }
+         {
+         Verbose("Binary Comparison mismatch...\n");
+         close(fd2);
+         close(fd1);
+         return true;
+         }
       }
    while (bytes1 > 0);
-  
+   
    close(fd2);
    close(fd1);
-
+   
    return false;  /* only if files are identical */
    }
-else
-   {
-   Debug("Using network md5 checksum instead\n");
-   return CompareMD5Net(file1,file2,ip); /* client.c */
-   }
-}
+ else
+    {
+    Debug("Using network md5 checksum instead\n");
+    return CompareMD5Net(file1,file2,ip); /* client.c */
+    }
+ }
 
 /*******************************************************************/
 
-void ChecksumFile(filename,digest,type)
-
-char *filename,type;
-unsigned char digest[EVP_MAX_MD_SIZE+1];
+void ChecksumFile(char *filename,unsigned char digest[EVP_MAX_MD_SIZE+1],char type)
 
 { FILE *file;
   EVP_MD_CTX context;
@@ -155,14 +144,7 @@ if ((file = fopen (filename, "rb")) == NULL)
    }
 else
    {
-   switch (type)
-      {
-      case 's': md = EVP_get_digestbyname("sha");
-  	        break;
-      case 'm': md = EVP_get_digestbyname("md5");
-	        break;
-      default: FatalError("Software failure in ChecksumFile");
-      }
+   md = EVP_get_digestbyname(ChecksumName(type));
    
    EVP_DigestInit(&context,md);
 
@@ -180,11 +162,31 @@ else
 
 /*******************************************************************/
 
-void ChecksumString(buffer,len,digest,type)
+void ChecksumList(struct Item *list,unsigned char digest[EVP_MAX_MD_SIZE+1],char type)
 
-char *buffer,type;
-int len;
-unsigned char digest[EVP_MAX_MD_SIZE+1];
+{ struct Item *ip;
+  EVP_MD_CTX context;
+  int md_len;
+  const EVP_MD *md = NULL;
+
+Debug2("ChecksumList(%c)\n",type);
+
+md = EVP_get_digestbyname(ChecksumName(type));
+
+EVP_DigestInit(&context,md);
+
+for (ip = list; ip != NULL; ip=ip->next) 
+   {
+   Debug(" digesting %s\n",ip->name);
+   EVP_DigestUpdate(&context,ip->name,strlen(ip->name));
+   }
+
+EVP_DigestFinal(&context,digest,&md_len);
+}
+
+/*******************************************************************/
+
+void ChecksumString(char *buffer,int len,unsigned char digest[EVP_MAX_MD_SIZE+1],char type)
 
 { EVP_MD_CTX context;
   const EVP_MD *md = NULL;
@@ -192,15 +194,8 @@ unsigned char digest[EVP_MAX_MD_SIZE+1];
 
 Debug2("ChecksumString(%c)\n",type);
 
-switch (type)
-   {
-   case 's': md = EVP_get_digestbyname("sha");
-       break;
-   case 'm': md = EVP_get_digestbyname("md5");
-       break;
-   default: FatalError("Software failure in ChecksumFile");
-   }
- 
+md = EVP_get_digestbyname(ChecksumName(type));
+
 EVP_DigestInit(&context,md); 
 EVP_DigestUpdate(&context,(unsigned char*)buffer,len);
 EVP_DigestFinal(&context,digest,&md_len);
@@ -208,20 +203,12 @@ EVP_DigestFinal(&context,digest,&md_len);
 
 /*******************************************************************/
 
-int ChecksumsMatch(digest1,digest2,type)
+int ChecksumsMatch(unsigned char digest1[EVP_MAX_MD_SIZE+1],unsigned char digest2[EVP_MAX_MD_SIZE+1],char type)
 
-unsigned char digest1[EVP_MAX_MD_SIZE+1],digest2[EVP_MAX_MD_SIZE+1];
-char type;
 
 { int i,size = EVP_MAX_MD_SIZE;
  
-switch(type)  /* Size of message digests */
-   {
-   case 'm': size = CF_MD5_LEN;
-       break;
-   case 's': size = CF_SHA_LEN;
-       break;
-   }
+size = ChecksumSize(type);
 
 for (i = 0; i < size; i++)
    {
@@ -233,3 +220,87 @@ for (i = 0; i < size; i++)
 
 return true;
 }
+
+/*********************************************************************/
+
+char *ChecksumPrint(char type,unsigned char digest[EVP_MAX_MD_SIZE+1])
+
+{ unsigned int i;
+  static char buffer[EVP_MAX_MD_SIZE*4];
+  int len = 16;
+
+switch(type)
+   {
+   case 's': sprintf(buffer,"SHA=  ");
+       len = 20;
+       break;
+   case 'm': sprintf(buffer,"MD5=  ");
+       len = 16;
+       break;
+   }
+  
+for (i = 0; i < len; i++)
+   {
+   sprintf((char *)(buffer+4+2*i),"%02x", digest[i]);
+   }
+
+return buffer; 
+}    
+
+/*********************************************************************/
+
+char ChecksumType(char *typestr)
+
+{ int i;
+
+for (i = 0; CF_DIGEST_TYPES[i][0] != NULL; i++)
+   {
+   if (strcmp(typestr,CF_DIGEST_TYPES[i][0]) == 0)
+      {
+      return *CF_DIGEST_TYPES[i][1];
+      }
+   }
+
+if (PARSING)
+   {
+   snprintf(OUTPUT,CF_BUFSIZE,"Illegal checksum/digest type: %s",typestr);
+   yyerror(OUTPUT);
+   FatalError("Should be md5,sha,sha1,sha224,sha256,sha348,sha512");
+   }
+
+return 'x';
+}
+
+/*********************************************************************/
+
+char *ChecksumName(char type)
+
+{ int i;
+
+for (i = 0; CF_DIGEST_TYPES[i][0] != NULL; i++)
+   {
+   if (type == *CF_DIGEST_TYPES[i][1])
+      {
+      return CF_DIGEST_TYPES[i][0];
+      }
+   }
+
+return NULL;
+}
+
+/*********************************************************************/
+
+int ChecksumSize(char type)
+
+{ int i,size = 0;
+ 
+for (i = 0; CF_DIGEST_TYPES[i][0] != NULL; i++)
+   {
+   if (type == *CF_DIGEST_TYPES[i][1])
+      {
+      return CF_DIGEST_SIZES[i];
+      }
+   }
+
+return size;
+}
