@@ -93,6 +93,7 @@ if (CONN->sd == CF_NOT_CONNECTED)
       return false;
       }
 
+   /*CheckRemoteVersion();*/
    AUTHENTICATED = true;
    return true;
    }
@@ -101,7 +102,10 @@ else
    Debug("Server connection to %s already open on %d\n",server,CONN->sd);
    }
 
-AUTHENTICATED = true; 
+AUTHENTICATED = true;
+
+/*CheckRemoteVersion();*/
+
 return true; 
 }
 
@@ -137,7 +141,7 @@ int cf_rstat(char *file,struct stat *buf,struct Image *ip,char *stattype)
   int ret,tosend,cipherlen;
   time_t tloc;
 
-Debug("cf_rstat(%s)\n",file);
+Debug("cf_rstat(%s,%s)\n",file,stattype);
 memset(recvbuffer,0,CF_BUFSIZE); 
 
 if (strlen(file) > CF_BUFSIZE-30)
@@ -327,7 +331,9 @@ CFDIR *cf_ropendir(char *dirname,struct Image *ip)
 
 { char sendbuffer[CF_BUFSIZE];
   char recvbuffer[CF_BUFSIZE];
-  int n, done=false;
+  char in[CF_BUFSIZE];
+  char out[CF_BUFSIZE];
+  int n, done=false, cipherlen = 0,plainlen = 0,tosend;
   CFDIR *cfdirh;
   char *sp;
 
@@ -349,10 +355,29 @@ cfdirh->cf_list = NULL;
 cfdirh->cf_listpos = NULL;
 cfdirh->cf_dirh = NULL;
 
-snprintf(sendbuffer,CF_BUFSIZE,"OPENDIR %s",dirname);
-
-if (SendTransaction(CONN->sd,sendbuffer,0,CF_DONE) == -1)
+if (ip->encrypt == 'y' && FULLENCRYPT)
    {
+   if (CONN->session_key == NULL)
+      {
+      CfLog(cferror,"Cannot do encrypted copy without keys (use cfkey)","");
+      return NULL;
+      }
+   
+   snprintf(in,CF_BUFSIZE,"OPENDIR %s",dirname);
+   cipherlen = EncryptString(in,out,CONN->session_key,strlen(in)+1);
+   snprintf(sendbuffer,CF_BUFSIZE-1,"SOPENDIR %d",cipherlen);
+   memcpy(sendbuffer+CF_PROTO_OFFSET,out,cipherlen);
+   tosend = cipherlen+CF_PROTO_OFFSET;
+   }
+else
+   {
+   snprintf(sendbuffer,CF_BUFSIZE,"OPENDIR %s",dirname);
+   tosend = strlen(sendbuffer);
+   }
+
+if (SendTransaction(CONN->sd,sendbuffer,tosend,CF_DONE) == -1)
+   {
+   free((char *)cfdirh);
    return NULL;
    }
 
@@ -364,7 +389,9 @@ while (!done)
          {
          continue;
          }
-      return false;
+
+      free((char *)cfdirh);
+      return NULL;
       }
 
    if (n == 0)
@@ -372,28 +399,38 @@ while (!done)
       break;
       }
 
+   if (ip->encrypt == 'y' && FULLENCRYPT)
+      {
+      memcpy(in,recvbuffer,n);
+      plainlen = DecryptString(in,recvbuffer,CONN->session_key,n);
+      }
+
    if (FailedProtoReply(recvbuffer))
       {
       snprintf(OUTPUT,CF_BUFSIZE*2,"Network access to %s:%s denied\n",ip->server,dirname);
       CfLog(cfinform,OUTPUT,"");
-      return false;      
+
+      free((char *)cfdirh);
+      return NULL;      
       }
 
    if (BadProtoReply(recvbuffer))
       {
       snprintf(OUTPUT,CF_BUFSIZE*2,"%s\n",recvbuffer+4);
       CfLog(cfinform,OUTPUT,"");
-      return false;      
+
+      free((char *)cfdirh);
+      return NULL;      
       }
 
    for (sp = recvbuffer; *sp != '\0'; sp++)
       {
-      if (strncmp(sp,CFD_TERMINATOR,strlen(CFD_TERMINATOR)) == 0)        /* End transmission */
+      if (strncmp(sp,CFD_TERMINATOR,strlen(CFD_TERMINATOR)) == 0)    /* End transmission */
          {
          cfdirh->cf_listpos = cfdirh->cf_list;
          return cfdirh;
          }
-      
+
       AppendItem(&(cfdirh->cf_list),sp,NULL);
       
       while(*sp != '\0')
