@@ -75,20 +75,20 @@ for (ip = VMOUNTLIST; ip != NULL; ip=ip->next)
          {
          continue;
          }
-
+      
       strcpy(VBUFF,ip->name);
       AddSlash(VBUFF);
       strcat(VBUFF,dirp->d_name);
-
+      
       if (IsHomeDir(VBUFF))
          {
          snprintf(OUTPUT,CF_BUFSIZE*2,"Host defines a home directory %s\n",VBUFF);
-  CfLog(cfverbose,OUTPUT,"");
+         CfLog(cfverbose,OUTPUT,"");
          }
       else
          {
          snprintf(OUTPUT,CF_BUFSIZE*2,"Host defines a potential mount point %s\n",VBUFF);
-  CfLog(cfverbose,OUTPUT,"");
+         CfLog(cfverbose,OUTPUT,"");
          }
 
       snprintf(path,CF_BUFSIZE,"%s%s",ip->name,dirp->d_name);
@@ -99,7 +99,7 @@ for (ip = VMOUNTLIST; ip != NULL; ip=ip->next)
          if ( MOUNTCHECK && ! RequiredFileSystemOkay(path) && VERBOSE)
             {
             snprintf(OUTPUT,CF_BUFSIZE*2,"Found a mountpoint %s but there was\n",path);
-     CfLog(cfinform,OUTPUT,"");
+            CfLog(cfinform,OUTPUT,"");
             CfLog(cfinform,"nothing mounted on it.\n\n","");
             }
          }
@@ -283,8 +283,8 @@ do
       case cfsco: CfLog(cferror,"Don't understand SCO mount format, no data","");
 
       default:
-                    printf("cfengine software error: case %d = %s\n",VSYSTEMHARDCLASS,CLASSTEXT[VSYSTEMHARDCLASS]);
-                    FatalError("System error in GetMountInfo - no such class!");
+          printf("cfengine software error: case %d = %s\n",VSYSTEMHARDCLASS,CLASSTEXT[VSYSTEMHARDCLASS]);
+          FatalError("System error in GetMountInfo - no such class!");
       }
 
    InstallMountedItem(host,mounton);
@@ -433,8 +433,8 @@ for (lp = VCHLINK; lp != NULL; lp = lp->next)
          if (stat(from,&statbuf) == -1)
             {
             snprintf(OUTPUT,CF_BUFSIZE*2,"Makechildlinks() can't stat %s\n",from);
-     CfLog(cferror,OUTPUT,"stat");
-     ResetOutputRoute('d','d');
+            CfLog(cferror,OUTPUT,"stat");
+            ResetOutputRoute('d','d');
             continue;
             }
          LinkChildren(from,lp->type,&statbuf,0,0,lp->inclusions,lp->exclusions,lp->copy,lp->nofile,lp);
@@ -490,7 +490,7 @@ void MakeLinks()     /* <binserver> should expand to a best fit filesys */
   int matched,varstring;
   short saveenforce;
   short savesilent;
-  int (*linkfiles) ARGLIST((char *from, char *to, struct Item *inclusions, struct Item *exclusions, struct Item *copy, short int nofile, struct Link *ptr));
+  int (*linkfiles)(char *from, char *to, struct Item *inclusions, struct Item *exclusions, struct Item *copy, short int nofile, struct Link *ptr);
 
 if (NOLINKS)
    {
@@ -556,7 +556,7 @@ for (lp = VLINK; lp != NULL; lp = lp->next)
 
    matched = varstring = false;
 
-   for( ip = VBINSERVERS; ip != NULL && (!matched); ip = ip->next)
+   for(ip = VBINSERVERS; ip != NULL && (!matched); ip = ip->next)
       {
       path[0] = '\0';
 
@@ -569,7 +569,7 @@ for (lp = VLINK; lp != NULL; lp = lp->next)
       else if (! varstring)
          {
          snprintf(OUTPUT,CF_BUFSIZE*2,"Error while trying to link %s -> %s\n",from,path);
-  CfLog(cfinform,OUTPUT,"");
+         CfLog(cfinform,OUTPUT,"");
          }
 
       if (! varstring)                       /* don't iterate over binservers if not var */
@@ -993,7 +993,7 @@ for (rp = VREQUIRED; rp != NULL; rp = rp->next)
 
 if (missing)
    {
-   time_t tloc;;
+   time_t tloc;
 
    if ((tloc = time((time_t *)NULL)) == -1)
       {
@@ -1876,6 +1876,7 @@ void MountHomeBinServers()
   char host[CF_MAXVARSIZE];
   char mountdir[CF_BUFSIZE];
   char maketo[CF_BUFSIZE];
+  struct Item *ip;
 
   /*
    * HvB: Bas van der Vlies
@@ -1898,6 +1899,8 @@ if (!IsPrivileged())
 
 Banner("Checking home and binservers");
 
+Debug("BINSERVER = %s\n",VDEFAULTBINSERVER.name);
+
 for (mp = VMOUNTABLES; mp != NULL; mp=mp->next)
    {
    sscanf(mp->filesystem,"%[^:]:%s",host,mountdir);
@@ -1924,6 +1927,8 @@ for (mp = VMOUNTABLES; mp != NULL; mp=mp->next)
       strcat(maketo,"/.");
       }
 
+   Debug("I am [%s], you are [%s]\n",host,VDEFAULTBINSERVER.name);
+
    if (strcmp(host,VDEFAULTBINSERVER.name) == 0) /* A host never mounts itself nfs */
       {
       Debug("Skipping host %s\n",host);
@@ -1931,7 +1936,7 @@ for (mp = VMOUNTABLES; mp != NULL; mp=mp->next)
       }
 
    /* HvB: Bas van der Vlies */
-   if ( mp->readonly ) 
+   if (mp->readonly) 
       {
       strcpy(mountmode, "ro");
       }
@@ -2028,11 +2033,11 @@ for (mp = VMISCMOUNT; mp != NULL; mp=mp->next)
    if (!IsItemIn(VMOUNTED,mtpt))
       {
       MakeDirectoriesFor(maketo,'n');
-      AddToFstab(host,mountdir,mp->onto,mp->options,NULL,false);
+      AddToFstab(host,mountdir,mp->onto,mp->mode,mp->options,false);
       }
    else
       {
-      AddToFstab(host,mountdir,mp->onto,mp->options,NULL,true);
+      AddToFstab(host,mountdir,mp->onto,mp->mode,mp->options,true);
       }
    }
 }
@@ -2285,7 +2290,7 @@ void CheckResolv()
 
 Verbose("Checking config in %s\n",VRESOLVCONF[VSYSTEMHARDCLASS]);
   
-if (VDOMAIN == NULL)
+if (strcmp(VDOMAIN,"") == 0)
    {
    CfLog(cferror,"Domain name not specified. Can't configure resolver\n","");
    return;
@@ -2407,9 +2412,17 @@ for (svp = VSERVERLIST; svp != NULL; svp=svp->next) /* order servers */
    for (ip = VIMAGE; ip != NULL; ip=ip->next)
       {
       ExpandVarstring(ip->server,server,NULL);            
+      AddMacroValue(CONTEXTID,"this",server);
       ExpandVarstring(ip->path,path,NULL);
       ExpandVarstring(ip->destination,destination,NULL);
+      DeleteMacro(CONTEXTID,"this");
 
+      if (strcmp(server,"none") == 0)
+         {
+         Verbose("Server none is a no-op\n");
+         continue;
+         }
+      
       if (strcmp(listserver,server) != 0)  /* group together similar hosts so */
          {                                /* can can do multiple transactions */
          continue;                        /* on one connection */
@@ -2464,7 +2477,7 @@ for (svp = VSERVERLIST; svp != NULL; svp=svp->next) /* order servers */
          continue;
          }
       
-      snprintf(vbuff,CF_BUFSIZE,"%.50s.%.50s_%.50s",path,destination,server); /* Unique ID for copy locking */
+      snprintf(vbuff,CF_BUFSIZE,"%.255s.%.50s_%.50s",path,destination,server); /* Unique ID for copy locking */
       
       if (!GetLock(ASUniqueName("copy"),CanonifyName(vbuff),ip->ifelapsed,ip->expireafter,VUQNAME,CFSTARTTIME))
          {
@@ -2676,12 +2689,18 @@ void CheckPackages()
 { struct Package *ptr;
   int match = 0;
   int i;
+  char lock[CF_BUFSIZE];
   /* pkgmgr_none will always be the highest number in the enum so set
      the array size with that */
   char *package_install_list[pkgmgr_none] = { NULL };
 
 for (ptr = VPKG; ptr != NULL; ptr=ptr->next)
    {
+   if (IsExcluded(ptr->classes))
+      {
+      continue;
+      }
+
    if (ptr->done == 'y' || strcmp(ptr->scope,CONTEXTID) != 0)
       {
       continue;
@@ -2691,12 +2710,9 @@ for (ptr = VPKG; ptr != NULL; ptr=ptr->next)
       ptr->done = 'y';
       }
 
-   if (IsExcluded(ptr->classes))
-      {
-      continue;
-      }
-
-   if (!GetLock(ASUniqueName("packages"),CanonifyName(ptr->name),ptr->ifelapsed,ptr->expireafter,VUQNAME,CFSTARTTIME))
+   snprintf(lock,CF_BUFSIZE-1,"%s_%d_%d",ptr->name,ptr->cmp,ptr->action);
+   
+   if (!GetLock(ASUniqueName("packages"),CanonifyName(lock),ptr->ifelapsed,ptr->expireafter,VUQNAME,CFSTARTTIME))
       {
       ptr->done = 'y';
       continue;
@@ -2713,6 +2729,12 @@ for (ptr = VPKG; ptr != NULL; ptr=ptr->next)
      case pkgmgr_sun:
        match = SUNPackageCheck(ptr->name, ptr->ver, ptr->cmp);
        break;
+     case pkgmgr_aix:
+       match = AIXPackageCheck(ptr->name, ptr->ver, ptr->cmp);
+       break;
+     case pkgmgr_portage:
+       match = PortagePackageCheck(ptr->name, ptr->ver, ptr->cmp);
+       break;
      default:
        /* UGH!  This should *never* happen.  GetPkgMgr() and
         * InstallPackagesItem() should have caught this before it
@@ -2811,7 +2833,7 @@ for (ptr = VMETHODS; ptr != NULL; ptr=ptr->next)
       }
 
    ChecksumList(ptr->send_args,digest,'m');
-   snprintf(label,CF_BUFSIZE-1,"%s/rpc_in/localhost_localhost_%s_%s",VLOCKDIR,ptr->name,ChecksumPrint('m',digest));
+   snprintf(label,CF_BUFSIZE-1,"%s/rpc_in/localhost+localhost+%s+%s",VLOCKDIR,ptr->name,ChecksumPrint('m',digest));
 
    if (!GetLock(ASUniqueName("methods-dispatch"),CanonifyName(label),ptr->ifelapsed,ptr->expireafter,VUQNAME,CFSTARTTIME))
       {
@@ -3055,17 +3077,21 @@ if (IsItemIn(VMOUNTED,buf))
 
 /*******************************************************************/
 
-
 void AddToFstab(char *host,char *rmountpt,char *mountpt,char *mode,char *options,int ismounted)
 
 { char fstab[CF_BUFSIZE];
   char *opts;
   FILE *fp;
+  char aix_lsnfsmnt[CF_BUFSIZE];
 
 Debug("AddToFstab(%s)\n",mountpt);
 
+if (mode == NULL)
+   {
+   mode = "rw";
+   }
 
-if ( options != NULL)
+if ((options != NULL) && (strlen(options) > 0))
    {
    opts = options;
    }
@@ -3096,7 +3122,11 @@ switch (VSYSTEMHARDCLASS)
                  break;
    case hp:      snprintf(fstab,CF_BUFSIZE,"%s:%s %s \t %s \t %s,%s 0 0",host,rmountpt,mountpt,VNFSTYPE,mode,opts);
                  break;
-   case aix:     snprintf(fstab,CF_BUFSIZE,"%s:\n\tdev\t= %s\n\ttype\t= %s\n\tvfs\t= %s\n\tnodename\t= %s\n\tmount\t= true\n\toptions\t= %s,%s\n\taccount\t= false\n\n",mountpt,rmountpt,VNFSTYPE,VNFSTYPE,host,mode,opts);
+   case aix:     snprintf(fstab,CF_BUFSIZE,"%s:\n\tdev\t= %s\n\ttype\t= %s\n\tvfs\t= %s\n\tnodename\t= %s\n\tmount\t= true\n\toptions\t= %s,%s\n\taccount\t= false\n",mountpt,rmountpt,VNFSTYPE,VNFSTYPE,host,mode,opts);
+		 snprintf(aix_lsnfsmnt, CF_BUFSIZE, 
+				 "%s:%s:%s:%s:%s",
+				 mountpt, rmountpt, host, VNFSTYPE, mode
+			 );
                  break;
    case GnU:
    case linuxx:  snprintf(fstab,CF_BUFSIZE,"%s:%s \t %s \t %s \t %s,%s",host,rmountpt,mountpt,VNFSTYPE,mode,opts);
@@ -3126,60 +3156,151 @@ switch (VSYSTEMHARDCLASS)
 
 if (MatchStringInFstab(mountpt))
    {
-   /* if the fstab entry has changed, remove the old entry and update */
-   if (!MatchStringInFstab(fstab))
-      { struct UnMount *saved_VUNMOUNT = VUNMOUNT;
-        char mountspec[MAXPATHLEN];
-        struct Item *mntentry = NULL;
-        struct UnMount cleaner;
-      
-      snprintf(OUTPUT,CF_BUFSIZE*2,"Removing \"%s\" entry from %s to allow update:\n",mountpt,VFSTAB[VSYSTEMHARDCLASS]);
-      CfLog(cfinform,OUTPUT,"");
-      CfLog(cfinform,"---------------------------------------------------","");
-      
-      /* delete current fstab entry and unmount if necessary */
-      snprintf(mountspec,CF_BUFSIZE,".+:%s",mountpt);
-      mntentry = LocateItemContainingRegExp(VMOUNTED,mountspec);
-      if (mntentry)
-         {
-         sscanf(mntentry->name,"%[^:]:",mountspec);  /* extract current host */
-         strcat(mountspec,":");
-         strcat(mountspec,mountpt);
-         }
-      else  /* mountpt isn't mounted, so Unmount can use dummy host name */
-          snprintf(mountspec,CF_BUFSIZE,"host:%s",mountpt);
-      
-      /* delete current fstab entry and unmount if necessary (don't rmdir) */
-      cleaner.name        = mountspec;
-      cleaner.classes     = NULL;
-      cleaner.deletedir   = 'n';
-      cleaner.deletefstab = 'y';
-      cleaner.force       = 'n';
-      cleaner.done        = 'n';
-      cleaner.scope       = CONTEXTID;
-      cleaner.next        = NULL;
-      
-      VUNMOUNT = &cleaner;
-      Unmount();
-      VUNMOUNT = saved_VUNMOUNT;
-      CfLog(cfinform,"---------------------------------------------------","");
-      }
-   else /* no need to update fstab - this mount entry is already there */
+   if (VSYSTEMHARDCLASS == aix) /* This AIX code by Graham Bevan */
       {
-      /* warn if entry's already in the fstab but hasn't been mounted */
-      if (!ismounted && !SILENT && !strstr(mountpt,"cdrom"))
+      FILE *pp;
+      int fs_found = 0;
+      int fs_changed = 0;
+      char comm[CF_BUFSIZE];
+
+      if ((pp = cfpopen("/usr/sbin/lsnfsmnt -c", "r")) == NULL)
          {
-         snprintf(OUTPUT,CF_BUFSIZE*2,"Warning the file system %s seems to be in %s\n",mountpt,VFSTAB[VSYSTEMHARDCLASS]);
-         CfLog(cfinform,OUTPUT,"");
-         snprintf(OUTPUT,CF_BUFSIZE*2,"already, but I was not able to mount it.\n");
-         CfLog(cfinform,OUTPUT,"");
-         snprintf(OUTPUT,CF_BUFSIZE*2,"Check the exports file on host %s? Check for file with same name as dir?\n",host);
-         CfLog(cfinform,OUTPUT,"");
+         CfLog(cferror,"Failed to open pipe to /usr/sbin/lsnfsmnt command.", "");
+         return;
          }
       
-      return;
+      while(!feof(pp))
+         {
+         ReadLine(VBUFF, CF_BUFSIZE, pp);
+         if (VBUFF[0] == '#')
+            {
+            continue;
+            }
+
+         if (strstr(VBUFF,mountpt))
+            {
+            fs_found++;
+            if (!strstr(VBUFF,aix_lsnfsmnt))
+               {
+               fs_changed = 1;
+               }
+            }
+         }
+      fclose(pp);
+      
+      if (fs_found == 1 && !fs_changed)
+         {
+         return;
+         }
+      else
+         {
+         /* if entry not found, duplicates found or entry is different from lookup string
+          * then remove for re-add */
+         int failed = 0;
+         snprintf(OUTPUT,CF_BUFSIZE*2,"Removing \"%s\" entry from %s to allow update (fs_found=%d):\n",
+                  mountpt,
+                  VFSTAB[VSYSTEMHARDCLASS],
+                  fs_found
+                  );
+
+         CfLog(cfinform,OUTPUT,"");
+         CfLog(cfinform,"---------------------------------------------------","");
+
+         snprintf(comm, CF_BUFSIZE, "/usr/sbin/rmnfsmnt -f %s", mountpt);
+
+         if ((pp = cfpopen(comm,"r")) == NULL)
+            {
+            CfLog(cferror,"Failed to open pipe to /usr/sbin/rmnfsmnt command.", "");
+            return;
+            }
+         
+         while(!feof(pp))
+            {
+            ReadLine(VBUFF, CF_BUFSIZE, pp);
+            if (VBUFF[0] == '#')
+               {
+               continue;
+               }
+            if (strstr(VBUFF,"busy"))
+               {
+               snprintf(OUTPUT,CF_BUFSIZE*2,"umount warned that the device under %s\n",mountpt);
+               CfLog(cfinform,OUTPUT,"");
+               CfLog(cfinform,"was busy. Cannot unmount (rmnfsmnt) that device.\n","");
+               failed = 1;
+               }
+            }
+         
+         if (fclose(pp) != 0)
+            {
+            snprintf(OUTPUT,CF_BUFSIZE*2,"rmnfsmnt failed on fclose() for %s: %s\n",mountpt, strerror(errno));
+            CfLog(cferror,OUTPUT,"");
+            return;
+            }
+         
+         if (failed)
+            {
+            return;
+            }
+         }
+      
+      } /* if aix */
+   else
+      { 
+      /* if the fstab entry has changed, remove the old entry and update */
+      if (!MatchStringInFstab(fstab))
+         { struct UnMount *saved_VUNMOUNT = VUNMOUNT;
+           char mountspec[MAXPATHLEN];
+           struct Item *mntentry = NULL;
+           struct UnMount cleaner;
+         
+         snprintf(OUTPUT,CF_BUFSIZE*2,"Removing \"%s\" entry from %s to allow update:\n",mountpt,VFSTAB[VSYSTEMHARDCLASS]);
+         CfLog(cfinform,OUTPUT,"");
+         CfLog(cfinform,"---------------------------------------------------","");
+         
+         /* delete current fstab entry and unmount if necessary */
+         snprintf(mountspec,CF_BUFSIZE,".+:%s",mountpt);
+         mntentry = LocateItemContainingRegExp(VMOUNTED,mountspec);
+         if (mntentry)
+            {
+            sscanf(mntentry->name,"%[^:]:",mountspec);  /* extract current host */
+            strcat(mountspec,":");
+            strcat(mountspec,mountpt);
+            }
+         else  /* mountpt isn't mounted, so Unmount can use dummy host name */
+             snprintf(mountspec,CF_BUFSIZE,"host:%s",mountpt);
+         
+         /* delete current fstab entry and unmount if necessary (don't rmdir) */
+         cleaner.name        = mountspec;
+         cleaner.classes     = NULL;
+         cleaner.deletedir   = 'n';
+         cleaner.deletefstab = 'y';
+         cleaner.force       = 'n';
+         cleaner.done        = 'n';
+         cleaner.scope       = CONTEXTID;
+         cleaner.next        = NULL;
+         
+         VUNMOUNT = &cleaner;
+         Unmount();
+         VUNMOUNT = saved_VUNMOUNT;
+         CfLog(cfinform,"---------------------------------------------------","");
+         }
+      else /* no need to update fstab - this mount entry is already there */
+         {
+         /* warn if entry's already in the fstab but hasn't been mounted */
+         if (!ismounted && !SILENT && !strstr(mountpt,"cdrom"))
+            {
+            snprintf(OUTPUT,CF_BUFSIZE*2,"Warning the file system %s seems to be in %s\n",mountpt,VFSTAB[VSYSTEMHARDCLASS]);
+            CfLog(cfinform,OUTPUT,"");
+            snprintf(OUTPUT,CF_BUFSIZE*2,"already, but I was not able to mount it.\n");
+            CfLog(cfinform,OUTPUT,"");
+            snprintf(OUTPUT,CF_BUFSIZE*2,"Check the exports file on host %s? Check for file with same name as dir?\n",host);
+            CfLog(cfinform,OUTPUT,"");
+            }
+         
+         return;
+         }
       }
-   }
+   } /* if aix */
  
  if (DONTDO)
     {
@@ -3188,20 +3309,28 @@ if (MatchStringInFstab(mountpt))
     }
  else
     {
-    if ((fp = fopen(VFSTAB[VSYSTEMHARDCLASS],"a")) == NULL)
+    struct Item *filelist = NULL;
+
+    if (! LoadItemList(&filelist,VFSTAB[VSYSTEMHARDCLASS]))
        {
-       snprintf(OUTPUT,CF_BUFSIZE*2,"Can't open %s for appending\n",VFSTAB[VSYSTEMHARDCLASS]);
-       CfLog(cferror,OUTPUT,"fopen");
-       ReleaseCurrentLock();
+       snprintf(OUTPUT,CF_BUFSIZE*2,"Couldn't open %s!\n",VFSTAB[VSYSTEMHARDCLASS]);
+       CfLog(cferror,OUTPUT,"");
        return;
        }
     
+    NUMBEROFEDITS = 0;
+    
     snprintf(OUTPUT,CF_BUFSIZE*2,"Adding filesystem to %s\n",VFSTAB[VSYSTEMHARDCLASS]);
     CfLog(cfinform,OUTPUT,"");
     snprintf(OUTPUT,CF_BUFSIZE*2,"%s\n",fstab);
     CfLog(cfinform,OUTPUT,"");
-    fprintf(fp,"%s\n",fstab);
-    fclose(fp);
+
+    if (!IsItemIn(filelist,fstab))
+       {
+       AppendItem(&filelist,fstab,NULL);
+       }
+
+    SaveItemList(filelist,VFSTAB[VSYSTEMHARDCLASS],VREPOSITORY);
     
     chmod(VFSTAB[VSYSTEMHARDCLASS],DEFAULTSYSTEMMODE);
     }
@@ -3634,4 +3763,4 @@ fclose(fp);
 return(false);
 }
 
-
+/* vim:expandtab:smarttab:sw=3 */
