@@ -37,9 +37,7 @@
 /* Parser                                                          */
 /*******************************************************************/
 
-void InstallFilter(filter)
-
-char *filter;
+void InstallFilter(char *filter)
 
 { struct Filter *ptr;
   int i;
@@ -96,14 +94,17 @@ VFILTERLISTTOP = ptr;
 
 /********************************************************************/
 
-void InstallFilterTest(alias,type,data)
+void InstallFilterTest(char *alias,char *type,char *data)
 
-char *alias,*type,*data;
-
-{ int crit = (int) FilterActionsToCode(type), i = -1;
+{ int crit, i = -1;
   struct Filter *fp;
-  char buffer[bufsize],units='x',*sp;
+  char ebuff[CF_EXPANDSIZE],vbuff[CF_BUFSIZE],units='x',*sp;
   time_t now;
+
+if (strlen(type) == 0)
+   {
+   return;
+   }
   
 Debug("InstallFilterTest(%s,%s,%s)\n",alias,type,data);
 
@@ -111,106 +112,108 @@ if (time(&now) == -1)
    {
    FatalError("Unable to access clock");
    }
+
+crit = (int) FilterActionsToCode(type); 
  
-buffer[0] = '\0'; 
-ExpandVarstring(data,buffer,NULL);
+ExpandVarstring(data,ebuff,NULL);
  
 if (crit == NoFilter)
    {
-   yyerror("Unknown filter criterion");
+   snprintf(OUTPUT,CF_BUFSIZE,"Unknown filter criterion (%s)\n",type);
+   yyerror(OUTPUT);
    }
 else
    {
    for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
       {
       if (strcmp(alias,fp->alias) == 0)
-	 {
+         {
          switch (crit)
-	    {
-	    case filterdefclasses:
-		fp->defines = strdup(buffer);
-		break;
-	    case filterelsedef:
-		fp->elsedef = strdup(buffer);
-		break;
-	    case filterresult:
-		IsInstallable(buffer); /* syntax check */
-		break;
-
-	    case filterfromctime:
-		now = Date2Number(buffer,now);
-		break;
-	    case filtertoctime:
-		now = Date2Number(buffer,now);
-		break;
-	    case filterfrommtime:
-		now = Date2Number(buffer,now);
-		break;
-	    case filtertomtime:
-		now = Date2Number(buffer,now);
-		break;
-	    case filterfromatime:
-		now = Date2Number(buffer,now);
-		break;
-	    case filtertoatime:
-		now = Date2Number(buffer,now);
-		break;
-	    case filterfromsize:
-	    case filtertosize:
-		sscanf(buffer,"%d%c",&i,&units);
-
-		if (i < 0)
-		   {
-		   yyerror("filter size attribute with silly value (must be a non-negative number)");
-		   }
-		
-		switch (units)
-		   {
-		   case 'k':
-		   case 'K': i *= 1024;
-		       break;
-		   case 'm':
-		   case 'M': i = i * 1024 * 1024;
-		       break;
-		   }
-
-		sprintf(buffer,"%d",i);
-		break;
-	    case filterexecregex:
-		for (sp = buffer+strlen(buffer)-1; (*sp != '(') && (sp > buffer); sp--)
-		   {
-		   }
-		
-		sscanf(sp+1,"%256[^)]",VBUFF);
-		if (!RegexOK(VBUFF))
-		   {
-		   yyerror("Regular expression error");
-		   }
-		break;
-		
-	    case filternameregex:
-		if (!RegexOK(buffer))
-		   {
-		   yyerror("Regular expression error");
-		   }
-		break;
-	    case filterexec:
-		/* test here */
-		break;
-	    case filtersymlinkto:
-		break;
-	    }
-	 
-	 if ((fp->criteria[crit] = strdup(buffer)) == NULL)
-	    {
-	    CfLog(cferror,"Couldn't allocate filter memory","strdup");
-	    FatalError("Dying..");
-	    }
-	 else
-	    {
-	    return;
-	    }
-	 }
+            {
+            case filterdefclasses:
+                fp->defines = strdup(ebuff);
+                break;
+            case filterelsedef:
+                fp->elsedef = strdup(ebuff);
+                break;
+            case filterresult:
+                IsInstallable(ebuff); /* syntax check */
+                break;
+                
+            case filterfromctime:
+                now = Date2Number(ebuff,now);
+                break;
+            case filtertoctime:
+                now = Date2Number(ebuff,now);
+                break;
+            case filterfrommtime:
+                now = Date2Number(ebuff,now);
+                break;
+            case filtertomtime:
+                now = Date2Number(ebuff,now);
+                break;
+            case filterfromatime:
+                now = Date2Number(ebuff,now);
+                break;
+            case filtertoatime:
+                now = Date2Number(ebuff,now);
+                break;
+            case filterfromsize:
+            case filtertosize:
+                sscanf(ebuff,"%d%c",&i,&units);
+                
+                if (i < 0)
+                   {
+                   yyerror("filter size attribute with silly value (must be a non-negative number)");
+                   }
+                
+                switch (units)
+                   {
+                   case 'k':
+                   case 'K': i *= 1024;
+                       break;
+                   case 'm':
+                   case 'M': i = i * 1024 * 1024;
+                       break;
+                   }
+                
+                sprintf(ebuff,"%d",i);
+                break;
+            case filterexecregex:
+                for (sp = ebuff+strlen(ebuff)-1; (*sp != '(') && (sp > ebuff); sp--)
+                   {
+                   }
+                
+                sscanf(sp+1,"%256[^)]",vbuff);
+                if (!RegexOK(vbuff))
+                   {
+                   yyerror("Regular expression error");
+                   }
+                break;
+                
+            case filternameregex:
+                if (!RegexOK(ebuff))
+                   {
+                   yyerror("Regular expression error");
+                   }
+                break;
+            case filterexec:
+                /* test here */
+                break;
+            case filtersymlinkto:
+                break;
+            }
+         
+         if ((fp->criteria[crit] = strdup(ebuff)) == NULL)
+            {
+            CfLog(cferror,"Couldn't allocate filter memory","strdup");
+            FatalError("Dying..");
+            }
+         else
+            {
+            return;
+            }
+         }
       }
    }
 }
@@ -231,7 +234,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
    {
    if (fp->criteria[filterresult] == NULL)
        {
-       snprintf(OUTPUT,bufsize*2,"No result specified for filter %s",fp->alias);
+       snprintf(OUTPUT,CF_BUFSIZE*2,"No result specified for filter %s",fp->alias);
        CfLog(cferror,OUTPUT,"");
        FatalError("Consistency error");
        }
@@ -239,7 +242,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
    if ((fp->criteria[filterfromctime]!=NULL && fp->criteria[filtertoctime]==NULL) ||
        (fp->criteria[filterfromctime]==NULL && fp->criteria[filtertoctime]!=NULL))
       {
-      snprintf(OUTPUT,bufsize*2,"Incomplete ctime limit specification of filter %s",fp->alias);
+      snprintf(OUTPUT,CF_BUFSIZE*2,"Incomplete ctime limit specification of filter %s",fp->alias);
       CfLog(cferror,OUTPUT,"");
       FatalError("Consistency errors");
       }
@@ -247,7 +250,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
    if ((fp->criteria[filterfromatime]!=NULL && fp->criteria[filtertoatime]==NULL) ||
        (fp->criteria[filterfromatime]==NULL && fp->criteria[filtertoatime]!=NULL))
       {
-      snprintf(OUTPUT,bufsize*2,"Incomplete atime limit specification of filter %s",fp->alias);
+      snprintf(OUTPUT,CF_BUFSIZE*2,"Incomplete atime limit specification of filter %s",fp->alias);
       CfLog(cferror,OUTPUT,"");
       FatalError("Consistency errors");
       }
@@ -255,7 +258,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
    if ((fp->criteria[filterfrommtime]!=NULL && fp->criteria[filtertomtime]==NULL) ||
        (fp->criteria[filterfrommtime]==NULL && fp->criteria[filtertomtime]!=NULL))
       {
-      snprintf(OUTPUT,bufsize*2,"Incomplete mtime limit specification of filter %s",fp->alias);
+      snprintf(OUTPUT,CF_BUFSIZE*2,"Incomplete mtime limit specification of filter %s",fp->alias);
       CfLog(cferror,OUTPUT,"");
       FatalError("Consistency errors");
       }
@@ -263,7 +266,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
    if ((fp->criteria[filterfromstime]!=NULL && fp->criteria[filtertostime]==NULL) ||
        (fp->criteria[filterfromstime]==NULL && fp->criteria[filtertostime]!=NULL))
       {
-      snprintf(OUTPUT,bufsize*2,"Incomplete stime limit specification of filter %s",fp->alias);
+      snprintf(OUTPUT,CF_BUFSIZE*2,"Incomplete stime limit specification of filter %s",fp->alias);
       CfLog(cferror,OUTPUT,"");
       FatalError("Consistency errors");
       }
@@ -271,7 +274,7 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
    if ((fp->criteria[filterfromttime]!=NULL && fp->criteria[filtertottime]==NULL) ||
        (fp->criteria[filterfromttime]==NULL && fp->criteria[filtertottime]!=NULL))
       {
-      snprintf(OUTPUT,bufsize*2,"Incomplete ttime limit specification of filter %s",fp->alias);
+      snprintf(OUTPUT,CF_BUFSIZE*2,"Incomplete ttime limit specification of filter %s",fp->alias);
       CfLog(cferror,OUTPUT,"");
       FatalError("Consistency errors");
       }
@@ -280,100 +283,99 @@ for (fp = VFILTERLIST; fp != NULL; fp = fp->next)
    if ((fp->criteria[filterfromsize]!=NULL && fp->criteria[filtertosize]==NULL) ||
        (fp->criteria[filterfromsize]==NULL && fp->criteria[filtertosize]!=NULL))
       {
-      snprintf(OUTPUT,bufsize*2,"Incomplete size limit specification of filter %s",fp->alias);
+      snprintf(OUTPUT,CF_BUFSIZE*2,"Incomplete size limit specification of filter %s",fp->alias);
       CfLog(cferror,OUTPUT,"");
       FatalError("Consistency errors");
       }
-
+   
    if (fp->criteria[filterfromatime] != NULL)
       {
       if (Date2Number(fp->criteria[filterfromatime],t) > Date2Number(fp->criteria[filtertoatime],t))
-	 {
-	 snprintf(OUTPUT,bufsize*2,"To/From atimes silly in filter %s (from > to)",fp->alias);
-	 CfLog(cferror,OUTPUT,"");
-	 FatalError("Consistency errors");
-	 }
+         {
+         snprintf(OUTPUT,CF_BUFSIZE*2,"To/From atimes silly in filter %s (from > to)",fp->alias);
+         CfLog(cferror,OUTPUT,"");
+         FatalError("Consistency errors");
+         }
       }
-
+   
    if (fp->criteria[filterfromctime] != NULL)
       {
       if (Date2Number(fp->criteria[filterfromctime],t) > Date2Number(fp->criteria[filtertoctime],t))
-	 {
-	 snprintf(OUTPUT,bufsize*2,"To/From ctimes silly in filter %s (from > to)",fp->alias);
-	 CfLog(cferror,OUTPUT,"");
-	 FatalError("Consistency errors");
-	 }
+         {
+         snprintf(OUTPUT,CF_BUFSIZE*2,"To/From ctimes silly in filter %s (from > to)",fp->alias);
+         CfLog(cferror,OUTPUT,"");
+         FatalError("Consistency errors");
+         }
       }
-
+   
    if (fp->criteria[filterfrommtime] != NULL)
       {
       if (Date2Number(fp->criteria[filterfrommtime],t) > Date2Number(fp->criteria[filtertomtime],t))
-	 {
-	 snprintf(OUTPUT,bufsize*2,"To/From mtimes silly in filter %s (from > to)",fp->alias);
-	 CfLog(cferror,OUTPUT,"");
-	 FatalError("Consistency errors");
-	 }
+         {
+         snprintf(OUTPUT,CF_BUFSIZE*2,"To/From mtimes silly in filter %s (from > to)",fp->alias);
+         CfLog(cferror,OUTPUT,"");
+         FatalError("Consistency errors");
+         }
       }
    
    if (fp->criteria[filterfromstime] != NULL)
       {
       if (Date2Number(fp->criteria[filterfromstime],t) > Date2Number(fp->criteria[filtertostime],t))
-	 {
-	 snprintf(OUTPUT,bufsize*2,"To/From stimes silly in filter %s (from > to)",fp->alias);
-	 CfLog(cferror,OUTPUT,"");
-	 FatalError("Consistency errors");
-	 }
+         {
+         snprintf(OUTPUT,CF_BUFSIZE*2,"To/From stimes silly in filter %s (from > to)",fp->alias);
+         CfLog(cferror,OUTPUT,"");
+         FatalError("Consistency errors");
+         }
       }
-
+   
    if (fp->criteria[filterfromttime] != NULL)
       {
       if (Date2Number(fp->criteria[filterfromttime],t) > Date2Number(fp->criteria[filtertottime],t))
-	 {
-	 snprintf(OUTPUT,bufsize*2,"To/From ttimes silly in filter %s (from > to)",fp->alias);
-	 CfLog(cferror,OUTPUT,"");
-	 FatalError("Consistency errors");
-	 }
+         {
+         snprintf(OUTPUT,CF_BUFSIZE*2,"To/From ttimes silly in filter %s (from > to)",fp->alias);
+         CfLog(cferror,OUTPUT,"");
+         FatalError("Consistency errors");
+         }
       if (strncmp(fp->criteria[filterfromttime],"accumulated",strlen("accumulated")) != 0)
-	 {
-	 yyerror("Must use accumulated time in FromTtime");
-	 }
+         {
+         yyerror("Must use accumulated time in FromTtime");
+         }
       if (strncmp(fp->criteria[filtertottime],"accumulated",strlen("accumulated")) != 0)
-	 {
-	 yyerror("Must use accumulated time in ToTtime");
-	 }
+         {
+         yyerror("Must use accumulated time in ToTtime");
+         }
       }
-
+   
    if (fp->criteria[filterfromsize] != NULL)
       {
       if (strcmp(fp->criteria[filterfromsize],fp->criteria[filtertosize]) > 0)
-	 {
-	 snprintf(OUTPUT,bufsize*2,"To/From size is silly in filter %s (from > to)",fp->alias);
-	 CfLog(cferror,OUTPUT,"");
-	 FatalError("Consistency errors");
-	 }
+         {
+         snprintf(OUTPUT,CF_BUFSIZE*2,"To/From size is silly in filter %s (from > to)",fp->alias);
+         CfLog(cferror,OUTPUT,"");
+         FatalError("Consistency errors");
+         }
       }
-
+   
    }
 }
-   
+
 /********************************************************************/
 
-enum filternames FilterActionsToCode(filtertype)
-
-char *filtertype;
+enum filternames FilterActionsToCode(char *filtertype)
 
 { int i;
 
-Debug1("FilterActionsToCode(%s)\n",filtertype);
+Debug("FilterActionsToCode(%s)\n",filtertype);
 
 if (filtertype[strlen(filtertype)-1] != ':')
    {
    yyerror("Syntax error in filter type");
    return NoFilter;
    }
-
-filtertype[strlen(filtertype)-1] = '\0';
  
+filtertype[strlen(filtertype)-1] = '\0';
+
+
 for (i = 0; VFILTERNAMES[i] != '\0'; i++)
    {
    if (strcmp(VFILTERNAMES[i],filtertype) == 0)
@@ -387,9 +389,7 @@ return (NoFilter);
 
 /*******************************************************************/
 
-int FilterExists(name)
-
-char *name;
+int FilterExists(char *name)
 
 { struct Filter *ptr;
 
@@ -405,10 +405,7 @@ return false;
 
 /*********************************************************************/
 
-time_t Date2Number(string,now)
-
-char *string;
-time_t now;
+time_t Date2Number(char *string,time_t now)
 
  /*
 
@@ -417,7 +414,7 @@ time_t now;
     
   */
 
-{ char type[16],datestr[63];
+{ char type[CF_SMALLBUF],datestr[CF_SMALLBUF];
   int year=-1,month=-1,day=-1,hr=-1,min=-1,sec=-1; 
   time_t time;
   struct tm tmv;
@@ -560,16 +557,12 @@ return now;
 /* Several filters are ANDed together....                          */
 /*******************************************************************/
 
-int ProcessFilter(proc,filterlist,names,start,end)
-
-char *proc, **names;
-int *start,*end;
-struct Item *filterlist;
+int ProcessFilter(char *proc,struct Item *filterlist,char **names,int *start,int *end)
 
 { struct Item *tests = NULL;
   struct Filter *fp;
   int result = true, tmpres, i;
-  char *line[noproccols];
+  char *line[CF_PROCCOLS];
   
 Debug("ProcessFilter(%.6s...)\n",proc);
 
@@ -583,7 +576,7 @@ if (strlen(proc) == 0)
    return false;
    }
  
-SplitLine(proc,filterlist,names,start,end,line);
+SplitLine(proc,names,start,end,line);
 
 for (fp = VFILTERLIST; fp != NULL; fp=fp->next)
    {
@@ -592,45 +585,40 @@ for (fp = VFILTERLIST; fp != NULL; fp=fp->next)
       Debug ("Applying filter %s\n",fp->alias);
 
       if (fp->criteria[filterresult] == NULL)
-	 {
-	 fp->criteria[filterresult] = strdup("Owner.PID.PPID.PGID.RSize.VSize.Status.Command.TTime.STime.TTY.Priority.Threads");
-	 }
+         {
+         fp->criteria[filterresult] = strdup("Owner.PID.PPID.PGID.RSize.VSize.Status.Command.TTime.STime.TTY.Priority.Threads");
+         }
       
       DoProc(&tests,fp->criteria,names,line);
-
+      
       if (tmpres = EvaluateORString(fp->criteria[filterresult],tests))
-	 {
-	 AddMultipleClasses(fp->defines);
-	 }
+         {
+         AddMultipleClasses(fp->defines);
+         }
       else
-	 {
-	 AddMultipleClasses(fp->elsedef);
-	 }
-
+         {
+         AddMultipleClasses(fp->elsedef);
+         }
+      
       result &= tmpres;
       }   
    }
-
-for (i = 0; i < noproccols; i++)
-   {
-   if (line[i] != NULL)
-      {
-      free(line[i]);
-      }
-   }
-
-DeleteItemList(tests);
-return result; 
+ 
+ for (i = 0; i < CF_PROCCOLS; i++)
+    {
+    if (line[i] != NULL)
+       {
+       free(line[i]);
+       }
+    }
+ 
+ DeleteItemList(tests);
+ return result; 
 }
 
 /*******************************************************************/
 
-int FileObjectFilter(file,lstatptr,filterlist,context)
-
-char *file;
-struct stat *lstatptr;
-struct Item *filterlist;
-enum actions context;
+int FileObjectFilter(char *file,struct stat *lstatptr,struct Item *filterlist,enum actions context)
 
 { struct Item *tests = NULL;
   struct Filter *fp;  int result = true, tmpres;
@@ -649,21 +637,21 @@ for (fp = VFILTERLIST; fp != NULL; fp=fp->next)
       Debug ("Applying filter %s\n",fp->alias);
       
       if (fp->criteria[filterresult] == NULL)
-	 {
-	 fp->criteria[filterresult] = strdup("Type.Owner.Group.Mode.Ctime.Mtime.Atime.Size.ExecRegex.NameRegex.IsSymLinkTo.Exec");
-	 }
+         {
+         fp->criteria[filterresult] = strdup("Type.Owner.Group.Mode.Ctime.Mtime.Atime.Size.ExecRegex.NameRegex.IsSymLinkTo.ExecProgram");
+         }
       
       DoFilter(&tests,fp->criteria,lstatptr,file);
-
+      
       if (tmpres = EvaluateORString(fp->criteria[filterresult],tests))
-	 {
-	 AddMultipleClasses(fp->defines);
-	 }
+         {
+         AddMultipleClasses(fp->defines);
+         }
       else
-	 {
-	 AddMultipleClasses(fp->elsedef);
-	 }
-
+         {
+         AddMultipleClasses(fp->elsedef);
+         }
+      
       result &= tmpres;
       }
    }
@@ -675,11 +663,8 @@ return result;
 
 /*******************************************************************/
 
-void DoFilter(attr,crit,lstatptr,filename)
+void DoFilter(struct Item **attr,char **crit,struct stat *lstatptr,char *filename)
 
-struct Item **attr;
-char **crit,*filename;
-struct stat *lstatptr;
 
 {
 if (crit[filtertype] != NULL)
@@ -773,10 +758,7 @@ if (crit[filterexec] != NULL)
 
 /*******************************************************************/
 
-void DoProc(attr,crit,names,line)
-
-struct Item **attr;
-char **crit,**names,**line;
+void DoProc(struct Item **attr,char **crit,char **names,char **line)
 
 {
 if (crit[filterowner] != NULL)
@@ -938,10 +920,7 @@ else
 
 /*******************************************************************/
 
-int FilterTypeMatch(lstatptr,crit)
-
-struct stat *lstatptr;
-char *crit;
+int FilterTypeMatch(struct stat *lstatptr,char *crit)
 
 { struct Item *attrib = NULL;
 
@@ -953,6 +932,7 @@ if (S_ISLNK(lstatptr->st_mode))
 if (S_ISREG(lstatptr->st_mode))
    {
    PrependItem(&attrib,"reg","");
+   PrependItem(&attrib,"file","");
    }
 
 if (S_ISDIR(lstatptr->st_mode))
@@ -1001,16 +981,14 @@ else
 
 /*******************************************************************/
 
-int FilterProcMatch(name1,name2,expr,names,line)
-
-char *name1,*name2,*expr;
-char **names,**line;
+int FilterProcMatch(char *name1,char *name2,char *expr,char **names,char **line)
 
 { int i;
   regex_t rx;
   regmatch_t pmatch;
 
-  Debug("FilterProcMatch(%s,%s,%s,%s,[%s])\n",name1,name2,expr,names,line);
+Debug("FilterProcMatch(%s,%s,%s,<%x>,<%x>)\n",name1,name2,expr,names,line);
+
 
 if (CfRegcomp(&rx,expr,REG_EXTENDED) != 0)
    {
@@ -1023,43 +1001,41 @@ for (i = 0; names[i] != NULL; i++)
       {
       Debug("Match (%s) to (%s)\n",expr,line[i]);
       if (regexec(&rx,line[i],1,&pmatch,0) == 0)
-	 {
-	 if ((pmatch.rm_so == 0) && (pmatch.rm_eo == strlen(line[i])))
-	    {
-	    regfree(&rx);
-	    return true;
-	    }
-	 }
-      
+         {
+         if ((pmatch.rm_so == 0) && (pmatch.rm_eo == strlen(line[i])))
+            {
+            regfree(&rx);
+            return true;
+            }
+         }
+
+      Debug("No match of (%s) to (%s)\n",expr,line[i]);
       regfree(&rx);
       return false;      
       }
    }
-
-regfree(&rx);
-return false; 
+ 
+ regfree(&rx);
+ return false; 
 }
 
 /*******************************************************************/
 
-int FilterProcSTimeMatch(name1,name2,fromexpr,toexpr,names,line)
-
-char *name1,*name2,*fromexpr,*toexpr;
-char **names,**line;
+int FilterProcSTimeMatch(char *name1,char *name2,char *fromexpr,char *toexpr,char **names,char **line)
 
 { int i;
   time_t fromtime,totime, now = CFSTARTTIME,pstime;
   char year[5],month[4],hr[3],min[3],day[3],timestr[256];
   
-bzero(year,5);
+memset(year,0,5);
 strcpy(year,VYEAR);
-bzero(month,4); 
+memset(month,0,4); 
 strcpy(month,VMONTH);
-bzero(day,3); 
+memset(day,0,3); 
 strcpy(day,VDAY);
-bzero(hr,3); 
+memset(hr,0,3); 
 strcpy(hr,VHR);
-bzero(min,3); 
+memset(min,0,3); 
 strcpy(min,VMINUTE); 
 
 fromtime = Date2Number(fromexpr,now);
@@ -1070,39 +1046,87 @@ for (i = 0; names[i] != NULL; i++)
    if ((strcmp(names[i],name1) == 0) || (strcmp(names[i],name2) == 0))
       {
       if (strstr(line[i],":")) /* Hr:Min:Sec */
-	 {
+         {
          sscanf(line[i],"%2[^:]:%2[^:]:",hr,min);
-	 snprintf(timestr,256,"date(%s,%d,%s,%s,%s,0)",year,Month2Number(month),day,hr,min);
-	 }
+         snprintf(timestr,256,"date(%s,%d,%s,%s,%s,0)",year,Month2Number(month),day,hr,min);
+         }
       else                     /* date Month */
-	 {
+         {
          sscanf(line[i],"%3[a-zA-Z] %2[0-9]",month,day);
-	 snprintf(timestr,256,"date(%s,%d,%s,%s,%s,0)",year,Month2Number(month),day,hr,min);
-	 }
+         snprintf(timestr,256,"date(%s,%d,%s,%s,%s,0)",year,Month2Number(month),day,hr,min);
+         }
       
       if (Month2Number(month) < 0)
-	 {
-	 continue;
-	 }
+         {
+         continue;
+         }
       
       pstime = Date2Number(timestr,now);
-
+      
       Debug("Stime %s converted to %s\n",timestr,ctime(&pstime));
-
+      
       return ((fromtime < pstime) && (pstime < totime));
       }
    } 
-return false;
+ return false;
 }
 
 /*******************************************************************/
 
-int FilterProcTTimeMatch(name1,name2,fromexpr,toexpr,names,line)
+/*
+ * HvB: Bas van der Vlies
+ *  Parse different TTime values
+*/
 
-char *name1,*name2,*fromexpr,*toexpr;
-char **names,**line;
+void ParseTTime(char *line, char *time_str)
 
-{ int i,hr=0,min=-1,sec=-1;
+{
+int  day=0, hr=0, min=0, sec=0;
+int r;
+
+if (strstr(line,":")) /* day-Hr:Min:Sec */
+   {
+   /* 
+    * first check the long fromat 
+    *   day-hr:min:sec (posix)
+    *       hr:min:sec (posix)
+    *          min:sec (old)
+   */ 
+   if ((r = sscanf(line,"%d-%d:%d:%d",&day,&hr,&min,&sec)) == 4)
+      {
+      snprintf(time_str,256,"accumulated(0,0,%d,%d,%d,%d)",day,hr,min,sec);
+      }
+   else if ((r = sscanf(line,"%d:%d:%d",&hr,&min,&sec)) == 3 )
+      {
+      snprintf(time_str,256,"accumulated(0,0,0,%d,%d,%d)",hr,min,sec);
+      }
+   else if ((r = sscanf(line,"%d:%d",&min,&sec)) == 2)
+      {
+/* Removed test - apparent bug
+   
+      if (min > 59)
+         {
+         day = min / (24 * 60);
+         hr  = (min - (day * 24 * 60)) / (60);
+         min = min % 60;
+         }
+*/
+
+      day = min / (24 * 60);
+      hr  = (min - (day * 24 * 60)) / (60);
+      min = min % 60;
+
+      snprintf(time_str,256,"accumulated(0,0,%d,%d,%d,%d)",day,hr,min,sec);
+      }
+   }
+}
+
+
+/*******************************************************************/
+
+int FilterProcTTimeMatch(char *name1,char *name2,char *fromexpr,char *toexpr,char **names,char **line)
+
+{ int i;
   time_t fromtime,totime,now = CFSTARTTIME,pstime = CFSTARTTIME;
   char timestr[256];
 
@@ -1113,19 +1137,10 @@ for (i = 0; names[i] != NULL; i++)
    {
    if ((strcmp(names[i],name1) == 0) || (strcmp(names[i],name2) == 0))
       {
-      if (strstr(line[i],":")) /* Hr:Min:Sec */
-	 {
-         sscanf(line[i],"%d:%d",&min,&sec);
-
-	 if (sec < 0 || min < 0)
-	    {
-	    CfLog(cferror,"Parse error checking process time","");
-	    return false;
-	    }
-	 
-	 snprintf(timestr,256,"accumulated(0,0,0,%d,%d,%d)",hr,min,sec);
-	 pstime = Date2Number(timestr,now);
-	 }
+      memset(timestr,0,sizeof(timestr));
+      ParseTTime(line[i], timestr);
+      Debug("ParseTTime = %s\n",timestr); 
+      pstime = Date2Number(timestr,now);
 
       return ((fromtime < pstime) && (pstime < totime));
       }
@@ -1135,13 +1150,10 @@ return false;
 
 /*******************************************************************/
 
-int FilterOwnerMatch(lstatptr,crit)
-
-char *crit;
-struct stat *lstatptr;
+int FilterOwnerMatch(struct stat *lstatptr,char *crit)
 
 { struct Item *attrib = NULL;
-  char buffer[64];
+  char buffer[CF_SMALLBUF];
   struct passwd *pw;
 
 sprintf(buffer,"%d",lstatptr->st_uid);
@@ -1170,13 +1182,10 @@ else
 
 /*******************************************************************/
 
-int FilterGroupMatch(lstatptr,crit)
-
-char *crit;
-struct stat *lstatptr;
+int FilterGroupMatch(struct stat *lstatptr,char *crit)
 
 { struct Item *attrib = NULL;
-  char buffer[64];
+  char buffer[CF_SMALLBUF];
   struct group *gr;
 
 sprintf(buffer,"%d",lstatptr->st_gid);
@@ -1205,24 +1214,25 @@ else
 
 /*******************************************************************/
 
-int FilterModeMatch(lstatptr,crit)
+int FilterModeMatch(struct stat *lstatptr,char *crit)
 
-char *crit;
-struct stat *lstatptr;
+{ mode_t plusmask,minusmask,newperm;
 
-{ mode_t plusmask,minusmask;
 
 ParseModeString(crit,&plusmask,&minusmask);
 
-return ((lstatptr->st_mode & plusmask) && !(lstatptr->st_mode & minusmask));
+newperm = (lstatptr->st_mode & 07777);
+newperm |= plusmask;
+newperm &= ~minusmask;
+ 
+Debug("FilterModeMatch %s -> (+%o,-%o/%o) = %d \n",crit,plusmask,minusmask,lstatptr->st_mode &07777,((newperm & 07777) == (lstatptr->st_mode & 07777)));
+
+return ((newperm & 07777) == (lstatptr->st_mode & 07777));
 } 
 
 /*******************************************************************/
 
-int FilterTimeMatch(stattime,from,to)
-
-char *from,*to;
-time_t stattime;
+int FilterTimeMatch(time_t stattime,char *from,char *to)
 
 { time_t fromtime,totime, now = CFSTARTTIME;
 
@@ -1234,9 +1244,7 @@ return ((fromtime < stattime) && (stattime < totime));
 
 /*******************************************************************/
 
-int FilterNameRegexMatch(filename,crit)
-
-char *filename,*crit;
+int FilterNameRegexMatch(char *filename,char *crit)
 
 { regex_t rx;
   regmatch_t pmatch;
@@ -1261,30 +1269,27 @@ return false;
 
 /*******************************************************************/
 
-int FilterExecRegexMatch(filename,crit)
-
-char *filename,*crit;
+int FilterExecRegexMatch(char *filename,char *crit)
 
 { regex_t rx;
   regmatch_t pmatch;
-  char buffer[bufsize],expr[bufsize],line[bufsize],*sp;
+  char buffer[CF_BUFSIZE],expr[CF_BUFSIZE],line[CF_BUFSIZE],ebuff[CF_EXPANDSIZE],*sp;
   FILE *pp;
 
 AddMacroValue(CONTEXTID,"this",filename);
-VBUFF[0] = '\0'; 
-ExpandVarstring(crit,VBUFF,NULL);
+ExpandVarstring(crit,ebuff,NULL);
 DeleteMacro(CONTEXTID,"this");
   
-bzero(buffer,bufsize);
-bzero(line,bufsize); 
-bzero(expr,bufsize);
+memset(buffer,0,CF_BUFSIZE);
+memset(line,0,CF_BUFSIZE); 
+memset(expr,0,CF_BUFSIZE);
 
-for (sp = VBUFF+strlen(VBUFF)-1; (*sp != '(') && (sp > VBUFF); sp--)
+for (sp = ebuff+strlen(ebuff)-1; (*sp != '(') && (sp > ebuff); sp--)
    {
    }
  
 sscanf(sp+1,"%256[^)]",expr);
-strncpy(buffer,VBUFF,(int)(sp-VBUFF));
+strncpy(buffer,ebuff,(int)(sp-ebuff));
  
 if (CfRegcomp(&rx,expr,REG_EXTENDED) != 0)
    {
@@ -1293,12 +1298,12 @@ if (CfRegcomp(&rx,expr,REG_EXTENDED) != 0)
 
 if ((pp = cfpopen(buffer,"r")) == NULL)
    {
-   snprintf(OUTPUT,bufsize*2,"Couldn't open pipe to command %s\n",buffer);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Couldn't open pipe to command %s\n",buffer);
    CfLog(cferror,OUTPUT,"cfpopen");
    return false;
    }
  
-ReadLine(line,bufsize,pp);  /* One buffer only */
+ReadLine(line,CF_BUFSIZE,pp);  /* One buffer only */
 
 cfpclose(pp); 
  
@@ -1317,24 +1322,22 @@ return false;
 
 /*******************************************************************/
 
-int FilterIsSymLinkTo(filename,crit)
-
-char *filename,*crit;
+int FilterIsSymLinkTo(char *filename,char *crit)
 
 { regex_t rx;
   regmatch_t pmatch;
-  char buffer[bufsize];
+  char buffer[CF_BUFSIZE];
 
 if (CfRegcomp(&rx,crit,REG_EXTENDED) != 0)
    {
    return false;
    }
 
-bzero(buffer,bufsize);
+memset(buffer,0,CF_BUFSIZE);
  
-if (readlink(filename,buffer,bufsize-1) == -1)
+if (readlink(filename,buffer,CF_BUFSIZE-1) == -1)
    {
-   snprintf(OUTPUT,bufsize*2,"Unable to read link %s in filter",filename);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Unable to read link %s in filter",filename);
    CfLog(cferror,OUTPUT,"readlink");
    regfree(&rx);
    return false;      
@@ -1355,21 +1358,19 @@ return false;
 
 /*******************************************************************/
 
-int FilterExecMatch(filename,crit)
-
-char *filename, *crit;
+int FilterExecMatch(char *filename,char *crit)
 
   /* command can include $(this) for the name of the file */
 
-{
+{ char ebuff[CF_EXPANDSIZE];
+ 
 AddMacroValue(CONTEXTID,"this",filename);
-VBUFF[0] = '\0'; 
-ExpandVarstring(crit,VBUFF,NULL);
+ExpandVarstring(crit,ebuff,NULL);
 DeleteMacro(CONTEXTID,"this");
 
-Debug("Executing filter command [%s]\n",VBUFF);
+Debug("Executing filter command [%s]\n",ebuff);
 
-if (ShellCommandReturnsZero(VBUFF))
+if (ShellCommandReturnsZero(ebuff,false))
    {
    return true;
    }
@@ -1381,15 +1382,12 @@ else
 
 /*******************************************************************/
 
-void GetProcessColumns(proc,names,start,end)
-
-char *proc, **names;
-int *start,*end;
+void GetProcessColumns(char *proc,char **names,int *start,int *end)
 
 { char *sp,title[16];
   int col,offset = 0;
   
-for (col = 0; col < noproccols; col++)
+for (col = 0; col < CF_PROCCOLS; col++)
    {
    start[col] = end[col] = -1;
    names[col] = NULL;
@@ -1404,23 +1402,23 @@ for (sp = proc; *sp != '\0'; sp++)
    if (isspace((int)*sp))
       {
       if (start[col] != -1)
-	 {
-	 Debug("End of %s is %d\n",title,offset-1);
-	 end[col++] = offset - 1;
-	 if (col > noproccols - 1)
-	    {
-	    CfLog(cferror,"Column overflow in process table","");
-	    break;
-	    }
-	 }
+         {
+         Debug("End of %s is %d\n",title,offset-1);
+         end[col++] = offset - 1;
+         if (col > CF_PROCCOLS - 1)
+            {
+            CfLog(cferror,"Column overflow in process table","");
+            break;
+            }
+         }
       continue;
       }
-
+   
    else if (start[col] == -1)
       {
       start[col] = offset;
-      Debug("Start of %s is %d\n",title,offset);
       sscanf(sp,"%15s",title);
+      Debug("Start of %s is %d\n",title,offset);
       names[col] = strdup(title);
       Debug("Col[%d]=%s\n",col,names[col]);
       }
@@ -1435,26 +1433,19 @@ if (end[col] == -1)
 
 /*******************************************************************/
 
-void SplitLine(proc,filterlist,names,start,end,line)
-
-char *proc, **names, **line;
-int *start,*end;
-struct Item *filterlist;
+void SplitLine(char *proc,char **names,int *start,int *end,char **line)
 
 { int i,s,e;
 
 Debug("SplitLine(%s)\n",proc); 
  
-for (i = 0; i < noproccols; i++)
+for (i = 0; i < CF_PROCCOLS; i++)
    {
    line[i] = NULL;
    }
  
 for (i = 0; names[i] != NULL; i++)
    {
-   s = start[i];
-   e = end[i];
-
    for (s = start[i]; (s >= 0) && !isspace((int)*(proc+s)); s--)
       {
       }
@@ -1476,22 +1467,29 @@ for (i = 0; names[i] != NULL; i++)
    else
       {
       for (e = end[i]; (e <= end[i]+10) && !isspace((int)*(proc+e)); e++)
-	 {
-	 }
+         {
+         }
       
       while (isspace((int)proc[e]))
-	 {
-	 if (e > 0)
-	    {
-	    e--;
-	    }
-	 }
+         {
+         if (e > 0)
+            {
+            e--;
+            }
+         }
+      }
+   
+   if (s <= e)
+      {
+      line[i] = (char *)malloc(e-s+2);
+      memset(line[i],0,(e-s+2));
+      strncpy(line[i],(char *)(proc+s),(e-s+1));
+      }
+   else
+      {
+      line[i] = (char *)malloc(1);
+      line[i][0] = '\0';
       }
-
-   line[i] = (char *)malloc(e-s+2);
-
-   bzero(line[i],(e-s+2));
-   strncpy(line[i],(char *)(proc+s),(e-s+1));
    
    Debug("  %s=(%s) of [%s]\n",names[i],line[i],proc);
    }
