@@ -135,10 +135,16 @@ switch (fn = FunctionStringToCode(name))
        HandleStatInfo(fn,args,value);
        break;
    case fn_execresult:
-       HandleFunctionExec(args,value);
+       HandleFunctionExec(args,value,false);
+       break;
+   case fn_execshellresult:
+       HandleFunctionExec(args,value,true);
        break;
    case fn_returnszero:
-       HandleReturnsZero(args,value);
+       HandleReturnsZero(args,value,false);
+       break;
+   case fn_returnszeroshell:
+       HandleReturnsZero(args,value,true);
        break;
    case fn_iprange:
        HandleIPRange(args,value);
@@ -345,7 +351,7 @@ void HandleStatInfo(enum builtin fn,char *args,char *value)
 
 FunctionArgs(args,argv,1);
 
-if (lstat(args,&statbuf) == -1)
+if (lstat(argv[0],&statbuf) == -1)
    {
    strcpy(value,CF_NOCLASS);
    return;
@@ -394,8 +400,11 @@ strcpy(value,CF_NOCLASS);
 void HandleIPRange(char *args,char *value)
 
 { struct Item *ip;
+  char argv[CF_MAXFARGS][CF_EXPANDSIZE];
+
+FunctionArgs(args,argv,1);
  
-if (strchr(args,','))
+if (strchr(argv[0],','))
    {
    yyerror("Illegal argument to unary class-function");
    return;
@@ -403,7 +412,7 @@ if (strchr(args,','))
  
 strcpy(value,CF_NOCLASS);
 
-if (!FuzzyMatchParse(args))
+if (!FuzzyMatchParse(argv[0]))
    {
    return;
    }
@@ -413,7 +422,7 @@ for (ip = IPADDRESSES; ip != NULL; ip = ip->next)
    {
    Debug("Checking IP Range against iface %s\n",VIPADDRESS);
    
-   if (FuzzySetMatch(args,ip->name) == 0)
+   if (FuzzySetMatch(argv[0],ip->name) == 0)
       {
       Debug("IPRange Matched\n");
       strcpy(value,CF_ANYCLASS);
@@ -424,7 +433,7 @@ for (ip = IPADDRESSES; ip != NULL; ip = ip->next)
 
 Debug("Checking IP Range against RDNS %s\n",VIPADDRESS);
 
-if (FuzzySetMatch(args,VIPADDRESS) == 0)
+if (FuzzySetMatch(argv[0],VIPADDRESS) == 0)
    {
    Debug("IPRange Matched\n");
    strcpy(value,CF_ANYCLASS);
@@ -439,11 +448,11 @@ strcpy(value,CF_NOCLASS);
 
 void HandleHostRange(char *args,char *value)
 
-{
-Debug("SRDEBUG in HandleHostRange()\n"); 
-Debug("SRDEBUG args=%s value=%s\n",args,value);
+{ char argv[CF_MAXFARGS][CF_EXPANDSIZE];
 
-if (!FuzzyHostParse(args))
+FunctionArgs(args,argv,2);
+
+if (!FuzzyHostParse(argv[0],argv[1]))
    {
    strcpy(value,CF_NOCLASS);
    return;
@@ -452,7 +461,9 @@ if (!FuzzyHostParse(args))
 /* VDEFAULTBINSERVER.name is relative domain name */
 /* (see nameinfo.c ~line 145)                     */
 
-if (FuzzyHostMatch(args,VDEFAULTBINSERVER.name) == 0)
+Debug("Parsing fuzzy host succeeded\n");
+
+if (FuzzyHostMatch(argv[0],argv[1],VUQNAME) == 0)
    {
    Debug("SRDEBUG SUCCESS!\n");
    strcpy(value,CF_ANYCLASS);
@@ -519,46 +530,53 @@ strcpy(value,CF_NOCLASS);
 
 /*********************************************************************/
 
-void HandleFunctionExec(char *args,char *value)
+void HandleFunctionExec(char *args,char *value,int useshell)
 
 { char command[CF_MAXVARSIZE];
+  char argv[CF_MAXFARGS][CF_EXPANDSIZE];
+  
+FunctionArgs(args,argv,1); 
 
- if (ACTION != control)
+if (ACTION != control)
    {
-   yyerror("Use of ExecResult(s) outside of variable assignment");
+   yyerror("Use of Exec[Shell]Result(s) outside of variable assignment");
    }
  
-if (*args == '/')
+if (*argv[0] == '/')
    {
-   strncpy(command,args,CF_MAXVARSIZE);
-   GetExecOutput(command,value);
+   strncpy(command,argv[0],CF_MAXVARSIZE);
+   GetExecOutput(command,value,useshell);
    Chop(value);
    value[CF_MAXVARSIZE-1] = '\0';  /* Truncate to CF_MAXVARSIZE */
    }
  else
     {
-    yyerror("ExecResult(/command) must specify an absolute path");
+    snprintf(OUTPUT,CF_BUFSIZE,"Exec[Shell]Result(%s) must specify an absolute path",argv[0]);
+    yyerror(OUTPUT);
     } 
 }
 
 /*********************************************************************/
 
-void HandleReturnsZero(char *args,char *value)
+void HandleReturnsZero(char *args,char *value,int useshell)
 
 { char command[CF_BUFSIZE];
-
+  char argv[CF_MAXFARGS][CF_EXPANDSIZE];
+  
 if (ACTION != groups)
    {
    yyerror("Use of ReturnsZero(s) outside of class assignment");
    }
 
-Debug("HandleReturnsZero(%s)\n",args); 
+FunctionArgs(args,argv,1); 
+
+Debug("HandleReturnsZero(%s)\n",argv); 
  
-if (*args == '/')
+if (*argv[0] == '/')
    {
-   strncpy(command,args,CF_BUFSIZE);
+   strncpy(command,argv[0],CF_BUFSIZE);
    
-   if (ShellCommandReturnsZero(command))
+   if (ShellCommandReturnsZero(command,useshell))
       {
       strcpy(value,CF_ANYCLASS);
       return;
@@ -577,15 +595,18 @@ if (*args == '/')
 
 void HandleIsDefined(char *args,char *value)
 
-{
+{ char argv[CF_MAXFARGS][CF_EXPANDSIZE];
+
+FunctionArgs(args,argv,1);
+
 if (ACTION != groups)
    {
    yyerror("Use of IsDefined(s) outside of class assignment");
    }
 
-Debug("HandleIsDefined(%s)\n",args); 
+Debug("HandleIsDefined(%s)\n",argv[0]); 
  
-if (GetMacroValue(CONTEXTID,args))
+if (GetMacroValue(CONTEXTID,argv[0]))
    {
    strcpy(value,CF_ANYCLASS);
    return;
@@ -1033,7 +1054,7 @@ val = atoi(maxbytes);
 if (strcmp(formattype,"lines") != 0)
    {
    snprintf(OUTPUT,CF_BUFSIZE,"Unknown format type in ReadList(%s)",args);
-   CfLog(cferror,OUTPUT,"");;
+   CfLog(cferror,OUTPUT,"");
    }
     
 if ((fp = fopen(filename,"r")) == NULL)
@@ -1155,7 +1176,7 @@ DeleteAgentConn(CONN);
 void HandleSelectPLeader(char *args,char *value)
 
 { FILE *fp;
-  char machine[128],first[128];
+  char machine[MAXHOSTNAMELEN],first[MAXHOSTNAMELEN];
   char argv[CF_MAXFARGS][CF_EXPANDSIZE];
   char *filename=argv[0],*commentchar=argv[1];
   char *policy=argv[2],*size=argv[3];
@@ -1211,9 +1232,9 @@ while (!feof(fp))
          }
       }
    
-   memset(machine,0,127);
+   memset(machine,0,MAXHOSTNAMELEN-1);
    
-   sscanf(buffer,"%127s",machine);
+   sscanf(buffer,"%255s",machine);
 
    if (strlen(machine) == 0)
       {
@@ -1228,7 +1249,7 @@ while (!feof(fp))
 
    if (strlen(first) == 0)
       {
-      strncpy(first,machine,127);
+      strncpy(first,machine,MAXHOSTNAMELEN-1);
       }
    
    if ((count == psize) || ((count > 0) && feof(fp)))
@@ -1237,7 +1258,7 @@ while (!feof(fp))
 
       done = false;
       
-      if (IsItemIn(list,VFQNAME)||IsItemIn(list,VIPADDRESS))
+      if (IsItemIn(list,VUQNAME)||IsItemIn(list,VFQNAME)||IsItemIn(list,VIPADDRESS))
          {
          /* This is my peer group */
          Verbose("Found my peer group:\n");
@@ -1260,11 +1281,11 @@ while (!feof(fp))
                Verbose(" %s (leader)\n",ip->name);
                if (strlen(ip->name) > 0)
                   {
-                  strncpy(value,ip->name,127);
+                  strncpy(value,ip->name,MAXHOSTNAMELEN-1);
                   }
                else
                   {
-                  strncpy(value,first,127);
+                  strncpy(value,first,MAXHOSTNAMELEN-1);
                   }
                done = true;
                }
@@ -1302,7 +1323,7 @@ while (!feof(fp))
 
 if (!done)
    {
-   strncpy(value,first,127); /* Fill up blanks in peer groups */
+   strncpy(value,first,MAXHOSTNAMELEN-1); /* Fill up blanks in peer groups */
    }
 
 if (list != NULL)
@@ -1318,11 +1339,11 @@ fclose(fp);
 
 void HandleSelectPGroup(char *args,char *value)
 
-{ char argv[CF_MAXFARGS][CF_EXPANDSIZE],machine[128],first[128];
+{ char argv[CF_MAXFARGS][CF_EXPANDSIZE],machine[MAXHOSTNAMELEN],first[MAXHOSTNAMELEN];
   char *filename=argv[0],*commentchar=argv[1];
   char *policy=argv[2],*size=argv[3];
   struct Item *list = NULL,*ip;
-  int i,psize = -1, count,done = false;
+  int i,psize = -1, count,done = false,len = 0;
   FILE *fp;
 
 FunctionArgs(args,argv,4);
@@ -1375,7 +1396,7 @@ while (!feof(fp))
       }
    
    machine[0] = '\0';
-   sscanf(buffer,"%127s",machine);
+   sscanf(buffer,"%255s",machine);
 
    if (strlen(machine) == 0)
       {
@@ -1390,14 +1411,15 @@ while (!feof(fp))
 
    if (strlen(first) == 0)
       {
-      strncpy(first,machine,127);
+      strncpy(first,machine,MAXHOSTNAMELEN-1);
       }
 
    if ((count == psize) || ((count > 0) && feof(fp)))
       {
       done = false;
+      len = 0;
       
-      if (IsItemIn(list,VFQNAME)||IsItemIn(list,VIPADDRESS))
+      if (IsItemIn(list,VUQNAME)||IsItemIn(list,VFQNAME)||IsItemIn(list,VIPADDRESS))
          {
          /* This is my peer group */
          Verbose("Found my peer group:\n");
@@ -1413,12 +1435,14 @@ while (!feof(fp))
                   return;
                   }
                
-               snprintf(value+strlen(value),CF_BUFSIZE,"%s%c",ip->name,LISTSEPARATOR);
-               Verbose(" %s (added peer) = %s\n",ip->name,value);            
+               snprintf(value+len,CF_BUFSIZE,"%s%c",ip->name,LISTSEPARATOR);
+               Verbose(" %s (added peer) = %s\n",ip->name,value);
+               len += strlen(ip->name)+1;
                }
             done = true;
             }
-         value[strlen(value)-1] = '\0';
+         
+         value[strlen(value)-1] = '\0'; /* Remove last separator from list */
          }
       else
          {
@@ -1446,7 +1470,7 @@ while (!feof(fp))
 
 if (!done)
    {
-   strncat(value,first,127); /* Fill up blanks in peer groups */
+   strncat(value,first,MAXHOSTNAMELEN-1); /* Fill up blanks in peer groups */
    }
  
 fclose(fp); 
@@ -1457,10 +1481,13 @@ fclose(fp);
 void HandleReturnValues(char *args,char *value)
 
 { struct Item *ip;
- 
-Verbose("This is a method with return value list: (%s)\n",args);
+  char argv[CF_MAXFARGS][CF_EXPANDSIZE];
 
-for (ip = SplitStringAsItemList(args,','); ip != NULL; ip=ip->next)
+FunctionArgs(args,argv,1);
+ 
+Verbose("This is a method with return value list: (%s)\n",argv[0]);
+
+for (ip = SplitStringAsItemList(argv[0],','); ip != NULL; ip=ip->next)
    {
    AppendItem(&METHODRETURNVARS,ip->name,CLASSBUFF);
    }
@@ -1473,12 +1500,15 @@ strcpy(value,"noinstall");
 void HandleReturnClasses(char *args,char *value)
 
 { struct Item *ip;
- 
-Verbose("This is a method with return class list: %s\n",args);
+  char argv[CF_MAXFARGS][CF_EXPANDSIZE];
 
-for (ip = SplitStringAsItemList(args,','); ip != NULL; ip=ip->next)
+FunctionArgs(args,argv,1);
+ 
+Verbose("This is a method with return class list: %s\n",argv[0]);
+
+for (ip = SplitStringAsItemList(argv[0],','); ip != NULL; ip=ip->next)
    {
-   AppendItem(&METHODRETURNCLASSES,args,CLASSBUFF);
+   AppendItem(&METHODRETURNCLASSES,argv[0],CLASSBUFF);
    }
 
 strcpy(value,"noinstall");
@@ -1496,8 +1526,10 @@ void HandleShowState(char *args,char *value)
   int maxlen = 0,count;
   double *dist = NULL, S = 0.0;
   char *offset = NULL;
-  
-  
+  char argv[CF_MAXFARGS][CF_EXPANDSIZE];
+
+FunctionArgs(args,argv,1);
+
 if ((ACTION != alerts) && PARSING)
    {
    yyerror("Use of ShowState(type) outside of alert declaration");
@@ -1511,7 +1543,7 @@ if (PARSING)
    return;
    }
  
-snprintf(buffer,CF_BUFSIZE-1,"%s/state/cf_%s",CFWORKDIR,args);
+snprintf(buffer,CF_BUFSIZE-1,"%s/state/cf_%s",CFWORKDIR,argv[0]);
 
 if (stat(buffer,&statbuf) == 0)
    {
@@ -1534,9 +1566,9 @@ if (stat(buffer,&statbuf) == 0)
          {
          Verbose("%s: (%2d) %s",VPREFIX,conns,buffer);
          
-         if (IsSocketType(args))
+         if (IsSocketType(argv[0]))
             {
-            if (strncmp(args,"incoming",8) == 0 || strncmp(args,"outgoing",8) == 0)
+            if (strncmp(argv[0],"incoming",8) == 0 || strncmp(argv[0],"outgoing",8) == 0)
                {
                if (strncmp(buffer,"tcp",3) == 0)
                   {
@@ -1551,7 +1583,7 @@ if (stat(buffer,&statbuf) == 0)
                DePort(vbuff);
                }
             }
-         else if (IsTCPType(args))
+         else if (IsTCPType(argv[0]))
             {
             count = 1;
             sscanf(buffer,"%d %[^\n]",&count,remote);
@@ -1562,6 +1594,7 @@ if (stat(buffer,&statbuf) == 0)
             }
          else      
             {
+            /* If we get here this is a process thing */
             if (offset == NULL)
                {
                if (offset = strstr(buffer,"CMD"))
@@ -1601,7 +1634,7 @@ if (stat(buffer,&statbuf) == 0)
    printf("%s: -----------------------------------------------------------------------------------\n",VPREFIX);
    printf("%s: In the last 40 minutes, the peak state was q = %d:\n",VPREFIX,conns);
 
-   if (IsSocketType(args)||IsTCPType(args))
+   if (IsSocketType(argv[0])||IsTCPType(argv[0]))
       {
       if (addresses != NULL)
          {
@@ -1692,12 +1725,12 @@ if (stat(buffer,&statbuf) == 0)
       }
    
    printf("%s: -----------------------------------------------------------------------------------\n",VPREFIX);
-   snprintf(buffer,CF_BUFSIZE,"State of %s peaked at %s\n",args,ctime(&statbuf.st_mtime));
+   snprintf(buffer,CF_BUFSIZE,"State of %s peaked at %s\n",argv[0],ctime(&statbuf.st_mtime));
    strcpy(value,buffer);
    }
 else 
    {
-   snprintf(buffer,CF_BUFSIZE,"State parameter %s is not known or recorded\n",args);   
+   snprintf(buffer,CF_BUFSIZE,"State parameter %s is not known or recorded\n",argv[0]);
    strcpy(value,buffer);
    }
 
@@ -1807,7 +1840,9 @@ value[0] = '\0';
 FunctionArgs(ebuff,argv,2);
 
 Debug("PrepModule(%s,%s)\n",argv[0],argv[1]);
- 
+
+BuildClassEnvironment();
+
 if (CheckForModule(argv[0],argv[1]))
    {
    strcpy(value,CF_ANYCLASS);
@@ -1879,6 +1914,7 @@ for (i = 0; i < number; i++)
    {
    ExpandVarstring(ipp->name,exbuf,NULL);
    strncpy(arg[i],exbuf,CF_EXPANDSIZE);
+   Debug("ARG[%d] %s\n",i,arg[i]);
    ipp = ipp->next;
    }
 
