@@ -61,6 +61,7 @@ char CFPUBKEYFILE[CF_BUFSIZE];
 char CFPRIVKEYFILE[CF_BUFSIZE];
 char AVDB[CF_MAXVARSIZE];
 char CFWORKDIR[CF_BUFSIZE];
+char PIDFILE[CF_BUFSIZE];
 
 RSA *PRIVKEY = NULL, *PUBKEY = NULL;
 
@@ -93,7 +94,7 @@ pthread_mutex_t MUTEX_LOCK = PTHREAD_MUTEX_INITIALIZER;
 /*                                                                 */
 /*******************************************************************/
 
-  PUBLIC char *COPYRIGHT = "Free Software Foundation 1994-\nDonated by Mark Burgess, Faculty of Engineering,\nOslo University College, 0254 Oslo, Norway";
+  PUBLIC char *COPYRIGHT = "Free Software Foundation 1994-\nDonated by Mark Burgess, Faculty of Engineering,\nOslo University College, Norway";
 
   PRIVATE char *VPRECONFIG = "cf.preconf";
   PRIVATE char *VRCFILE = "cfrc";
@@ -134,6 +135,8 @@ pthread_mutex_t MUTEX_LOCK = PTHREAD_MUTEX_INITIALIZER;
   PUBLIC short LOGTIDYHOMEFILES = true;
   PUBLIC short UPDATEONLY = false;
   PUBLIC short SKIPIDENTIFY = false;
+  PUBLIC short ALL_SINGLECOPY = false;
+  PUBLIC short FULLENCRYPT = false;
 
   PUBLIC char FORK = 'n';
 
@@ -185,19 +188,19 @@ pthread_mutex_t MUTEX_LOCK = PTHREAD_MUTEX_INITIALIZER;
  /* Anomaly                                                         */
  /*******************************************************************/
 
-char *ECGSOCKS[ATTR][2] =
+struct sock ECGSOCKS[ATTR] = /* extended to map old to new using enum*/
    {
-   {"137","netbiosns"},
-   {"138","netbiosdgm"},
-   {"139","netbiosssn"},
-   {"194","irc"},
-   {"5308","cfengine"},
-   {"2049","nfsd"},
-   {"25","smtp"},
-   {"80","www"},
-   {"21","ftp"},
-   {"22","ssh"},
-   {"443","wwws"}
+   {"137","netbiosns",ob_netbiosns_in,ob_netbiosns_out},
+   {"138","netbiosdgm",ob_netbiosdgm_in,ob_netbiosdgm_out},
+   {"139","netbiosssn",ob_netbiosssn_in,ob_netbiosssn_out},
+   {"194","irc",ob_irc_in,ob_irc_out},
+   {"5308","cfengine",ob_cfengine_in,ob_cfengine_out},
+   {"2049","nfsd",ob_nfsd_in,ob_nfsd_out},
+   {"25","smtp",ob_smtp_in,ob_smtp_out},
+   {"80","www",ob_www_in,ob_www_out},
+   {"21","ftp",ob_ftp_in,ob_ftp_out},
+   {"22","ssh",ob_ssh_in,ob_ssh_out},
+   {"443","wwws",ob_wwws_in,ob_wwws_out}
    };
 
 char *TCPNAMES[CF_NETATTR] =
@@ -211,6 +214,101 @@ char *TCPNAMES[CF_NETATTR] =
    "misc"
    };
 
+char *OBS[CF_OBSERVABLES] =
+    {
+    "users",
+    "rootprocs",
+    "otherprocs",
+    "diskfree",
+    "loadavg",
+    "netbiosns_in",
+    "netbiosns_out",
+    "netbiosdgm_in",
+    "netbiosdgm_out",
+    "netbiosssn_in",
+    "netbiosssn_out",
+    "irc_in",
+    "irc_out",
+    "cfengine_in",
+    "cfengine_out",
+    "nfsd_in",
+    "nfsd_out",
+    "smtp_in",
+    "smtp_out",
+    "www_in",
+    "www_out",
+    "ftp_in",
+    "ftp_out",
+    "ssh_in",
+    "ssh_out",
+    "wwws_in",
+    "wwws_out",
+    "icmp_in",
+    "icmp_out",
+    "udp_in",
+    "udp_out",
+    "dns_in",
+    "dns_out",
+    "tcpsyn_in",
+    "tcpsyn_out",
+    "tcpack_in",
+    "tcpack_out",
+    "tcpfin_in",
+    "tcpfin_out",
+    "tcpmisc_in",
+    "tcpmisc_out",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    "spare",
+    };
+
  /*******************************************************************/
  /* Methods                                                         */
  /*******************************************************************/
@@ -244,11 +342,12 @@ char *TCPNAMES[CF_NETATTR] =
  /* Data/list structures - root pointers                            */
  /*******************************************************************/
 
+  PUBLIC struct Item *ABORTHEAP = NULL;
+  
   PROTECTED  struct Item *VTIMEZONE = NULL;
   PROTECTED  struct Item *VMOUNTLIST = NULL;
   PROTECTED  struct Item *VEXCLUDECOPY = NULL;
   PROTECTED  struct Item *VAUTODEFINE = NULL;
-  PROTECTED  struct Item *VSINGLECOPY = NULL;
   PROTECTED  struct Item *VEXCLUDELINK = NULL;
   PROTECTED  struct Item *VCOPYLINKS = NULL;
   PROTECTED  struct Item *VLINKCOPIES = NULL;
@@ -284,7 +383,7 @@ char *TCPNAMES[CF_NETATTR] =
   PROTECTED  struct Mounted *MOUNTED = NULL;             /* Files systems already mounted */
   PROTECTED  struct MiscMount *VMISCMOUNT = NULL;
   PROTECTED  struct MiscMount *VMISCMOUNTTOP = NULL;
-  PROTECTED  struct Item *VBINSERVERS = &VDEFAULTBINSERVER;
+  PROTECTED  struct Item *VBINSERVERS = NULL;
   PROTECTED  struct Link *VLINK = NULL;
   PROTECTED  struct Link *VLINKTOP = NULL;
   PROTECTED  struct File *VFILE = NULL;
@@ -548,7 +647,9 @@ char *TCPNAMES[CF_NETATTR] =
       "islink",
       "isplain",
       "execresult",
+      "execshellresult",
       "returnszero",
+      "returnszeroshell",
       "iprange",
       "hostrange",
       "isdefined",
@@ -640,6 +741,7 @@ char *TCPNAMES[CF_NETATTR] =
       "ge",
       "le",
       "ne",
+      "none",
       NULL
       };
 
@@ -651,6 +753,8 @@ char *TCPNAMES[CF_NETATTR] =
       "rpm",
       "dpkg",   /* aptget ? */
       "sun",    /* pkginfo/pkgadd/pkgrm */
+      "aix",    /* lslpp/installp */
+      "portage",
       NULL
       };
 
@@ -714,6 +818,7 @@ char *TCPNAMES[CF_NETATTR] =
   PRIVATE short NOSPLAY = false;
   PRIVATE short DONESPLAY = false;
 
+  PUBLIC  struct Item *VSINGLECOPY = NULL;
 
   PROTECTED  struct Item *VACLBUILD = NULL;
   PROTECTED  struct Item *VFILTERBUILD = NULL;
@@ -771,8 +876,8 @@ char *TCPNAMES[CF_NETATTR] =
   PRIVATE char *MOUNTOPTS;
   PRIVATE char *DESTINATION;
   PRIVATE char *IMAGEACTION;
-  PRIVATE char *VIFNAME[16];
-  PRIVATE char *VIFNAMEOVERRIDE[16];
+  PRIVATE char VIFNAME[16];
+  PRIVATE char VIFNAMEOVERRIDE[16];
   PRIVATE char *CHDIR;
   PRIVATE char *LOCALREPOS;
   PRIVATE char *EXPR;
@@ -1025,6 +1130,7 @@ char *TCPNAMES[CF_NETATTR] =
      "AppendIfNoSuchLinesFromFile",
      "IncrementPointer",
      "ReplaceLineWith",
+     "ExpandVariables",
      "DeleteToLineMatching",
      "HashCommentToLineMatching",
      "PercentCommentToLineMatching",
@@ -1039,8 +1145,10 @@ char *TCPNAMES[CF_NETATTR] =
      "GotoLastLine",
      "BreakIfLineMatches",
      "BeginGroupIfNoMatch",
+     "BeginGroupIfMatch",
      "BeginGroupIfNoLineMatching",
      "BeginGroupIfNoSuchLine",
+     "BeginGroupIfLineMatching",
      "EndGroup",
      "Append",
      "Prepend",
@@ -1066,6 +1174,7 @@ char *TCPNAMES[CF_NETATTR] =
      "BeginGroupIfFileIsNewer",
      "BeginGroupIfFileExists",
      "BeginGroupIfNoLineContaining",
+     "BeginGroupIfLineContaining",
      "BeginGroupIfDefined",
      "BeginGroupIfNotDefined",
      "AutoCreate",
@@ -1141,6 +1250,8 @@ char *TCPNAMES[CF_NETATTR] =
    "SAUTH",
    "SSYNCH",
    "SGET",
+   "VERSION",
+   "SOPENDIR",
    NULL
    };
 
@@ -1164,5 +1275,40 @@ char *TCPNAMES[CF_NETATTR] =
   PUBLIC char CFLAST[CF_BUFSIZE]; 
   PUBLIC char LOCKDB[CF_BUFSIZE];
 
+
+/*******************************************************************/
+/*                                                                 */
+/* Checksums                                                       */
+/*                                                                 */
+/*******************************************************************/
+
+/* These string lengths should not exceed CF_MAXDIGESTNAMELEN
+   characters for packing */
+
+  PUBLIC char *CF_DIGEST_TYPES[8][2] =
+     {
+     "md5","m",
+     "sha224","c",
+     "sha256","C",
+     "sha384","h",
+     "sha512","H",
+     "sha1","S",
+     "sha","s",   /* Should come last, since substring */
+     NULL,NULL
+     };
+
+  PUBLIC int CF_DIGEST_SIZES[8] =
+     {
+     CF_MD5_LEN,
+     CF_SHA224_LEN,
+     CF_SHA256_LEN,
+     CF_SHA384_LEN,
+     CF_SHA512_LEN,
+     CF_SHA1_LEN,
+     CF_SHA_LEN,
+     0
+     };
+
+
 /* EOF */
 
