@@ -68,6 +68,10 @@
 #include "cf.defs.h"
 #include "cf.extern.h"
 
+#ifndef INET_ADDRSTRLEN
+#define INET_ADDRSTRLEN sizeof("255.255.255.255")
+#endif
+
 #if !defined(NT) && !defined(IRIX)
 
 /* IRIX makes the routing stuff obsolete unless we do this */
@@ -469,7 +473,7 @@ if ( strcmp(VROUTE[VSYSTEMHARDCLASS], "-") != 0 )
          /* get the route command and the format for the delete argument */
          snprintf(routefmt,CF_MAXVARSIZE,"%s %s",VROUTE[VSYSTEMHARDCLASS],VROUTEDELFMT[VSYSTEMHARDCLASS]);
          snprintf(VBUFF,CF_MAXVARSIZE,routefmt,"default",VDEFAULTROUTE->name);
-         if (ShellCommandReturnsZero(VBUFF))
+         if (ShellCommandReturnsZero(VBUFF,false))
             {
             CfLog(cfinform,"Removing old default route","");
             CfLog(cfinform,VBUFF,"");
@@ -480,11 +484,12 @@ if ( strcmp(VROUTE[VSYSTEMHARDCLASS], "-") != 0 )
             }
          }
       }
+   
    if (! DONTDO)
       {
       snprintf(routefmt,CF_MAXVARSIZE,"%s %s",VROUTE[VSYSTEMHARDCLASS],VROUTEADDFMT[VSYSTEMHARDCLASS]);
       snprintf(VBUFF,CF_MAXVARSIZE,routefmt,"default",VDEFAULTROUTE->name);
-      if (ShellCommandReturnsZero(VBUFF))
+      if (ShellCommandReturnsZero(VBUFF,false))
          {
          CfLog(cfinform,"Setting default route","");
          CfLog(cfinform,VBUFF,"");
