@@ -35,48 +35,54 @@
 
 /*********************************************************************/
 
-int LoadItemList(liststart,file)
-
-struct Item **liststart;
-char *file;
+int LoadItemList(struct Item **liststart,char *file)
 
 { FILE *fp;
   struct stat statbuf;
 
 if (stat(file,&statbuf) == -1)
    {
-   snprintf(OUTPUT,bufsize*2,"Couldn't stat %s\n",file);
-   CfLog(cfverbose,OUTPUT,"stat");
-   return false;
+   if (DONTDO)
+      {
+      snprintf(OUTPUT,CF_BUFSIZE*2,"Currently no file %s to edit\n",file);
+      CfLog(cfverbose,OUTPUT,"");
+      return false;
+      }
+   else
+      {
+      snprintf(OUTPUT,CF_BUFSIZE*2,"Couldn't stat %s - no file to edit\n",file);
+      CfLog(cfverbose,OUTPUT,"stat");
+      return false;
+      }
    }
 
 if ((EDITFILESIZE != 0) &&(statbuf.st_size > EDITFILESIZE))
    {
-   snprintf(OUTPUT,bufsize*2,"File %s is bigger than the limit <editfilesize>\n",file);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"File %s is bigger than the limit <editfilesize>\n",file);
    CfLog(cfinform,OUTPUT,"");
    return(false);
    }
 
 if (! S_ISREG(statbuf.st_mode))
    {
-   snprintf(OUTPUT,bufsize*2,"%s is not a plain file\n",file);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"%s is not a plain file\n",file);
    CfLog(cfinform,OUTPUT,"");
    return false;
    }
 
 if ((fp = fopen(file,"r")) == NULL)
    {
-   snprintf(OUTPUT,bufsize*2,"Couldn't read file %s for editing\n",file);
-   CfLog(cferror,OUTPUT,"fopen");
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Couldn't read file %s for editing\n",file);
+   CfLog(cfinform,OUTPUT,"fopen");
    return false;
    }
 
 
-bzero(VBUFF,bufsize); 
+memset(VBUFF,0,CF_BUFSIZE); 
 
 while(!feof(fp))
    {
-   ReadLine(VBUFF,bufsize,fp);
+   ReadLine(VBUFF,CF_BUFSIZE,fp);
 
    if (!feof(fp) || (strlen(VBUFF) != 0))
       {
@@ -91,23 +97,20 @@ return (true);
 
 /*********************************************************************/
 
-int SaveItemList(liststart,file,repository)
-
-struct Item *liststart;
-char *file, *repository;
+int SaveItemList(struct Item *liststart,char *file,char *repository)
 
 { struct Item *ip;
   struct stat statbuf;
-  char new[bufsize],backup[bufsize];
+  char new[CF_BUFSIZE],backup[CF_BUFSIZE];
   FILE *fp;
   mode_t mask;
-  char stamp[bufsize]; 
+  char stamp[CF_BUFSIZE]; 
   time_t STAMPNOW;
   STAMPNOW = time((time_t *)NULL);
 
 if (stat(file,&statbuf) == -1)
    {
-   snprintf(OUTPUT,bufsize*2,"Couldn't stat %s, which needed editing!\n",file);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Couldn't stat %s, which needed editing!\n",file);
    CfLog(cferror,OUTPUT,"");
    CfLog(cferror,"Check definition in program - is file NFS mounted?\n\n","");
    return false;
@@ -131,7 +134,7 @@ unlink(new); /* Just in case of races */
  
 if ((fp = fopen(new,"w")) == NULL)
    {
-   snprintf(OUTPUT,bufsize*2,"Couldn't write file %s after editing\n",new);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Couldn't write file %s after editing\n",new);
    CfLog(cferror,OUTPUT,"");
    return false;
    }
@@ -149,7 +152,7 @@ if (fclose(fp) == -1)
  
 if (ISCFENGINE && (ACTION == editfiles))
    {
-   snprintf(OUTPUT,bufsize*2,"Edited file %s \n",file);
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Edited file %s \n",file);
    CfLog(cfinform,OUTPUT,""); 
    }
 
@@ -158,23 +161,23 @@ if (IMAGEBACKUP != 'n')
    if (! IsItemIn(VREPOSLIST,new))
       {
       if (rename(file,backup) == -1)
-	 {
-	 snprintf(OUTPUT,bufsize*2,"Error while renaming backup %s\n",file);
-	 CfLog(cferror,OUTPUT,"rename ");
-	 unlink(new);
-	 return false;
-	 }
+         {
+         snprintf(OUTPUT,CF_BUFSIZE*2,"Error while renaming backup %s\n",file);
+         CfLog(cfverbose,OUTPUT,"rename ");
+         unlink(new);
+         return false;
+         }
       else if (Repository(backup,repository))
-	 {
-	 unlink(backup);
-	 }
+         {
+         unlink(backup);
+         }
       }
    }
 
 if (rename(new,file) == -1)
    {
-   snprintf(OUTPUT,bufsize*2,"Error while renaming %s\n",file);
-   CfLog(cferror,OUTPUT,"rename");
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Error while renaming %s\n",file);
+   CfLog(cfverbose,OUTPUT,"rename");
    return false;
    }       
 
@@ -187,16 +190,15 @@ return true;
 
 /*********************************************************************/
 
-int CompareToFile(liststart,file)
+int CompareToFile(struct Item *liststart,char *file)
 
 /* returns true if file on disk is identical to file in memory */
 
-struct Item *liststart;
-char *file;
-
 { FILE *fp;
   struct stat statbuf;
   struct Item *ip = liststart;
+  unsigned char *finmem = NULL, fdata;
+  unsigned long fip = 0, tmplen, idx;
 
 Debug("CompareToFile(%s)\n",file);
 
@@ -209,67 +211,67 @@ if (liststart == NULL)
    {
    return false;
    }
- 
-if ((fp = fopen(file,"r")) == NULL)
-   {
-   snprintf(OUTPUT,bufsize*2,"Couldn't read file %s for editing\n",file);
-   CfLog(cferror,OUTPUT,"fopen");
-   return false;
-   }
-
-bzero(VBUFF,bufsize);
 
 for (ip = liststart; ip != NULL; ip=ip->next)
    {
-   ReadLine(VBUFF,bufsize,fp);
-   
-   if (feof(fp) && (ip->next != NULL))
-      {
-      fclose(fp);
-      return false;
-      }
-
-   if ((ip->name == NULL) && (strlen(VBUFF) == 0))
-      {
-      continue;
-      }
-   
-   if (ip->name == NULL)
-      {
-      fclose(fp);
-      return false;
-      }
-   
-   if (strcmp(ip->name,VBUFF) != 0)
-      {
-      fclose(fp);
-      return false;
-      }
-
-   VBUFF[0] = '\0';
+   tmplen = strlen(ip->name);
+   if ((finmem = realloc(finmem, fip+tmplen+1)) == NULL)
+	{
+	Debug("CompareToFile(%s): can't realloc() memory\n",file);
+	free(finmem);
+	return false;
+	}
+   memcpy(finmem+fip, ip->name, tmplen);
+   fip += tmplen;
+   *(finmem+fip++) = '\n';
    }
 
-if (!feof(fp) && (ReadLine(VBUFF,bufsize,fp) != false))
+if (statbuf.st_size != fip)
    {
-   fclose(fp);
+   Debug("CompareToFile(%s): sizes are different: MEM:(%u) FILE:(%u)\n",file, fip, statbuf.st_size);
+   free(finmem);
+   return false;
+   }
+ 
+if ((fp = fopen(file,"r")) == NULL)
+   {
+   snprintf(OUTPUT,CF_BUFSIZE*2,"Couldn't read file %s for editing\n",file);
+   CfLog(cferror,OUTPUT,"fopen");
+   free(finmem);
    return false;
    }
 
+for (idx = 0; idx < fip; idx++)
+   {
+   if (fread(&fdata, 1, 1, fp) != 1)
+	{
+	Debug("CompareToFile(%s): non-zero fread() before file-in-mem finished at %u-th byte MEM:(0x%x/%c)\n",file, idx, *(finmem+idx), *(finmem+idx));
+	free(finmem);
+	fclose(fp);
+	return false;
+	}
+   if (fdata != *(finmem+idx))
+	{
+	Debug("CompareToFile(%s): difference found at %u-th byte MEM:(0x%x/%c) != FILE:(0x%x/%c)\n",file, idx, *(finmem+idx), *(finmem+idx), fdata, fdata);
+	free(finmem);
+	fclose(fp);
+	return false;
+	}
+   }
+
+free(finmem);
 fclose(fp);
 return (true);
 }
 
 /*********************************************************************/
 
-void InsertFileAfter (filestart,ptr,string)
+void InsertFileAfter (struct Item **filestart,struct Item *ptr,char *string)
 
-struct Item *ptr, **filestart;
-char *string;
-
-{ struct Item *ip, *old;
+{ struct Item *ip;
   char *sp;
   FILE *fp;
-  char linebuf[bufsize];
+  char linebuf[CF_BUFSIZE];
 
 EditVerbose("Edit: Inserting file %s \n",string);
 
@@ -279,7 +281,7 @@ if ((fp=fopen(string,"r")) == NULL)
    return;
    }
 
-while(!feof(fp) && ReadLine(linebuf,bufsize,fp))
+while(!feof(fp) && ReadLine(linebuf,CF_BUFSIZE,fp))
    {
    if ((ip = (struct Item *)malloc(sizeof(struct Item))) == NULL)
       {
@@ -296,16 +298,16 @@ while(!feof(fp) && ReadLine(linebuf,bufsize,fp))
    if (CURRENTLINEPTR == NULL)
       {
       if (*filestart == NULL)
-	 {
-	 *filestart = ip;
-	 ip->next = NULL;
-	 }
+         {
+         *filestart = ip;
+         ip->next = NULL;
+         }
       else
-	 {
-	 ip->next = (*filestart)->next;
-	 (*filestart)->next = ip;	    
-	 }
-
+         {
+         ip->next = (*filestart)->next;
+         (*filestart)->next = ip;     
+         }
+      
       strcpy(sp,linebuf);
       ip->name = sp;
       ip->classes = NULL;
