@@ -187,6 +187,30 @@ return(false);
 
 /*********************************************************************/
 
+void CopyList (struct Item **dest, struct Item *source)
+
+/* Copy or concat lists */
+    
+{ struct Item *ip;
+
+if (*dest != NULL)
+   {
+   FatalError("CopyList - list not initialized");
+   }
+ 
+if (source == NULL)
+   {
+   return;
+   }
+ 
+for (ip = source; ip != NULL; ip = ip ->next)
+   {
+   AppendItem(dest,ip->name,ip->classes);
+   }
+}
+
+/*********************************************************************/
+
 struct Item *ConcatLists (struct Item *list1,struct Item *list2)
 
 /* Notes: * Refrain from freeing list2 after using ConcatLists
@@ -202,6 +226,7 @@ if (list1 == NULL)
 for (endOfList1=list1; endOfList1->next!=NULL; endOfList1=endOfList1->next)
    {
    }
+
 endOfList1->next = list2;
 return list1;
 }
@@ -564,7 +589,7 @@ int FuzzyMatchParse(char *s)
 
 { char *sp;
   short isCIDR = false, isrange = false, isv6 = false, isv4 = false, isADDR = false; 
-  char address[128];
+  char address[CF_ADDRSIZE];
   int mask,count = 0;
 
 Debug("Check ParsingIPRange(%s)\n",s);
@@ -666,7 +691,7 @@ if (isv4 && isCIDR)
 if (isv4 && isrange)
    {
    long i, from = -1, to = -1;
-   char *sp1,buffer1[8];
+   char *sp1,buffer1[CF_MAX_IP_LEN];
    
    sp1 = s;
    
@@ -698,7 +723,7 @@ if (isv4 && isrange)
  
  if (isv6 && isCIDR)
     {
-    char address[128];
+    char address[CF_ADDRSIZE];
     int mask,blocks;
     
     if (strlen(s) < 20)
@@ -743,7 +768,7 @@ int FuzzySetMatch(char *s1,char *s2)
    FuzzySetMatch("128.39.74.10/23","128.39.75.56") == 0 */
 
 { short isCIDR = false, isrange = false, isv6 = false, isv4 = false;
-  char address[128];
+  char address[CF_ADDRSIZE];
   int mask;
   unsigned long a1,a2;
 
@@ -830,7 +855,7 @@ if (isv4)
    else
       {
       long i, from = -1, to = -1, cmp = -1;
-      char *sp1,*sp2,buffer1[8],buffer2[8];
+      char *sp1,*sp2,buffer1[CF_MAX_IP_LEN],buffer2[CF_MAX_IP_LEN];
       
       sp1 = s1;
       sp2 = s2;
@@ -919,7 +944,7 @@ if (isv6)
    else
       {
       long i, from = -1, to = -1, cmp = -1;
-      char *sp1,*sp2,buffer1[16],buffer2[16];
+      char *sp1,*sp2,buffer1[CF_MAX_IP_LEN],buffer2[CF_MAX_IP_LEN];
 
       sp1 = s1;
       sp2 = s2;
@@ -972,7 +997,7 @@ return -1;
 /* written by steve rader <rader@hep.wisc.edu>                       */
 /*********************************************************************/
 
-int FuzzyHostParse(char *s)
+int FuzzyHostParse(char *arg1,char *arg2)
 
 /*
  * do something like...
@@ -981,36 +1006,10 @@ int FuzzyHostParse(char *s)
  */ 
 
 { struct Item *args;
-  char *sp;
   long start = -1, end = -1, where = -1;
   int n;
-  Debug("SRDEBUG in FuzzyHostParse(): %s\n",s);
-  args = SplitStringAsItemList(s,',');
 
-if ( args->next == NULL )
-   { 
-   yyerror("HostRange() syntax error: not enough args (expecting two)");
-   return false;
-   }
-
-if ( args->next->next != NULL )
-   { 
-   yyerror("HostRange() syntax error: too many args (expecting two)");
-   return false;
-   }
-
-sp = args->next->name;
-
-n = sscanf(sp,"%ld-%ld%n",&start,&end,&where);
-
-Debug("SRDEBUG start=%d end=%d num_matched=%d\n",start,end,n);
-
-if ( n >= 2 && sp[where] != '\0' )
-   {
-   /* X-Ycrud syntax error */
-   yyerror("HostRange() syntax error: second arg should have X-Y format where X and Y are decimal numbers");
-   return false;
-   }
+n = sscanf(arg2,"%ld-%ld%n",&start,&end,&where);
 
 if ( n != 2 )
    {
@@ -1018,91 +1017,63 @@ if ( n != 2 )
    yyerror("HostRange() syntax error: second arg should have X-Y format where X and Y are decimal numbers");
    return false;
    } 
-Debug("SRDEBUG syntax is okay\n");
+
 return true; 
 }
 
 /*********************************************************************/
 
-
-int FuzzyHostMatch(s1,s2)
+int FuzzyHostMatch(char *arg0, char* arg1, char *refhost)
 
 /*
  * do something like...
  *   @args = split(/,/,$s1);
- *   if ( $s2 !~ /(\d+)$/ ) {
+ *   if ( $refhost !~ /(\d+)$/ ) {
  *     return 1; # failure: no num in hostname
  *   }
  *   if ( $1 < $args[1] || $1 > $args[2] ) {
  *     return 1; # failure: hostname num not in range
  *   }
- *   $s2 =~ s/^(.*?)\d.*$/$1/;
- *   if ( $s2 ne $args[0] ) {
+ *   $refhost =~ s/^(.*?)\d.*$/$1/;
+ *   if ( $refhost ne $args[0] ) {
  *     return 1; # failure: hostname doesn't match basename
  *   }
  *   return 0; # success
  */
 
-char *s1, *s2;
-
 { struct Item *args;
-  char *sp;
+ char *sp, refbase[CF_MAXVARSIZE];
   long cmp = -1, start = -1, end = -1;
-  char host_basename[CF_MAXVARSIZE];
-  Debug("SRDEBUG in FuzzyHostMatch(): %s vs %s\n",s2,s1);
-  args = SplitStringAsItemList(s1,',');
-  sp = s2;
-  
-  for (sp = s2+strlen(s2)-1; sp > s2; sp--)
-     {
-     if ( ! isdigit((int)*sp) )
-        {
-        sp++;
-        if ( sp != s2+strlen(s2) )
-           {
-           Debug("SRDEBUG extracted string %s\n",sp);
-           }
-        break;
-        }
-     }
-  
-  if ( sp == s2+strlen(s2) )
-     {
-     Debug("SRDEBUG FuzzyHostMatch() failed: did not extract int from the end of %s\n",s2);
-     return 1;
-     }
-  sscanf(sp,"%ld",&cmp);
-  Debug("SRDEBUG extracted int %d\n",cmp,sp);
 
-  /* HvB basename is */
-  strncpy(host_basename, s2, strlen(s2) - strlen(sp));
-  Debug("SRDEBUG host basename is  %s\n",host_basename,sp);
+sscanf(refhost,"%[^0-9]%ld",&refbase,&cmp);
 
-  if ( cmp < 0 )
-     {
-     Debug("SRDEBUG FuzzyHostMatch() failed: %s doesn't have an int in it's domain name\n",s2);
-     return 1;
-     }
+Debug("Found refbase=%s,cmp=%d\n",refbase,cmp);
 
-  sscanf(args->next->name,"%ld-%ld",&start,&end);
-  if ( cmp < start || cmp > end )
-     {
-     Debug("SRDEBUG FuzzyHostMatch() failed: %ld is not in (%ld..%ld)\n",cmp,start,end);
-     return 1;
-     }
-  Debug("SRDEBUG FuzzyHostMatch() %s is in (%ld..%ld)\n",s2,start,end);
+if (cmp < 0)
+   {
+   return 1;
+   }
 
+if (strlen(refbase) == 0)
+   {
+   return 1;
+   }
 
-  Debug("SRDEBUG host basename check: %s vs %s...\n",host_basename,args->name);
-  if ( strcmp(host_basename,args->name) != 0 )
-     {
-     Debug("SRDEBUG FuzzyHostMatch() failed: basename %s does not match %s\n",s2,args->name);
-     return 1;
-     }
-  Debug("SRDEBUG basename matches\n");
+sscanf(arg1,"%ld-%ld",&start,&end);
 
-  Debug("SRDEBUG FuzzyHostMatch() succeeded\n");
-  return 0;
+if ( cmp < start || cmp > end )
+   {
+   Debug("Failed on numberical range (%d < %d < %d)",end,cmp,start);
+   return 1;
+   }
+
+if (strcmp(refbase,arg0) != 0)
+   {
+   Debug("Failed on name %s != %s)",refbase,arg0);
+   return 1;
+   }
+
+return 0;
 }
 
 
