@@ -74,7 +74,7 @@ DB *DBP;
 void PreLockState()
 
 {
- strcpy(CFLOCK,"pre-lock-state");
+strcpy(CFLOCK,"pre-lock-state");
 }
 
 /********************************************************************/
@@ -96,6 +96,41 @@ strcpy(CFLOCK,SAVELOCK);
 
 /********************************************************************/
 
+void WritePID(char *filename)
+
+{ FILE *fp;
+  char *sp;
+
+if (!IsPrivileged())
+   {
+   Verbose("\n(Non privileged user...)\n\n");
+   
+   if ((sp = getenv("HOME")) == NULL)
+      {
+      FatalError("You do not have a HOME variable pointing to your home directory");
+      }
+   
+   snprintf(PIDFILE,CF_BUFSIZE-1,"%s/%s",sp,filename);
+   }
+else
+   {
+   snprintf(PIDFILE,CF_BUFSIZE-1,"%s/%s",WORKDIR,filename);
+   }
+
+if ((fp = fopen(PIDFILE,"w")) == NULL)
+   {
+   snprintf(OUTPUT,CF_BUFSIZE,"Could not write to PID file %s\n",filename);
+   CfLog(cfinform,OUTPUT,"fopen");
+   return;
+   }
+
+fprintf(fp,"%d\n",getpid());
+
+fclose(fp);
+}
+
+/********************************************************************/
+
 void HandleSignal(int signum)
  
 {
@@ -113,6 +148,7 @@ if (signum != SIGCHLD)
    
    if (signum == SIGTERM || signum == SIGINT || signum == SIGHUP || signum == SIGSEGV || signum == SIGKILL|| signum == SIGPIPE)
       {
+      unlink(PIDFILE);
       ReleaseCurrentLock();
       closelog();
       exit(0);
@@ -157,8 +193,6 @@ if ((errno = db_create(&DBP,NULL,0)) != 0)
    return;
    }
 
-
-
 #ifdef CF_OLD_DB
 if ((errno = DBP->open(DBP,LOCKDB,NULL,DB_BTREE,DB_CREATE,0644)) != 0)
 #else
@@ -487,7 +521,7 @@ if (PutLock(CFLOCK) == -1)
    {
    snprintf(OUTPUT,CF_BUFSIZE*2,"GetLock: can't open new lock file %s\n",CFLOCK);
    CfLog(cferror,OUTPUT,"fopen");
-   FatalError("");;
+   FatalError("");
    }
 }
 
