@@ -71,7 +71,7 @@ if ((ptr = (struct cfObject *)malloc(sizeof(struct cfObject))) == NULL)
    {
    FatalError("Memory Allocation failed for cfObject");
    }
- 
+
 if (VOBJTOP == NULL)
    {
    VOBJ = ptr;
@@ -367,6 +367,9 @@ if (ptr == NULL)
    ptr = ObjectContext(scope);
    }
  
+Debug("GetMacroValue(%s,%s): using scope '%s' for variable '%s'\n",
+      scope,name,ptr->scope,vname);
+ 
 i = slot = Hash(vname);
  
 if (CompareMacro(vname,ptr->hashtable[slot]) != 0)
@@ -401,6 +404,7 @@ else
       {
       }
 
+   Debug1("Return Macrovalue={%s}\n",sp+1);
    return(sp+1);
    }   
 }
