@@ -118,18 +118,24 @@ void DispatchNewMethod(struct Method *ptr)
 
 { struct Item *ip;
   char label[CF_BUFSIZE]; 
-  char serverip[64],clientip[64];
+  char serverip[CF_SMALLBUF],clientip[CF_SMALLBUF];
 
 serverip[0] = clientip[0] = '\0';
   
 for (ip = ptr->servers; ip != NULL; ip=ip->next)
-   {  
+   {
+   if (strcmp(ip->name,"none") == 0)
+      {
+      Verbose("Method with no server is a no-op\n");
+      continue;
+      }
+   
    if ((strcmp(ip->name,"localhost") == 0) || (strcmp(ip->name,"::1") == 0) || (strcmp(ip->name,"127.0.0.1") == 0))
       {
       Verbose("\nDispatch method localhost:%s to %s/rpc_in\n",ptr->name,VLOCKDIR);
 
       ptr->invitation = 'y';
-      snprintf(label,CF_BUFSIZE-1,"%s/rpc_in/localhost_localhost_%s_%s",VLOCKDIR,ptr->name,ChecksumPrint('m',ptr->digest));
+      snprintf(label,CF_BUFSIZE-1,"%s/rpc_in/localhost+localhost+%s+%s",VLOCKDIR,ptr->name,ChecksumPrint('m',ptr->digest));
       EncapsulateMethod(ptr,label);
       }
    else
@@ -153,7 +159,7 @@ for (ip = ptr->servers; ip != NULL; ip=ip->next)
          }
       
       Verbose("Hailing server (%s) from our calling id %s\n",serverip,VFQNAME);
-      snprintf(label,CF_BUFSIZE-1,"%s/rpc_out/%s_%s_%s_%s",VLOCKDIR,clientip,serverip,ptr->name,ChecksumPrint('m',ptr->digest));
+      snprintf(label,CF_BUFSIZE-1,"%s/rpc_out/%s+%s+%s+%s",VLOCKDIR,clientip,serverip,ptr->name,ChecksumPrint('m',ptr->digest));
       EncapsulateMethod(ptr,label);
       Debug("\nDispatched method %s to %s/rpc_out\n",label,VLOCKDIR);
       }
@@ -175,7 +181,6 @@ struct Item *GetPendingMethods(int state)
   
 Debug("GetPendingMethods(%d) in (%s/rpc_in)\n",state,VLOCKDIR);
 
-
 Debug("Processing applications...\n");
  
 snprintf(filename,CF_MAXVARSIZE-1,"%s/rpc_in",VLOCKDIR);
@@ -270,7 +275,7 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
    
    if (mp = IsDefinedMethod(ip->name,digeststring))
       {
-      if (IsItemIn(mp->servers,"localhost") || IsItemIn(mp->servers,IPString2Hostname(server)) || IsItemIn(mp->servers,VIPADDRESS) || IsItemIn(mp->servers,server) || IsItemIn(mp->servers,VFQNAME))
+      if (IsItemIn(mp->servers,"localhost") || IsItemIn(mp->servers,IPString2Hostname(server)) || IsItemIn(mp->servers,IPString2UQHostname(server)) || IsItemIn(mp->servers,VIPADDRESS) || IsItemIn(mp->servers,server) || IsItemIn(mp->servers,VFQNAME) || IsItemIn(mp->servers,VUQNAME))
          {
          if (state == CF_METHODREPLY)
             {
@@ -294,6 +299,7 @@ for (dirp = readdir(dirh); dirp != NULL; dirp = readdir(dirh))
       else
          {
          Verbose("No local method match on this host for received request (%s) for host %s=%s=%s in\n",dirp->d_name,server,IPString2Hostname(server),VIPADDRESS);
+
          if (VERBOSE)
             {
             DebugListItemList(mp->servers);
@@ -325,7 +331,7 @@ void EvaluatePendingMethod(char *methodname)
 
 { struct Method *ptr;
   char line[CF_BUFSIZE];
-  char options[32];
+  char options[CF_BUFSIZE];
   char client[CF_BUFSIZE],server[CF_BUFSIZE],name[CF_BUFSIZE],digeststring[CF_BUFSIZE],extra[CF_BUFSIZE];
   char *sp;
   char execstr[CF_BUFSIZE];
@@ -345,7 +351,6 @@ else
    Debug("Nothing to do for %s\n",methodname);
    return;
    }
- 
 
 options[0] = '\0';
 
@@ -479,7 +484,7 @@ return;
 int ParentLoadReplyPackage(char *methodname)
 
 { char client[CF_MAXVARSIZE],server[CF_BUFSIZE],name[CF_BUFSIZE],buf[CF_BUFSIZE];
-  char line[CF_BUFSIZE],type[64],arg[CF_BUFSIZE], **methodargv = NULL;
+  char line[CF_BUFSIZE],type[CF_SMALLBUF],arg[CF_BUFSIZE], **methodargv = NULL;
   char basepackage[CF_BUFSIZE],digeststring[CF_BUFSIZE],extra[CF_MAXVARSIZE];
   char *sp;
   struct Item *ip;
@@ -540,6 +545,7 @@ snprintf(buf,CF_BUFSIZE,"rpc_in_%s_%s",methodname,digeststring);
 
 if (IsDefinedClass(CanonifyName(buf)))
    {
+   free((char *)methodargv);
    return false;
    }
 else
@@ -621,6 +627,7 @@ if ((fp = fopen(basepackage,"r")) == NULL)
                  {
                  snprintf(OUTPUT,CF_BUFSIZE,"could not open argument bundle %s\n",arg);
                  CfLog(cferror,OUTPUT,"fopen");
+                 free((char *)methodargv);
                  return false;
                  }
               
@@ -628,6 +635,7 @@ if ((fp = fopen(basepackage,"r")) == NULL)
                  {
                  snprintf(OUTPUT,CF_BUFSIZE,"Could not write to local parameter file %s\n",methodargv[argnum]);
                  CfLog(cferror,OUTPUT,"fopen");
+                 free((char *)methodargv);
                  fclose(fin);
                  return false;
                  }
@@ -717,7 +725,7 @@ return true;
 
 int ChildLoadMethodPackage(char *name,char *digeststring)
 
-{ char line[CF_BUFSIZE],type[64],arg[CF_BUFSIZE];
+{ char line[CF_BUFSIZE],type[CF_SMALLBUF],arg[CF_BUFSIZE];
   char basepackage[CF_BUFSIZE],cffilename[CF_BUFSIZE],buf[CF_BUFSIZE],ebuff[CF_EXPANDSIZE];
   char clientip[CF_BUFSIZE],ipaddress[CF_BUFSIZE];
   struct stat statbuf;
@@ -745,8 +753,7 @@ else
    strncpy(ipaddress,"localhost",63);     
    }
 
-
-snprintf(basepackage,CF_BUFSIZE-1,"%s/rpc_in/%s_%s_%s_%s",VLOCKDIR,clientip,ipaddress,METHODNAME,METHODMD5);
+snprintf(basepackage,CF_BUFSIZE-1,"%s/rpc_in/%s+%s+%s+%s",VLOCKDIR,clientip,ipaddress,METHODNAME,METHODMD5);
 
 argnum = CountAttachments(basepackage);
  
@@ -965,13 +972,13 @@ return true;
 void DispatchMethodReply()
 
 { char label[CF_BUFSIZE]; 
-  char ipaddress[64],clientip[64];
+  char ipaddress[CF_SMALLBUF],clientip[CF_SMALLBUF];
 
 Verbose("DispatchMethodReply()\n\n");
 
 if (strcmp(METHODREPLYTO,"localhost") == 0 || strcmp(METHODREPLYTO,"::1") == 0 || strcmp(METHODREPLYTO,"127.0.0.1") == 0)
    {    
-   snprintf(label,CF_BUFSIZE-1,"%s/rpc_in/localhost_localhost_%s:Reply_%s",VLOCKDIR,METHODNAME,METHODMD5);
+   snprintf(label,CF_BUFSIZE-1,"%s/rpc_in/localhost+localhost+%s:Reply+%s",VLOCKDIR,METHODNAME,METHODMD5);
    EncapsulateReply(label);
    }
 else
@@ -981,7 +988,7 @@ else
 
    Verbose("\nDispatch method %s:%s to %s/rpc_out\n",METHODREPLYTO,METHODNAME,VLOCKDIR);
 
-   snprintf(label,CF_BUFSIZE-1,"%s/rpc_out/%s_%s_%s:Reply_%s",VLOCKDIR,clientip,ipaddress,METHODNAME,METHODMD5);
+   snprintf(label,CF_BUFSIZE-1,"%s/rpc_out/%s+%s+%s:Reply+%s",VLOCKDIR,clientip,ipaddress,METHODNAME,METHODMD5);
    EncapsulateReply(label);
    }
 }
@@ -991,7 +998,7 @@ else
 
 char *GetMethodFilename(struct Method *ptr)
 
-{ char line[CF_BUFSIZE],type[64],arg[CF_BUFSIZE];
+{ char line[CF_BUFSIZE],type[CF_SMALLBUF],arg[CF_BUFSIZE];
   char basepackage[CF_BUFSIZE],cffilename[CF_BUFSIZE],ebuff[CF_EXPANDSIZE];
   static char returnfile[CF_BUFSIZE];
   FILE *fp;
@@ -1053,7 +1060,7 @@ if ((fp = fopen(basepackage,"r")) == NULL)
                  snprintf(OUTPUT,CF_BUFSIZE,"SECURITY ALERT. Method (%s) source is a symbolic link",ptr->name);
                  CfLog(cferror,OUTPUT,"");
                  fclose(fp);
-                 continue;
+                 return returnfile;
                  }
               }
            
@@ -1062,7 +1069,7 @@ if ((fp = fopen(basepackage,"r")) == NULL)
               snprintf(OUTPUT,CF_BUFSIZE,"Method name %s did not find package name %s",ptr->name,cffilename);
               CfLog(cfinform,OUTPUT,"stat");
               fclose(fp);
-              continue;
+              return returnfile;
               }
            
            strncpy(returnfile,cffilename,CF_MAXVARSIZE);
@@ -1328,11 +1335,11 @@ if ((fp = fopen(name,"w")) == NULL)
        {
        char correction[CF_BUFSIZE];
        snprintf(correction,CF_BUFSIZE-1,"%s/rpc_in/%s",VLOCKDIR,name+strlen(VLOCKDIR)+2+strlen("rpc_out"));
-       fprintf(fp,"%s %s_%d\n",VMETHODPROTO[cfmeth_attacharg],correction,i);
+       fprintf(fp,"%s %s+%d\n",VMETHODPROTO[cfmeth_attacharg],correction,i);
        }
     else
        {
-       fprintf(fp,"%s %s_%d\n",VMETHODPROTO[cfmeth_attacharg],name,i);
+       fprintf(fp,"%s %s+%d\n",VMETHODPROTO[cfmeth_attacharg],name,i);
        }
     }
 
@@ -1349,7 +1356,7 @@ if ((fp = fopen(name,"w")) == NULL)
  for (ip = ptr->send_args; ip != NULL; ip=ip->next)
     {
     i++;    
-    snprintf(filename,CF_BUFSIZE-1,"%s_%d",name,i);
+    snprintf(filename,CF_BUFSIZE-1,"%s+%d",name,i);
     Debug("Create arg_file %s\n",filename);
     
     if (IsBuiltinFunction(ip->name))
@@ -1519,11 +1526,11 @@ if ((fp = fopen(name,"w")) == NULL)
        {
        char correction[CF_BUFSIZE];
        snprintf(correction,CF_BUFSIZE-1,"%s/rpc_in/%s",VLOCKDIR,name+strlen(VLOCKDIR)+2+strlen("rpc_out"));
-       fprintf(fp,"%s %s_%d (%s)\n",VMETHODPROTO[cfmeth_attacharg],correction,i,expbuf);
+       fprintf(fp,"%s %s+%d (%s)\n",VMETHODPROTO[cfmeth_attacharg],correction,i,expbuf);
        }
     else
        {
-       fprintf(fp,"%s %s_%d (%s)\n",VMETHODPROTO[cfmeth_attacharg],name,i,expbuf);
+       fprintf(fp,"%s %s+%d (%s)\n",VMETHODPROTO[cfmeth_attacharg],name,i,expbuf);
        }
     }
 
@@ -1541,7 +1548,7 @@ if ((fp = fopen(name,"w")) == NULL)
  for (ip = METHODRETURNVARS; ip != NULL; ip=ip->next)
     {
     i++;    
-    snprintf(filename,CF_BUFSIZE-1,"%s_%d",name,i);
+    snprintf(filename,CF_BUFSIZE-1,"%s+%d",name,i);
     Debug("Create arg_file %s\n",filename);
     
     if (IsBuiltinFunction(ip->name))
@@ -1664,5 +1671,11 @@ server[0] = '\0';
 digeststring[0] = '\0';
 extra[0] = '\0';
  
-sscanf(name,"%1024[^_]_%1024[^_]_%1024[^_]_%[^_]_%64s",client,server,methodname,digeststring,extra);
+sscanf(name,"%1024[^+]+%1024[^+]+%1024[^+]+%[^+]+%64s",client,server,methodname,digeststring,extra);
+
+Debug("Method client -> %s\n",client);
+Debug("Method server -> %s\n",server);
+Debug("Method name -> %s\n",methodname);
+Debug("Method digeststring -> %s\n",digeststring);
+Debug("Method extra -> %s\n",extra);
 }
