@@ -43,6 +43,8 @@ int LoadProcessTable(struct Item **procdata,char *psopts)
 
 { FILE *pp;
   char pscomm[CF_MAXLINKSIZE], imgbackup;
+  struct Item *rootprocs = NULL;
+  struct Item *otherprocs = NULL;
 
 snprintf(pscomm,CF_MAXLINKSIZE,"%s %s",VPSCOMM[VSYSTEMHARDCLASS],psopts);
 
@@ -64,12 +66,37 @@ while (!feof(pp))
 
 cfpclose(pp);
 
-snprintf(VBUFF,CF_MAXVARSIZE,"%s/state/cf_procs",WORKDIR);
+/* Now save the data */
 
 imgbackup = IMAGEBACKUP;
 IMAGEBACKUP = 'n'; 
+
+snprintf(VBUFF,CF_MAXVARSIZE,"%s/state/cf_procs",CFWORKDIR);
 SaveItemList(*procdata,VBUFF,"none");
-IMAGEBACKUP = imgbackup; 
+
+CopyList(&rootprocs,*procdata);
+CopyList(&otherprocs,*procdata);
+
+while (DeleteItemNotMatching(&rootprocs,".*root.*"))
+   {
+   }
+
+while (DeleteItemMatching(&otherprocs,".*root.*"))
+   {
+   }
+
+PrependItem(&rootprocs,otherprocs->name,NULL);
+
+snprintf(VBUFF,CF_MAXVARSIZE,"%s/state/cf_rootprocs",CFWORKDIR);
+SaveItemList(rootprocs,VBUFF,"none");
+DeleteItemList(rootprocs);
+    
+snprintf(VBUFF,CF_MAXVARSIZE,"%s/state/cf_otherprocs",CFWORKDIR);
+SaveItemList(otherprocs,VBUFF,"none");
+DeleteItemList(otherprocs);
+
+IMAGEBACKUP = imgbackup;
+
 return true;
 }
 
@@ -263,7 +290,7 @@ int FindMatches(struct Process *pp,struct Item *procdata,struct Item **killlist)
 
 { struct Item *ip, *ip2;
   char *sp,saveuid[16];
-  int pid=-1, ret, matches=0, got, i;
+  int pid=-1, ret, matches=0, got, i, one_space;
   regex_t rx,rxcache;
   regmatch_t pmatch;
   pid_t cfengine_pid = getpid();
@@ -329,33 +356,35 @@ for (ip = procdata; ip != NULL; ip=ip->next)
          }
       
       Debug("Matched proc[%s]\n",ip->name);
-      
+
       for (sp = ip->name; *sp != '\0'; sp++) /* if first field contains alpha, skip */
          {
          while (true)
             {
-            while (!isdigit((int)*sp) && (*sp != '\0'))
-               {
-               sp++;
-               }
-            
-            if ((sp > ip->name) && isalnum((int)*(sp-1))) /* Username contains number*/
-               {
-               sp++;
-               }
-            else
-               {
-               break;
-               }
-            }
+            /* HvB If start with alpha then skip it till the first space */
+	    if (isalnum((int)*sp))
+	       {
+	       while ((*sp != ' ') && (*sp != '\0'))
+                  {
+                  sp++;
+		  }
+	       }
+
+	       /* skip spaces and tabs */
+               while ((*sp == ' ') && (*sp == '\t'))
+	          {
+		  sp++;
+		  }
+             break;
+             } /* end while */
   
          sscanf(sp,"%d",&pid);
-         
+	
          if (pid != -1)
             {
             break;
             }
-         }  
+         } /* end for */
       
       if (pid == -1)
          {
