@@ -150,7 +150,7 @@ void ListDefinedInterfaces()
  
 for (ifp = VIFLIST; ifp !=NULL; ifp=ifp->next)
    {
-   printf("Interface %s, netmask=%s, broadcast=%s\n",ifp->ifdev,ifp->netmask,ifp->broadcast);
+   printf("Interface %s, ipv4=%s, netmask=%s, broadcast=%s\n",ifp->ifdev,ifp->ipaddress,ifp->netmask,ifp->broadcast);
    }
 }
 
@@ -185,7 +185,7 @@ for (ptr = VBINSERVERS; ptr != NULL; ptr=ptr->next)
    
    if (ptr->classes)
       {
-      printf("(if defined %s), ",ptr->classes);
+      printf("(pred::%s), ",ptr->classes);
       }
    }
 
@@ -203,7 +203,7 @@ printf ("\nDEFINED LINKS\n\n");
 
 for (ptr = VLINK; ptr != NULL; ptr=ptr->next)
    {
-   printf("\nFrom %s -> %s force=%c, attr=%d type=%c nofile=%d\n",ptr->from,ptr->to,ptr->force,ptr->silent,ptr->type, ptr->nofile);
+   printf("\nLINK %s -> %s force=%c, attr=%d type=%c nofile=%d\n",ptr->from,ptr->to,ptr->force,ptr->silent,ptr->type, ptr->nofile);
    for (ip = ptr->copy; ip != NULL; ip = ip->next)
       {
       printf(" Copy %s\n",ip->name);
@@ -252,8 +252,10 @@ printf ("\nDEFINED CHILD LINKS\n\n");
 
 for (ptr = VCHLINK; ptr != NULL; ptr=ptr->next)
    {
-   printf("\n%s->%s force=%c attr=%d, rec=%d\n",ptr->from,ptr->to,
-	  ptr->force,ptr->silent,ptr->recurse);
+   printf("\nCLINK %s->%s force=%c attr=%d, rec=%d\n",ptr->from,ptr->to,
+   ptr->force,ptr->silent,ptr->recurse);
+
+   printf(" IfElapsed=%d, ExpireAfter=%d\n",ptr->ifelapsed,ptr->expireafter);
    
    for (ip = ptr->copy; ip != NULL; ip = ip->next)
       {
@@ -311,7 +313,64 @@ printf ("\nDEFINED ALERTS\n\n");
 
 for (ptr = VALERTS; ptr != NULL; ptr=ptr->next)
    {
-   printf("%s: [%s]\n",ptr->classes,ptr->name);
+   printf("%s: if [%s] ifelapsed %d, expireafter %d\n",ptr->name,ptr->classes,ptr->ifelapsed,ptr->expireafter);
+   }
+}
+
+/*********************************************************************/
+
+void ListDefinedMethods()
+
+{ struct Method *ptr;
+  struct Item *ip;
+  int i;
+
+printf ("\nDEFINED METHODS\n\n");
+
+for (ptr = VMETHODS; ptr != NULL; ptr=ptr->next)
+   {
+   printf("\n METHOD: [%s] if class (%s)\n",ptr->name,ptr->classes);
+   printf("   IfElapsed=%d, ExpireAfter=%d\n",ptr->ifelapsed,ptr->expireafter);
+   printf("   Executable file: %s\n",ptr->file);
+   printf("   Run with Uid=%d,Gid=%d\n",ptr->uid,ptr->gid);
+   printf("   Run in chdir=%s, chroot=%s\n",ptr->chdir,ptr->chroot);
+
+   i = 1;
+   
+   for (ip = ptr->send_args; ip != NULL; ip=ip->next)
+      {
+      printf("   Send arg %d: %s\n",i++,ip->name);
+      }
+
+   printf("   %s\n",ChecksumPrint('m',ptr->digest));
+
+   i = 1;
+   
+   for (ip = ptr->send_classes; ip != NULL; ip=ip->next)
+      {
+      printf("   Send class %d: %s\n",i++,ip->name);
+      }
+
+   i = 1;
+   
+   for (ip = ptr->servers; ip != NULL; ip=ip->next)
+      {
+      printf("   Encrypt for server %d: %s\n",i++,ip->name);
+      }
+
+   i = 1;
+   
+   for (ip = ptr->return_vars; ip != NULL; ip=ip->next)
+      {
+      printf("   Return value %d: $(%s.%s)\n",i++,ptr->name,ip->name);
+      }
+
+   i = 1;
+   
+   for (ip = ptr->return_classes; ip != NULL; ip=ip->next)
+      {
+      printf("   Return class %d: %s\n",i++,ip->name);
+      }   
    }
 }
 
@@ -325,9 +384,10 @@ printf ("\nDEFINED SHELLCOMMANDS\n\n");
 
 for (ptr = VSCRIPT; ptr != NULL; ptr=ptr->next)
    {
-   printf("\n%s\n timeout=%d\n uid=%d,gid=%d\n",ptr->name,ptr->timeout,ptr->uid,ptr->gid);
+   printf("\nSHELLCOMMAND %s\n timeout=%d\n uid=%d,gid=%d\n",ptr->name,ptr->timeout,ptr->uid,ptr->gid);
    printf(" umask = %o, background = %c\n",ptr->umask,ptr->fork);
    printf (" ChDir=%s, ChRoot=%s\n",ptr->chdir,ptr->chroot);
+   printf(" IfElapsed=%d, ExpireAfter=%d\n",ptr->ifelapsed,ptr->expireafter);
    
    if (ptr->defines)
       {
@@ -354,141 +414,144 @@ void ListDefinedImages()
   struct UidList *up;
   struct GidList *gp;
   struct Item *iip, *svp;
-  struct sockaddr_in raddr;
   
 printf ("\nDEFINED FILE IMAGES\n\n");
 
-for (svp = VSERVERLIST; svp != NULL; svp=svp->next) /* order servers */
-   {
-   for (ptr = VIMAGE; ptr != NULL; ptr=ptr->next)
-      {
-      if (strcmp(svp->name,ptr->server) != 0)  /* group together similar hosts so */
-	 {                                     /* can can do multiple transactions */
-	 continue;                             /* on one connection */
-	 } 
-      
-      printf("\n%s\n +%o\n -%o\n dest: %s\n action: %s\n",ptr->path,ptr->plus,ptr->minus,
-	     ptr->destination,ptr->action);
-
-      printf(" Size %c %d\n",ptr->comp,ptr->size);
-      
-      if (ptr->recurse == INFINITERECURSE)
-	 {
-	 printf(" recurse=inf\n");
-	 }
-      else
-	 {
-	 printf(" recurse=%d\n",ptr->recurse);
-	 }
-      
-      printf(" uids = ( ");
-      
-      for (up = ptr->uid; up != NULL; up=up->next)
-	 {
-	 printf("%d ",up->uid);
-	 }
-      
-      printf(")\n gids = ( ");
-      
-      for (gp = ptr->gid; gp != NULL; gp=gp->next)
-	 {
-	 printf("%d ",gp->gid);
-	 }
-      
-      printf(")\n filters:");
-      
-      for (iip = ptr->filters; iip != NULL; iip=iip->next)
-	 {
-	 printf(" %s",iip->name);
-	 }
-      
-      printf("\n exclude:");
-      
-      for (iip = ptr->acl_aliases; iip != NULL; iip=iip->next)
-	 {
-	 printf(" ACL object %s\n",iip->name);
-	 }
-      
-      printf("\n ignore:");
-      
-      for (iip = ptr->ignores; iip != NULL; iip = iip->next)
-	 {
-	 printf(" %s",iip->name);
-	 }
-      
-      printf("\n");
-      printf(" symlink:");
-      
-      for (iip = ptr->symlink; iip != NULL; iip = iip->next)
-	 {
-	 printf(" %s",iip->name);
-	 }
-      
-      printf("\n include:");
-      
-      for (iip = ptr->inclusions; iip != NULL; iip = iip->next)
-	 {
-	 printf(" %s",iip->name);
-	 }
-      printf("\n");
-      
-      printf(" classes = %s\n",ptr->classes);
-      
-      printf(" method = %c (time/checksum)\n",ptr->type);
-      
-      printf(" server = %s (encrypt=%c,verified=%c)\n",ptr->server,ptr->encrypt,ptr->verify);
-      printf(" accept the server's public key on trust? %c\n",ptr->trustkey);
-      
-      printf(" purging = %c\n",ptr->purge);
-      
-      if (ptr->dns != NULL)
-	 {
-	 raddr.sin_addr.s_addr = (ptr->dns)->s_addr;
-	 printf(" host entry cache = %d = %s\n",ptr->dns,inet_ntoa(raddr.sin_addr));
-	 }
-      
-      if (ptr->defines)
-	 {
-	 printf(" Define %s\n",ptr->defines);
-	 }
-      
-      if (ptr->elsedef)
-	 {
-	 printf(" ElseDefine %s\n",ptr->elsedef);
-	 }
-
-      if (ptr->failover)
-	 {
-	 printf(" FailoverClasses %s\n",ptr->failover);
-	 }
-
-      if (ptr->backup == 'n')
-	 {
-	 printf(" NOT BACKED UP\n");
-	 }
-
-      if (ptr->repository)
-	 {
-	 printf(" Local repository = %s\n",ptr->repository);
-	 }
-      
-      if (ptr->stealth == 'y')
-	 {
-	 printf(" Stealth copy\n");
-	 }
-
-      if (ptr->preservetimes == 'y')
-	 {
-	 printf(" File times preserved\n");
-	 }
-
-      if (ptr->forcedirs == 'y')
-	 {
-	 printf(" Forcible movement of obstructing files in recursion\n");
-	 }
-
-      }
-   }
+ for (svp = VSERVERLIST; svp != NULL; svp=svp->next) /* order servers */
+    {
+    for (ptr = VIMAGE; ptr != NULL; ptr=ptr->next)
+       {
+       if (strcmp(svp->name,ptr->server) != 0)  /* group together similar hosts so */
+          {                                     /* can can do multiple transactions */
+          continue;                             /* on one connection */
+          } 
+       
+       printf("\nCOPY %s\n Mode +%o\n     -%o\n TO dest: %s\n action: %s\n",ptr->path,ptr->plus,ptr->minus,
+              ptr->destination,ptr->action);
+       
+       printf(" Size %c %d\n",ptr->comp,ptr->size);
+       printf(" IfElapsed=%d, ExpireAfter=%d\n",ptr->ifelapsed,ptr->expireafter);
+       
+       if (ptr->recurse == CF_INF_RECURSE)
+          {
+          printf(" recurse=inf\n");
+          }
+       else
+          {
+          printf(" recurse=%d\n",ptr->recurse);
+          }
+       
+       printf(" xdev = %c\n",ptr->xdev);
+       
+       printf(" uids = ( ");
+       
+       for (up = ptr->uid; up != NULL; up=up->next)
+          {
+          printf("%d ",up->uid);
+          }
+       
+       printf(")\n gids = ( ");
+       
+       for (gp = ptr->gid; gp != NULL; gp=gp->next)
+          {
+          printf("%d ",gp->gid);
+          }
+       
+       printf(")\n filters:");
+       
+       for (iip = ptr->filters; iip != NULL; iip=iip->next)
+          {
+          printf(" %s",iip->name);
+          }
+       
+       printf("\n exclude:");
+       
+       for (iip = ptr->acl_aliases; iip != NULL; iip=iip->next)
+          {
+          printf(" ACL object %s\n",iip->name);
+          }
+       
+       printf("\n ignore:");
+       
+       for (iip = ptr->ignores; iip != NULL; iip = iip->next)
+          {
+          printf(" %s",iip->name);
+          }
+       
+       printf("\n");
+       printf(" symlink:");
+       
+       for (iip = ptr->symlink; iip != NULL; iip = iip->next)
+          {
+          printf(" %s",iip->name);
+          }
+       
+       printf("\n include:");
+       
+       for (iip = ptr->inclusions; iip != NULL; iip = iip->next)
+          {
+          printf(" %s",iip->name);
+          }
+       printf("\n");
+       
+       printf(" classes = %s\n",ptr->classes);
+       
+       printf(" method = %c (time/checksum)\n",ptr->type);
+       
+       printf(" server = %s (encrypt=%c,verified=%c)\n",ptr->server,ptr->encrypt,ptr->verify);
+       printf(" accept the server's public key on trust? %c\n",ptr->trustkey);
+       
+       printf(" purging = %c\n",ptr->purge);
+       
+       if (ptr->defines)
+          {
+          printf(" Define %s\n",ptr->defines);
+          }
+       
+       if (ptr->elsedef)
+          {
+          printf(" ElseDefine %s\n",ptr->elsedef);
+          }
+       
+       if (ptr->failover)
+          {
+          printf(" FailoverClasses %s\n",ptr->failover);
+          }
+       
+       switch (ptr->backup)
+          {
+          case 'n': printf(" NOT BACKED UP\n");
+              break;
+          case 'y': printf(" Single backup archive\n");
+              break;
+          case 's': printf(" Timestamped backups (full history)\n");
+              break;
+          default: printf (" UNKNOWN BACKUP POLICY!!\n");
+          }
+       
+       
+       if (ptr->repository)
+          {
+          printf(" Local repository = %s\n",ptr->repository);
+          }
+       
+       if (ptr->stealth == 'y')
+          {
+          printf(" Stealth copy\n");
+          }
+       
+       if (ptr->preservetimes == 'y')
+          {
+          printf(" File times preserved\n");
+          }
+       
+       if (ptr->forcedirs == 'y')
+          {
+          printf(" Forcible movement of obstructing files in recursion\n");
+          }
+       
+       }
+    }
 }
 
 /*********************************************************************/
@@ -503,14 +566,17 @@ printf ("\nDEFINED TIDY MASKS\n\n");
 
 for (ptr = VTIDY; ptr != NULL; ptr=ptr->next)
    {
-   if (ptr->maxrecurse == INFINITERECURSE)
+   if (ptr->maxrecurse == CF_INF_RECURSE)
       {
-      printf("%s (maxrecurse = inf)\n",ptr->path);
+      printf("\nTIDY %s (maxrecurse = inf)\n",ptr->path);
       }
    else
       {
-      printf("%s (maxrecurse = %d)\n",ptr->path,ptr->maxrecurse);
+      printf("\nTIDY %s (maxrecurse = %d)\n",ptr->path,ptr->maxrecurse);
       }
+
+   printf(" IfElapsed=%d, ExpireAfter=%d\n",ptr->ifelapsed,ptr->expireafter);
+   printf(" xdev = %c\n",ptr->xdev);
    
    for (ip = ptr->exclusions; ip != NULL; ip = ip->next)
       {
@@ -524,32 +590,33 @@ for (ptr = VTIDY; ptr != NULL; ptr=ptr->next)
       
    for(tp = ptr->tidylist; tp != NULL; tp=tp->next)
       {
+      printf("\n    FOR CLASSES (%s)\n",tp->classes);
       printf("    pat=%s, %c-age=%d, size=%d, linkdirs=%c, rmdirs=%c, travlinks=%c compress=%c\n",
-	     tp->pattern,tp->searchtype,tp->age,tp->size,tp->dirlinks,tp->rmdirs,tp->travlinks,tp->compress);
+             tp->pattern,tp->searchtype,tp->age,tp->size,tp->dirlinks,tp->rmdirs,tp->travlinks,tp->compress);
       
       if (tp->defines)
-	 {
-	 printf("       Define %s\n",tp->defines);
-	 }
+         {
+         printf("       Define %s\n",tp->defines);
+         }
       
       if (tp->elsedef)
-	 {
-	 printf("       ElseDefine %s\n",tp->elsedef);
-	 }
+         {
+         printf("       ElseDefine %s\n",tp->elsedef);
+         }
       
       for (ip = tp->filters; ip != NULL; ip=ip->next)
-	 {
-	 printf(" Filter %s\n",ip->name);
-	 }   
+         {
+         printf(" Filter %s\n",ip->name);
+         }   
       
-      if (tp->recurse == INFINITERECURSE)
-	 {
-	 printf("       recurse=inf\n");
-	 }
+      if (tp->recurse == CF_INF_RECURSE)
+         {
+         printf("       recurse=inf\n");
+         }
       else
-	 {
-	 printf("       recurse=%d\n",tp->recurse);
-	 }
+         {
+         printf("       recurse=%d\n",tp->recurse);
+         }
       }
    }
 }
@@ -568,12 +635,18 @@ for (ptr = VMOUNTABLES; ptr != NULL; ptr=ptr->next)
    printf("%s ",ptr->filesystem);
 
    if ( ptr->readonly )
+      {
       printf(" ro\n");
+      }
    else
+      {
       printf(" rw\n");
+      }
 
    if ( ptr->mountopts != NULL )
+      {
       printf("\t %s\n", ptr->mountopts );
+      }
    }
 }
 
@@ -587,7 +660,8 @@ printf ("\nDEFINED MISC MOUNTABLES\n\n");
 
 for (ptr = VMISCMOUNT; ptr != NULL; ptr=ptr->next)
    {
-   printf("%s on %s (%s)\n",ptr->from,ptr->onto,ptr->options);
+   printf("%s on %s mode = (%s) options (%s)\n",ptr->from,ptr->onto,ptr->mode,ptr->options);
+   printf(" IfElapsed=%d, ExpireAfter=%d\n",ptr->ifelapsed,ptr->expireafter);
    }
 }
 
@@ -602,8 +676,9 @@ printf ("\nDEFINED REQUIRE\n\n");
 for (ptr = VREQUIRED; ptr != NULL; ptr=ptr->next)
    {
    /* HvB : Bas van der Vlies */
-   printf("%s, freespace=%d, force=%c, define=%s\n",
-	ptr->name,ptr->freespace, ptr->force,ptr->define);
+   printf("%s, freespace=%d, force=%c, define=%s\n",ptr->name,ptr->freespace, ptr->force,ptr->define);
+   printf(" IfElapsed=%d, ExpireAfter=%d\n",ptr->ifelapsed,ptr->expireafter);
+   printf(" scanarrivals=%c\n",ptr->scanarrivals);
    }
 }
 
@@ -639,9 +714,16 @@ printf ("\nDEFINED DISABLE\n\n");
 
 for (ptr = VDISABLELIST; ptr != NULL; ptr=ptr->next)
    {
-   printf("\n %s:\n rotate=%d, type=%s, size%c%d action=%c\n",
-	  ptr->name,ptr->rotate,ptr->type,ptr->comp,ptr->size,ptr->action);
-
+   if (strlen(ptr->destination) > 0)
+      {
+      printf("\nRENAME: %s to %s\n",ptr->name,ptr->destination);
+      }
+   else
+      {
+      printf("\nDISABLE %s:\n rotate=%d, type=%s, size%c%d action=%c\n",
+             ptr->name,ptr->rotate,ptr->type,ptr->comp,ptr->size,ptr->action);
+      }
+   printf(" IfElapsed=%d, ExpireAfter=%d\n",ptr->ifelapsed,ptr->expireafter);
    if (ptr->repository)
       {
       printf(" Local repository = %s\n",ptr->repository);
@@ -651,7 +733,7 @@ for (ptr = VDISABLELIST; ptr != NULL; ptr=ptr->next)
       {
       printf(" Define %s\n",ptr->defines);
       }
-
+   
    if (ptr->elsedef)
       {
       printf(" ElseDefine %s\n",ptr->elsedef);
@@ -672,9 +754,10 @@ printf ("\nDEFINED DIRECTORIES\n\n");
 
 for (ptr = VMAKEPATH; ptr != NULL; ptr=ptr->next)
    {
-   printf("%s\n +%o\n -%o\n %s\n",ptr->path,ptr->plus,ptr->minus,FILEACTIONTEXT[ptr->action]);
-
-   if (ptr->recurse == INFINITERECURSE)
+   printf("\nDIRECTORY %s\n +%o\n -%o\n %s\n",ptr->path,ptr->plus,ptr->minus,FILEACTIONTEXT[ptr->action]);
+   printf(" IfElapsed=%d, ExpireAfter=%d\n",ptr->ifelapsed,ptr->expireafter);
+   
+   if (ptr->recurse == CF_INF_RECURSE)
       {
       printf(" recurse=inf\n");
       }
@@ -756,11 +839,13 @@ printf ("\nDEFINED FILES\n\n");
 
 for (ptr = VFILE; ptr != NULL; ptr=ptr->next)
    {
-   printf("\n%s\n +%o\n -%o\n +%o\n -%o\n %s\n travelinks=%c\n",
-	  ptr->path,ptr->plus,ptr->minus,ptr->plus_flags,ptr->minus_flags,
-	  FILEACTIONTEXT[ptr->action],ptr->travlinks);
-   
-   if (ptr->recurse == INFINITERECURSE)
+   printf("\nFILE OBJECT %s\n +%o\n -%o\n +%o\n -%o\n %s\n travelinks=%c\n",
+   ptr->path,ptr->plus,ptr->minus,ptr->plus_flags,ptr->minus_flags,
+   FILEACTIONTEXT[ptr->action],ptr->travlinks);
+
+   printf(" IfElapsed=%d, ExpireAfter=%d\n",ptr->ifelapsed,ptr->expireafter);
+
+   if (ptr->recurse == CF_INF_RECURSE)
       {
       printf(" recurse=inf\n");
       }
@@ -768,6 +853,9 @@ for (ptr = VFILE; ptr != NULL; ptr=ptr->next)
       {
       printf(" recurse=%d\n",ptr->recurse);
       }
+
+   printf(" checksum-type = %c\n",ptr->checksum);
+   printf(" xdev = %c\n",ptr->xdev);
    
    printf(" uids = ( ");
 
@@ -794,7 +882,7 @@ for (ptr = VFILE; ptr != NULL; ptr=ptr->next)
       {
       printf(" Filter %s\n",ip->name);
       }   
-	  
+   
    for (ip = ptr->exclusions; ip != NULL; ip = ip->next)
       {
       printf(" Exclude %s\n",ip->name);
@@ -849,6 +937,7 @@ printf("\nDEFINED UNMOUNTS\n\n");
 for (ptr=VUNMOUNT; ptr!=NULL; ptr=ptr->next)
    {
    printf("%s (classes=%s) deletedir=%c deletefstab=%c force=%c\n",ptr->name,ptr->classes,ptr->deletedir,ptr->deletefstab,ptr->force);
+   printf(" IfElapsed=%d, ExpireAfter=%d\n",ptr->ifelapsed,ptr->expireafter);
    printf(" Context scope: %s\n",ptr->scope);
    }
 }
@@ -874,10 +963,11 @@ for (ptr = VPROCLIST; ptr != NULL; ptr=ptr->next)
       sp = ptr->restart;
       }
    
-   printf("\n%s\n Restart = %s (useshell=%c)\n matches: %c%d\n signal=%s\n action=%c\n",
-	  ptr->expr,sp,ptr->useshell,ptr->comp,ptr->matches,SIGNALS[ptr->signal],ptr->action);
+   printf("\nPROCESS %s\n Restart = %s (useshell=%c)\n matches: (%c)%d\n signal=%s\n action=%c\n",
+   ptr->expr,sp,ptr->useshell,ptr->comp,ptr->matches,SIGNALS[ptr->signal],ptr->action);
 
    printf (" ChDir=%s, ChRoot=%s\n",ptr->chdir,ptr->chroot);
+   printf(" IfElapsed=%d, ExpireAfter=%d\n",ptr->ifelapsed,ptr->expireafter);
    
    if (ptr->defines)
       {
@@ -905,7 +995,7 @@ for (ptr = VPROCLIST; ptr != NULL; ptr=ptr->next)
       }
    }
 
-printf(")\n");
+printf("\n");
 }
 
 /*******************************************************************/
@@ -943,19 +1033,18 @@ void ListDefinedStrategies()
  
 printf("\nDEFINED STRATEGIES\n\n");
 
-for (ptr = VSTRATEGYLIST; ptr != NULL; ptr=ptr->next)
-   {
-   printf("%s (type=%c)\n",ptr->name,ptr->type);
-   if (ptr->strategies)
-      {
-      for (ip = ptr->strategies; ip !=NULL; ip=ip->next)
-	 {
-	 printf("  %s - weight %s\n",ip->name,ip->classes);
-	 }
-      }
-   printf("\n");
-   }
-
+ for (ptr = VSTRATEGYLIST; ptr != NULL; ptr=ptr->next)
+    {
+    printf("%s (type=%c)\n",ptr->name,ptr->type);
+    if (ptr->strategies)
+       {
+       for (ip = ptr->strategies; ip !=NULL; ip=ip->next)
+          {
+          printf("  %s - weight %s\n",ip->name,ip->classes);
+          }
+       }
+    printf("\n");
+    } 
 }
 
 /*******************************************************************/
@@ -969,8 +1058,9 @@ printf("\nDEFINED FILE EDITS\n\n");
 
 for (ptr=VEDITLIST; ptr != NULL; ptr=ptr->next)
    {
-   printf("%s (%c)(r=%d)\n",ptr->fname,ptr->done,ptr->recurse);
+   printf("EDITFILE  %s (%c)(r=%d)\n",ptr->fname,ptr->done,ptr->recurse);
    printf(" Context scope: %s\n",ptr->scope);
+   printf(" IfElapsed=%d, ExpireAfter=%d\n",ptr->ifelapsed,ptr->expireafter);
    
    if (ptr->repository)
       {
@@ -981,11 +1071,11 @@ for (ptr=VEDITLIST; ptr != NULL; ptr=ptr->next)
       {
       if (ep->data == NULL)
          {
-         printf(" %s [nodata]\n",VEDITNAMES[ep->code]);
+         printf(" %s [nodata] if (%s)\n",VEDITNAMES[ep->code],ep->classes);
          }
       else
          {
-         printf(" %s [%s]\n",VEDITNAMES[ep->code],ep->data);
+         printf(" %s [%s] if (%s)\n",VEDITNAMES[ep->code],ep->data,ep->classes);
          }
       }
    printf("\n");
@@ -1014,13 +1104,64 @@ for (ptr=VFILTERLIST; ptr != NULL; ptr=ptr->next)
       {
       printf(" ElseDefines: %s\n",ptr->elsedef);
       }
-
+   
    for (i = 0; i < NoFilter; i++)
       {
       if (ptr->criteria[i] != NULL)
-	 {
-	 printf(" (%s) [%s]\n",VFILTERNAMES[i],ptr->criteria[i]);
-	 }
+         {
+         printf(" (%s) [%s]\n",VFILTERNAMES[i],ptr->criteria[i]);
+         }
       }
    }
 }
+
+/*******************************************************************/
+
+void ListDefinedVariables()
+
+{ struct cfObject *cp = NULL;
+
+ printf("\nDEFINED MACRO/VARIABLES (by contexts)\n");
+ 
+ for (cp = VOBJ; cp != NULL; cp=cp->next)
+    {
+    printf("\nOBJECT: %s\n",cp->scope);
+    PrintHashTable(cp->hashtable);
+    }
+}
+
+/*******************************************************************/
+
+void ListDefinedPackages()
+
+{ struct Package *ptr = NULL;
+
+ printf("\nDEFINED PACKAGE CHECKS\n\n");
+
+ for (ptr = VPKG; ptr != NULL; ptr = ptr->next)
+    {
+    printf("PACKAGE NAME: %s\n", ptr->name);
+    printf(" Package database: %s\n", PKGMGRTEXT[ptr->pkgmgr]);
+    printf(" IfElapsed=%d, ExpireAfter=%d\n",ptr->ifelapsed,ptr->expireafter);
+    
+    if (ptr->ver && *(ptr->ver) != '\0')
+       {
+       printf(" Version is %s %s\n",
+                CMPSENSETEXT[ptr->cmp], ptr->ver);
+       }
+    else
+       {
+       printf(" Matches any package version.\n");
+       }
+    if (ptr->defines)
+       {
+       printf(" Define %s\n",ptr->defines);
+       }
+    
+    if (ptr->elsedef)
+       {
+       printf(" ElseDefine %s\n",ptr->elsedef);
+       }
+    }
+ printf("\n");
+}
